<?php

namespace PlaNext\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;

class PedidoAlteracaoCardapioController {
    
    protected $crudService;
    protected $utilities;
    
    public function __construct($connection, $environment, $service, $crudService, $Utilities) {
    $this->connection   = $connection;
    $this->environment  = $environment;
    $this->service      = $service;
    $this->crudService  = $crudService;
    $this->utilities = $Utilities;
  }
    public function insereNovoPedido(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getParameters();
            $row['NRORG'] = $this->environment->getNrOrg();
            $row['CDOPERADOR'] = $this->environment->getCdOperador();
            
            $row['NRPEDALTCARD'] = $this->service->retornaUltimoNRPEDALT($row['NRORG']);
            $valorDataMes = $row['DTMESMODAL'];
            list($mes, $ano) = explode('-', $valorDataMes);
            $dataFormatada = '01-'. $mes. '-'. $ano;
            $row['DTMESMODAL'] = $dataFormatada;
            
            $row['CDPEDALTCARD'] = str_pad($row['NRPEDALTCARD'], 10, 0, STR_PAD_LEFT);
            
            $this->service->insertNovoPedido($row);
            $response->addDataSet(new DataSet("valoresInseridos", $row));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function deletaPedido(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $parameters = $request->getParameters();
            $row = $parameters['rowDelete'];
            $this->service->deletePedidoService($row);
            $response->addDataSet(new DataSet("valoresDeletados", $row));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function atualizaPedido(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $parameters = $request->getParameters();
            $valoresUpdate = $parameters['valoresUpdate'];
            $row = $valoresUpdate['row'];
            $row['DSJUSTICATIVA'] = $valoresUpdate['justificativa'];
            $this->service->atualizaPedidoService($row);
            $rowsAtualizadas = array(
                $row['DSJUSTICATIVA'],
                $row['NRPEDALTCARD'],
                $row['CDPEDALTCARD'],
                $row['CDFILIAL']
            );
            $response->addDataSet(new DataSet("valoresAtualizados", $rowsAtualizadas));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function carregaDadosCardapio(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $parameters = $request->getParameters();
            $row = $parameters['currentRow'];
            $dataColumns = $this->service->carregaColunasCardapioService($row);
            $response->addDataSet(new DataSet('ColunasCardapio', $dataColumns));
            
            $dataPratos = $this->service->retornaPratosCardapio($row);
            $response->addDataSet(new DataSet('PratosCardapio', $dataPratos));
            
            $dataComensais = $this->service->retornaComensaisCardapio($row);
            $response->addDataSet(new DataSet('ComensaisCardapio', $dataComensais));
            
            $changedDiners = $this->service->returnsChangedDiners($row);
            $changedDishes = $this->service->returnsChangedDishes($row);
            $response->addDataSet(new DataSet('ChangedDiners', $changedDiners));
            $response->addDataSet(new DataSet('ChangedDishes', $changedDishes));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaExisteCardapio(Request\Row $request, Response $response) {
        try{
            $row = $request->getParameters();
            $dataVerifica = $this->service->verificaExisteCardapio($row);
            $response->addDataSet(new DataSet("verificaExisteCardapio", $dataVerifica));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaTurnosPontoEntrega(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getParameters()['currentRowGrid'];
            $dataTurnos = $this->service->verificaTurnosCardaDia($row);
            $response->addDataSet(new DataSet("turnosCardaDia", $dataTurnos));
            $dataPontosEntrega = $this->service->verificaPontoEntrega($row);
            if($dataPontosEntrega === []) {
                $dataPontosEntrega = $this->service->retornaCenario($row);
            }
            $response->addDataSet(new DataSet("pontosDeEntrega", $dataPontosEntrega));
        }catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function alteraComensaisTurno(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getParameters();
            $row['NRORG'] = $this->environment->getNrOrg();
            $row['CDOPERADOR'] = $this->environment->getCdOperador();
            $nrCardapio = $this->service->retornaNrCardapio($row['valoresUpdate']['row']);
            if (!empty($nrCardapio)) {
                $row['NRCARDAPIO'] = $nrCardapio[0]['NRCARDAPIO'];
            } else {
                $row['NRCARDAPIO'] = null;
            }
            $changed = $this->service->changedDinerDtMenu($row);
            if(count($changed) > 0){
                $row['NRPEDALTDIA'] = $changed[0]['NRPEDALTCARDDIA'];
                $this->service->alteraComensaisTurnoService($row);
            } else {
                $row['NRPEDALTCARDDIA'] = $this->service->retornaNRPEDALTCARDDIA($row['NRORG']);
                $this->service->insereComensaisTurnoService($row);
            }
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function changedDish(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getParameters();
            $row['NRORG'] = $this->environment->getNrOrg();
            $row['CDOPERADOR'] = $this->environment->getCdOperador();
            $nrCardapio = $this->service->retornaNrCardapio($row['valoresUpdate']['row']);
            $dish = $row['valoresUpdate']['dish']['selectPratoModal'];
            if (!empty($nrCardapio)) {
                $row['NRCARDAPIO'] = $nrCardapio[0]['NRCARDAPIO'];
            } else {
                $row['NRCARDAPIO'] = null;
            }
            $params = [
                'CDFILIAL' => $row['valoresUpdate']['row']['CDFILIAL'],
                'CDSERVFILI' => $row['valoresUpdate']['row']['CDSERVFILI'],
                'CDFILAUXPRAT' => $dish['CDFILAUXPRAT'],
                'CDPRATO' => $dish['CDPRATO'],
                'CDNVPRATO' => $row['valoresUpdate']['CDNVPRATO']
            ];
            $dishInfo = $this->utilities->getDatasourceFirstRow('dishesWithCost', $params);
            if(isset($dishInfo)) {
                if($dishInfo['IDATIVO'] !== 'S')
                    throw new \Exception('plaNext::pedAltPratRecInat');
                if(!isset($dishInfo['TIPORECEITA']))
                    throw new \Exception('plaNext::pedAltPratSemRec');
                $allowDefaultRec = $this->utilities->getFirstRowValue('generalPlaParams', 'IDUTRPADCA');
                if($allowDefaultRec === 'N' && $dishInfo['TIPORECEITA'] === 'P') {
                    throw new \Exception('plaNext::pedAltPratComRecPad');
                }
            }
            $changed = $this->service->changedDishDtMenu($row);
            if(count($changed) > 0){
                $this->service->updateChangedDish($row);
            } else {
                $row['NRPEDALTCARDCOMP'] = $this->service->returnNRPEDALTCARDCOMP($this->environment->getNrOrg());
                $this->service->insertChangedDish($row);
            }
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function checksWithdrawalsEffectives($row) {
        $withdrawalMenu = $this->service->withdrawalMenu($row);
        if($withdrawalMenu[0]['RETIRADAS'] != 0) {
            throw new \Exception('plaNext::errorWithdrawals');
        }
        $effectivesMenu = $this->service->effectivesMenu($row);
        if($effectivesMenu[0]['EFETIVOS'] != 0) {
            throw new \Exception('plaNext::errorEffectives');
        }
    }
    
    public function completeOrder(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $this->connection->beginTransaction();
        $operation = 'conc';
        try{
            $changedDiners = $this->service->returnsChangedDiners($row['CURRENTROW']);
            if(count($changedDiners) === 0) {
                $changedDishes = $this->service->returnsChangedDishes($row['CURRENTROW']);
                if(count($changedDishes) === 0) {
                    throw new \Exception('plaNext::noChangedMenu');
                }
            }
            $this->checksWithdrawalsEffectives($row['CURRENTROW']);
            $operator = $this->environment->getCdOperador();
            if($row['EXTRAREQUEST'] === "S" || ($row['EXTRAREQUEST'] === "N" && count($changedDiners) != 0)) {
                if($row['EXTRAREQUEST'] === "S") {
                  $operation = 'concNeedOM';
                }
                $this->service->completeOrder($row['CURRENTROW'], $row['EXTRAREQUEST'], $operator);
            } else if ($row['EXTRAREQUEST'] === "N" && count($changedDiners) === 0) {
                $operation = 'concAprovAuto';
                $this->service->approve($row, $operator);
            }
            $response->addDataSet(new DataSet('completeOrder', ['operation' => $operation]));
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function requestAgain(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getRow();
            $operator = $this->environment->getCdOperador();
            $this->service->requestAgain($row, $operator);
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function approveDisapprove(Request\Row $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $row = $request->getRow();
            $operator = $this->environment->getCdOperador();
            if ($row['APPROVALPAGE']) {
                if($row['APPROVE'] === 'A') {
                    $this->service->approve($row, $operator);
                } else {
                    $this->service->disapprove($row, $operator);
                }
            } else {
                if($row['APPROVE'] === 'A') {
                    $this->service->approveGO($row, $operator);
                } else {
                    $this->service->disapproveGO($row, $operator);
                }
            }
            $this->connection->commit();
        }catch(\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function isOperatorGO(Request\Row $request, Response $response){
        $cdfilial = $request->getRow()['CDFILIAL'];
        $G_O = $this->utilities->getFirstRowValue('filialGO', 'CDGOPERAC', ['CDFILIAL' => $cdfilial]);
        $isGO = ( $this->environment->getCdOperador() == $G_O );
        $response->addDataSet(new DataSet('isOperatorGO', ['isGO' => $isGO]));
    }
    
    public function setAsViewedCompleted(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->service->setAsViewedCompleted($row['NRPEDALTCARD']);
    }
    
    public function setAsViewedApDisap(Request\Row $request, Response $response){
        $row = $request->getRow();
        $this->service->setAsViewedApDisap($row['NRPEDALTCARD']);
    }
    
    public function getEmailPlan(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $emailPlan = $this->utilities->getDatasourceFirstRow('emailPedAltCard', ['CDFILIAL' => $row['CDFILIAL']]);
        if($emailPlan)
            $response->addDataSet(new DataSet('emailPlan', $emailPlan));
    }
    
    public function setEmailPlan(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $cdfilial = $row['CDFILIAL'];
        $dsemail = $row['DSEMAIL'];
        $emailPlan = $this->utilities->getDatasourceFirstRow('emailPedAltCard', ['CDFILIAL' => $cdfilial]);
        $isNew = !isset($emailPlan);
        $this->service->setEmailPlan($cdfilial, $dsemail, $isNew);
    }
    
    public function getDatesToTranslate(Request\Row $request, Response $response){
        $row = $request->getRow();
        $dates = $this->utilities->getDatasourceFirstRow('pedAltCardDates', ['NRPEDALTCARD' => $row['NRPEDALTCARD']]);
        if($dates)
            $response->addDataSet(new DataSet('dates', $dates));
    }
    
    private function getSubjectByOperation($operation, $msgs) {
        if (isset($msgs[$operation])) {
            return $msgs[$operation];
        }
        return '';
    }
    
    private function getReceiverByOperation($operation, $emailInfos) {
        $to = [];
        $emailPlan = $this->utilities->getFirstRowValue('emailPedAltCard', 'DSEMAIL', ['CDFILIAL' => $emailInfos['CDFILIAL']]);
        $emails = [];
        if(!empty($emailPlan)) {
            $emailPlan = str_replace(',', ';', $emailPlan);
            $emailsTmp = explode(';', $emailPlan);
            foreach($emailsTmp as $tmp) {
                $email = trim($tmp);
                if (!empty($email)) {
                    $emails[] = $email;
                }
            }
        }
        $emailGO = $this->utilities->getFirstRowValue('filialGO', 'DSEMAILOPERGO', ['CDFILIAL' => $emailInfos['CDFILIAL']]);
        if($operation == 'conc' ||
           $operation == 'concAprovAuto' ||
           $operation == "concNeedOM" ||
           $operation == 'aprovOM') {
            foreach($emails as $em) {
                $to[] = $em;
            }
            if ($operation == "concNeedOM" && !empty($emailGO)) {
                $to[] = $emailGO;
            }
        } else if ($operation == "aprov" || $operation == "reprov") {
            $to[] = $emailInfos['DSEMAILOPER'];
            if(!empty($emailInfos['OPERGO']) && !empty($emailGO)) {
                $to[] = $emailGO;
            }
        }
        return $to;
    }
    
    private function getEmailMessage($operation, $subject, $emailInfos, $msgs, $dates){
        $classTitle = ' ';
        if ($operation == 'aprov' || $operation == 'concAprovAuto') {
            $classTitle = 'approved';
        } else if ($operation == 'reprov') {
            $classTitle = 'reproved';
        }
        $params = [
            'title' => $subject,
            'requestLabel'  => $msgs['requestLabel'] . ': ',
            'request' => $emailInfos['CDPEDALTCARD'],
            'creationLabel'  => $msgs['creationLabel'] . ': ',
            'creation' => $dates['DTPEDALTCARD']  . ' ' . $emailInfos['NMOPERADOR'],
            'unitLabel'  => $msgs['unitLabel'] . ': ',
            'unit' => $emailInfos['CDFILIAL']   . ' ' . $emailInfos['NMFILIAL'],
            'serviceLabel'  => $msgs['serviceLabel'] . ': ',
            'service' => $emailInfos['CDSERVFILI']   . ' ' . $emailInfos['NMSERVICO'],
            'monthLabel'  => $msgs['monthLabel'] . ': ',
            'month' => $dates['DTMESCARD'],
            'approveLabel'  => $msgs['approveLabel'] . ': ', 
            'approve'  => $dates['DTAPROVREPR'] . ' ' . $emailInfos['OPERAPROV'],
            'approveGOLabel' => $msgs['approveGOLabel'] . ': ',
            'approveGO'  => $dates['DTAPROVREPRGO'] . ' ' . $emailInfos['OPERGO'],
            'justifyLabel' => $msgs['justifyLabel'] . ': ',
            'justify' => ( !empty($emailInfos['DSJUSTICATIVA']) ) ? $emailInfos['DSJUSTICATIVA'] : ' ',
            'responseLabel' => $msgs['responseLabel'] . ': ',
            'response' => ( !empty($emailInfos['DSRESPOSTA']) ) ? $emailInfos['DSRESPOSTA'] : ' ',
            'sendByTecfood' => $msgs['sendByTecfood'],
            'approvreprov' => $classTitle
        ];
        return $this->utilities->getEmailMessageFromTemplate('pedAltCard', $params);
    }

    public function sendEmailPedAltCard(Request\Row $request, Response $response){
        $row = $request->getRow();
        $nrPedAltCard = $row['NRPEDALTCARD'];
        $emailInfos = $this->utilities->getDatasourceFirstRow('pedAltCardEmailInfos', ['NRPEDALTCARD' => $nrPedAltCard]);
        $operation = $row['OPERATION'];
        $dates = $row['dates'];
        $msgs = $row['msgs'];
        if($emailInfos) {
            $cdfilial = $emailInfos['CDFILIAL'];
            $subject = $this->getSubjectByOperation($operation, $row['msgs']);
            $to = $this->getReceiverByOperation($operation, $emailInfos);
            $message = $this->getEmailMessage($operation, $subject, $emailInfos, $msgs, $dates);
            $this->utilities->sendMail($cdfilial, $to, $subject, $message);
        }
    }
    
    public function returnResumeChanges(Request\Row $request, Response $response){
        $this->connection->beginTransaction();
        try{
            $parameters = $request->getParameters();
            $row = $parameters['currentRow'];
            $changedDiners = $this->service->returnsChangedDiners($row);
            $changedDishes = $this->service->returnsChangedDishes($row);
            $dataDishes = $this->service->retornaPratosCardapio($row);
            $dataOldDishes = array();
            foreach ($dataDishes as $dish){
                foreach ($changedDishes as $changedDish){
                    $dateTime = $changedDish['DTCARDAPIO'];
                    $dateChangedDish = explode(" ", $dateTime)[0];
                    if($dateChangedDish === $dish['DTCARDAPIO'] && $changedDish['NRSEQPRATCAR'] === $dish['NRSEQPRATCAR']) {
                        $dataOldDishes[] = $dish;
                    }
                }
            }
            $dataDiners = $this->service->retornaComensaisCardapio($row);
            $dataOldDiners = array();
            foreach ($dataDiners as $diner){
                foreach ($changedDiners as $changedDiner){
                    $dateTime = $changedDiner['DTCARDAPIO'];
                    $dateChangedDiner = explode(" ", $dateTime)[0];
                    if($dateChangedDiner === $diner['DTCARDAPIO'] && $changedDiner['NRCARDAPIO'] === $diner['NRCARDAPIO']) {
                        $dataOldDiners[] = $diner;
                    }
                }
            }
            $response->addDataSet(new DataSet('OldDishes', $dataOldDishes));
            $response->addDataSet(new DataSet('OldDiners', $dataOldDiners));
            $response->addDataSet(new DataSet('ChangedDiners', $changedDiners));
            $response->addDataSet(new DataSet('ChangedDishes', $changedDishes));
        }catch(\Exception $e) {
            throw $e;
        }
    }
}