<?php

namespace PlaNext\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\FilterCriteria;

class RetiradaPlanejamentoController {
    const PREVRETI = 'pla02000_prevretiexistente';
    const SERVICO = 'servico';
    
    protected $crudService;
    protected $dataSourceManager;
    protected $dataSourceName = "pla02000_retiradaplanejamento";
    protected $libUtilities;
    protected $parameterBagColumns = array(
        "NRORG",
        "CDFILIAL",
        "CDSERVFILI",
        "DTRETIRADA",
        "DTCARDAPIO",
        "NRREQUEXTRRE"
    );

  public function __construct($connection, $environment, $service, $crudService, $dataSourceManager, $libUtilities) {
    $this->connection   = $connection;
    $this->environment  = $environment;
    $this->service      = $service;
    $this->crudService    = $crudService;
    $this->dataSourceManager = $dataSourceManager;
    $this->libUtilities=$libUtilities;
  }

  public function getDadosIniciais(Request\Row $request, Response $response) {
    
    $data = array(
          'NRORG' => $this->environment->getNrOrg(),
          'CDOPERADOR' => $this->environment->getCdOperador(),
          "FILIAL_LOGADA" => $this->environment->getCdFilial()
          
    );
    $response->addDataSet(new DataSet("dadosIniciais", $data));
  }
  
  public function carregaParametrosUnidade(Request\Row $request, Response $response) {
    $row = $request->getParameters();
    $data = $this->service->retornaParametrosUnidade($row);
    $response->addDataSet(new DataSet("parametros", $data));
  }
 
  public function excluirRomaneio(Request\Row $request, Response $response) { 
    try {
        $row = $request->getParameters();
        $this->service->excluirRomaneioService($row);
    } catch (\Exception $e) {
        $this->treatError($e, $response);
    }
  }
  
  private function setParamsEstoq(&$row){ 
        $row['CDALMOXARIFE'] = isset($row['CDALMOXARIFE']) && $row['CDALMOXARIFE'] ? $row['CDALMOXARIFE'] : ' '; 
        $row['CDLOCALESTOQ'] = isset($row['CDLOCALESTOQ']) && $row['CDLOCALESTOQ'] ? $row['CDLOCALESTOQ'] : ' '; 
        $row['NRLOTEESTQ']   = isset($row['NRLOTEESTQ'])   && $row['NRLOTEESTQ']   ? $row['NRLOTEESTQ']   : ' '; 
        $row['NRSUBLOTE']    = isset($row['NRSUBLOTE'])    && $row['NRSUBLOTE']    ? $row['NRSUBLOTE']    : ' ';   
    }
  
  public function salvaAlteracaoProduto(Request\Row $request, Response $response, &$errosParaRetornar = null) { 
        $row = $request->getRow();
        try{
            $cdfilimovi = $row['CDFILIAL'];
            $nrlancestq = $row['NRLANCESTQ'];
            
            $error = array();

            $validaEstoque = true;
            if(isset($row['NRSEQUITRETI'])){
                $validaEstoque = $this->service->validaEstoque($row, $error, $row["NRRETIRADA"], $row["NRCARDAPIO"], $row["NRSEQUITRETI"]);
            }else{
                $validaEstoque = $this->service->validaEstoque($row, $error);
            }  

            if(!$validaEstoque) {
                if($errosParaRetornar === null){
                    $response->addDataSet(new DataSet('error', $error));
                    throw new \Exception("INVALID_STOCK");
                }
                else {
                    array_push($errosParaRetornar, $row["CDARVPROD"] . " " . $row["NMPRODUTO"] . "&&" . $error[0]);
                }
            }
            $prodJaCad = $this->service->Vd_ExistProdAlmox($row); 
            if($prodJaCad){
                if($errosParaRetornar === null){
                    throw new \Exception("prodJaCadastradoParaRetiradaAtual");
                }
                else {
                    array_push($errosParaRetornar, $row["CDARVPROD"] . " " . $row["NMPRODUTO"] . "&&" . "prodJaCadastradoParaRetiradaAtual");
                }
    
            }
            
            $cdfilancestq = $this->service->retornaFilialLancamentoRetirada($row['CDFILIAL'], $row['NRCARDAPIO']);
            $itlanctoest = $this->service->retornaItlanctoest($cdfilancestq, $nrlancestq, $row['NRSEQUITRETI']);
    
            $row['CDFILANCESTQ'] = $cdfilancestq;
            if($row['QTRETIRADA'] > 0) {
                if($itlanctoest) {
                    $rowItemLancto = clone $row;
                    $rowItemLancto['CDFILIAL'] = $cdfilancestq;
                    $this->service->alteraItemLancto($rowItemLancto);
                } else {
                    $row['NRSEQUITEM'] = $row['NRSEQUITRETI'];
                    $row['IDTIPOMOVI'] = '3';
                    $row['IDORDEMCLC'] = 9;
                    $row['IDEVOLESTOQ'] = 'N';
                    $this->setParamsEstoq($row);
                    $this->service->insereItemLancto($row, $cdfilancestq);
                }
            } else {
                if($itlanctoest) {
                    $this->service->removeRelacaoItemRetiItlanctoest($row);
                    $this->service->excluiItemLancto($row);
                }
            }
            $this->service->alteraItemreti($row);
            if($errosParaRetornar === null){
                $this->service->atualizarEstoque($cdfilancestq, $nrlancestq);// passava idevolestoq como 'S'
            }
            
        } catch (\Exception $e) {
            if($errosParaRetornar != null){
                throw $e;
            }
            $this->treatError($e, $response);
        }
    }
    
    public function salvaEditLineProduto(Request\DataSet $request, Response $response) {
        $this->connection->beginTransaction();
        try{
            $rows = $request->getDataSet()->getRows();
            //coleta de dados
            $this->libUtilities->saveCollectedData($rows[0]['CDFILIAL'],'RETIRADA');
            $errosParaRetornar = [];
            $cdfilancestq = null;
            $nrlancestq = null;
            foreach ($rows as $row) {
                $row['CDPRODUTO'] = str_replace(".", "", $row['CDPRODUTO']);
                if ($cdfilancestq == null){
                    $cdfilancestq = $row['CDFIITLANEST'];
                    if (empty($cdfilancestq)) {
                        $cdfilancestq = $this->service->retornaFilialLancamentoRetirada($row['CDFILIAL'], $row['NRCARDAPIO']);
                    }
                    $nrlancestq = $row['NRLANCESTQ'];
                }
                $this->salvaAlteracaoProduto(new Request\Row($row, "", "", ""), $response, $errosParaRetornar);
            }

            if(count($errosParaRetornar) > 0){
                $response->addDataSet(new DataSet('produtos', $errosParaRetornar));
                $response->setError(new Response\Error('SALDO_INDISPONIVEL', 1));
                return;
            }
            $this->service->atualizarEstoque($cdfilancestq, $nrlancestq);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $this->treatError($e, $response);
        }
    }
    
    public function excluirTipoPrato(Request\DataSet $request, Response $response) { 
        try {
            $this->connection->beginTransaction();
            $pratos = $request->getDataSet()->getRows();
            $nrlancestq = '';
            $cdfiitlanest = null;
            foreach ($pratos as $prato) {
                $itens = $this->service->retornaItensPorPrato($prato);
                foreach ($itens as $item) {
                    $nrlancestq = $item->getNrlancestq();
                    $cdfiitlanest = $item->getCdfiitlanest();
                    $nrsequitreti = $item->getNrsequitreti();
                    $prato['NRSEQUITRETI'] = $nrsequitreti;
                    $this->service->excluiItemreti($prato);

                    if($nrlancestq) {
                        $rowItem = array(
                            'CDFILANCESTQ' => $item->getCdfiitlanest(),
                            'NRLANCESTQ' => $nrlancestq,
                            'NRSEQUITRETI' => $nrsequitreti
                        );
                        $this->service->excluiItemLancto($rowItem);
                    }
                }
            }
            if($nrlancestq && $cdfiitlanest){
                $this->service->atualizarEstoque($cdfiitlanest, $nrlancestq); 
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $this->treatError($e, $response);
        }
    }
    
    public function excluirProduto(Request\DataSet $request, Response $response) { 
        
        try {
            $this->connection->beginTransaction();
            $itens = $request->getDataSet()->getRows();
            $firstItem = null;
            if(count($itens) > 0){
                $firstItem = $itens[0];
            }
            $cdfilancestq = $this->service->retornaFilialLancamentoRetirada($firstItem['CDFILIAL'], $firstItem['NRCARDAPIO']);
            foreach ($itens as $item) {
                $this->service->excluiItemreti($item);
                if (isset($item['NRLANCESTQ']) && $item['NRLANCESTQ']) {
                    $item['CDFILANCESTQ'] = $cdfilancestq;
                    $this->service->excluiItemLancto($item);
                }
            }
            if($firstItem){
                $lancamento = $this->service->dadosLancamento($firstItem['CDFILIAL'], $firstItem['NRCARDAPIO'], $firstItem['NRRETIRADA']);
                if(!empty($lancamento['CDFILANCESTQ'])){
                    $this->service->atualizarEstoque($lancamento['CDFILANCESTQ'], $lancamento['NRLANCESTQ']);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
    
    public function adicionaRetirada(Request\Row $request, Response $response) { 
        try {
            $row = $request->getParameters();
            $cdservfili = $row['CDSERVFILI'];
            $dtretirada = $row['DTRETIRADA'];
            $cdtpretipadr = $row['CDTPRETIPADR'];
            $cdfilial_logada = $this->environment->getCdfilial();
            $row['NRREQUEXTRRE'] = $row['NRRETIRADA'];
            
             //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
           
            $this->setParamsEstoq($row); 
            $retirada = self::retornaRetirada($row);

            if (!empty($retirada)) {
                throw new \Exception("existeRetiradaCadastrada");
            }

            $validaDataRetirada = $this->validaDataRetirada($row);
            if ($validaDataRetirada) {
                throw new \Exception($validaDataRetirada);
            }

            if (!$cdtpretipadr) {
                throw new \Exception("naoExistTipoRetiParam");
            }
            $row['DTPREVRETI'] = $row['DTCARDAPIO'];
            $prevreti = $this->retornaPrevreti($row);
            $nrcardapio = null;
            $nrretirada = null;
            if($prevreti) {
            	$idretiprat = $prevreti[0]['IDPREVTPPRAT'];
            	$nrcardapio = $this->service->retornaNrcardapio($row);
            	if(!$nrcardapio) {
            		$nrcardapio = $this->service->insereCardapio($row);
            	}
            	$row['IDTPLANCTO'] = 3;
            	$row['DTLANCESTQ'] = $dtretirada;
            	$row['DSLANCESTQ'] = "Retirada para o cardápio: ".$dtretirada.", Servico: ".$cdservfili;
            	$row['NRCARDAPIO'] = $nrcardapio;
                $row['IDUTRETIPRAT'] = $idretiprat;
            	
            	$nrlancto = $this->service->insereLancamento($row);
                $row['NRLANCESTQ'] = $nrlancto;
                
            	$nrretirada = $this->service->insereRetirada($row);

                $index = 0;
            	foreach ($prevreti as $item) {
                    $seq = str_pad($index++, 4, "0", STR_PAD_LEFT);
                    $item['NRLANCESTQ'] = $nrlancto;
                    $item['NRSEQUITEM'] = $seq;
                    $item['QTRETIRADA'] = $item['QTPREVLIB'];
                    $item['IDTIPOMOVI'] = '3';
                    $item['DTRETIRADA'] = $row['DTRETIRADA'];
                    $item['IDORDEMCLC'] = 9;
                    $item['CDFILIAL'] = $row['CDFILIAL'];
                    $item['IDEVOLESTOQ'] = 'N';
                    $item['CDTIPORETI'] = $row['CDTPRETIPADR'];
                    $item['NRRETIRADA'] = $nrretirada;
                    $item['NRCARDAPIO'] = $nrcardapio;
                    $item['IDINTESIST'] = 'N';
                    $item['QTPREVISRETI'] = $item['QTPREVRETI'];
                    $item['QTDEVOLVRETI'] = $item['QTPREVDEVOL'];
                    $item['QTEXTRARETI'] = $item['QTPREVEXTRA'];
                    $this->setParamsEstoq($item);
         			$this->service->insereItemLancto($item, $cdfilial_logada);
            		$this->service->insereItemreti(	$item, $cdfilial_logada );
            	}

            }
            $tipoRetirada = $this->service->retornaIdutretiprat($row);
            $retorno = array(
                "TIPO_RETIRADA" => $tipoRetirada,
                "CDFILIAL" => $row['CDFILIAL'],
                "CDSERVFILI" => $row['CDSERVFILI'],
                "NRCARDAPIO" => $nrcardapio,
                "NRRETIRADA" => $nrretirada,
                "DTRETIRADA" => $row['DTRETIRADA']
            );
            $response->addDataSet(new DataSet("retirada", array($retorno)));
        } catch (\Exception $e) {
           $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    public function retornaRetirada($row) { 
        $nrcardapio = $this->service->retornaNrcardapio($row);
        $retiradaExistente = $this->service->retornaRetiradasExistentes($row['CDFILIAL'], $nrcardapio, $row["NRREQUEXTRRE"]);
        return $retiradaExistente;
    }
    
    public function validaDataRetirada($row) { 
        $mensagem = false;
        $dtprocessa = \DateTime::createFromFormat('d/m/Y H:i:s', $row['DTPROCESSA']."00:00:00");
        $dtretirada = \DateTime::createFromFormat('d/m/Y H:i:s', $row['DTRETIRADA']."00:00:00");
        $idlancdtmen = $row['IDLANCDTMEN'];
        $idlancdtmai = $row['IDLANCDTMAI'];
        $nrdiassaid = $row['NRDIASSAID'] ? $row['NRDIASSAID'] : 0;
        $nrdiassaidpd = $row['NRDIASSAIDPD'] ? $row['NRDIASSAIDPD'] : 0;
        $idlancdtmenfil = $row['IDLANCDTMENFIL'];
        $nrdiassaidfil = $row['NRDIASSAIDFIL'] ? $row['NRDIASSAIDFIL'] : 0;
        $idlancdtmaifil = $row['IDLANCDTMAIFIL'];
        $nrdiassaidpdfil = $row['NRDIASSAIDPDFIL'] ? $row['NRDIASSAIDPDFIL'] : 0;
        $dateProcessa = \DateTime::createFromFormat('d/m/Y', explode(" ",$row['DTPROCESSA'])[0]);
        $dateRetirada = \DateTime::createFromFormat('d/m/Y', $row['DTRETIRADA']);
        $dtprocessaformat = strtotime($dateProcessa->format('Y-m-d'));
        $dtretiradaformat = strtotime($dateRetirada->format('Y-m-d'));
        if ($dtretiradaformat < $dtprocessaformat) {
            if ($idlancdtmenfil == 'N') {
                if ($idlancdtmen == 'N') {
                    $mensagem = "dtMenorDtProcess";
                } else {
                    if( $dtretirada < ($dtprocessa->sub(new \DateInterval('P'.$nrdiassaid.'D')))) {
                        $mensagem = "dtForaPeriodoPermitido";
                    }
                }
            } else {
                if( $dtretirada < ($dtprocessa->sub(new \DateInterval('P'.$nrdiassaidfil.'D')))) {
                    $mensagem = "dtForaPeriodoPermitido";
                }
            }
        } else if ($dtretiradaformat > $dtprocessaformat) {
            if ($idlancdtmaifil == 'S') {
                if( $dtretirada > ($dtprocessa->add(new \DateInterval('P'.$nrdiassaidpdfil.'D')))) {
                    $mensagem = "dtForaPeriodoPermitido";
                }
            } else {
                if ($idlancdtmai == 'S') {
                    if ( $dtretirada > ($dtprocessa->add(new \DateInterval('P'.$nrdiassaidpd.'D')))) {
                        $mensagem = "dtForaPeriodoPermitido";
                    }
                } else {
                    $mensagem = "dtMaiorDtProcess";
                }
            }
        }
        return $mensagem;
    }
    
    public function retornaPrevreti($row) { 
		$filterCriteria = new FilterCriteria(self::PREVRETI);
		$this->parameterBagColumns = array('NRORG', 'CDFILIAL', 'CDSERVFILI', 'DTPREVRETI', 'NRREQUEXTRRE');
		$filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
		$filterCriteria->addCondition('CDFILIAL', $row['CDFILIAL']);
		$filterCriteria->addCondition('CDSERVFILI', $row['CDSERVFILI']);
		$filterCriteria->addCondition('DTPREVRETI', $row['DTPREVRETI']);
		$filterCriteria->addCondition('NRREQUEXTRRE', $row['NRREQUEXTRRE']);
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
    	$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
    	$rows = $dataSet->getRows();

    	return $rows;
    }
    
    private function treatError(\Exception $e, $response){ 
        $message = $e->getMessage();
        $code = $e->getCode();
        $arrayMessage = explode("\n", $message);
        if($this->isExceptionPebbian($e)){
            $response->setError(new Response\Error($message, $code));
            return;
        }
        if(!empty($arrayMessage)){
            $clean_message = "";
            foreach($arrayMessage as $currentString){
                if(stripos($currentString, "ORA-20501") !== false){
                    $clean_message = str_replace("ORA-20501: ", "", $currentString);
                }
            }
            if(!empty($clean_message)) {
                $response->setError(new Response\Error(utf8_decode($clean_message), 1)); 
            } else {
                $response->setError(new Response\Error($e->getMessage(), 1));
            }
        } else {
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    private function isExceptionPebbian(\Exception $error){ 
        return $error->getCode() === $this->service->getCodErrorPebbian();
    }
    
    private function verificaControleEstoque($row) { 
        $conditions = array (
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array(
                "columnName" => "CDPRODUTO",
                "operator" => "=",
                "value" => $row['CDPRODUTO']
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $this->environment->getNrOrg()
            )
        );

        $dataSet = $this->crudService->getFilteredDataSet('prodfili', $conditions);
        $rows = $dataSet->getRows();

        if ($rows[0]['IDCNTRESTOQ'] == 'N')
            return false;
        return true;
    }
    
    public function salvaNovaRetirada(Request\Row $request, Response $response) {
        try {
            $row = $request->getParameters();
            $dtretirada = $row['DTRETIRADA'];
            
             //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
            
            $this->setParamsEstoq($row);          
            $dtposicao = $this->service->getDataDeString($dtretirada);
            $dtposicao->add(new \DateInterval('P1D'));
            $dtposicao = $dtposicao->format('d/m/Y');
            $cdfilial_logada = $this->environment->getCdfilial();
            $row['CDFILILANC'] = null;
            $row['NRLANCESTQ'] = null;
            $row['NRSEQUITEM'] = '0000';
            $row['CDPRODMOVI'] = null;
            $row['DTPOSICAO'] = $dtposicao;
            $this->connection->beginTransaction(); 
            try {
                $error = array();
                $nrcardapio = $this->service->retornaNrcardapio($row);
                if(!$nrcardapio) {
                    $nrcardapio = $this->service->insereCardapio($row);
                }
                $row['IDTPLANCTO'] = 3;
                $row['DTLANCESTQ'] = $dtretirada;
                $row['DSLANCESTQ'] = "Retirada para o cardápio: ".$dtretirada.", Servico: ".$row['CDSERVFILI'];
                
                $nrlancto = $this->service->insereLancamento($row);
                $row['NRCARDAPIO'] = $nrcardapio;
                $row['NRLANCESTQ'] = $nrlancto;
                $row['NRREQUEXTRRE'] = $row['NRRETIRADA'];
                $row['IDUTRETIPRAT'] = $this->service->retornaIdutretiprat($row);
                $nrretirada = $this->service->insereRetirada($row);
                
                $row['IDTIPOMOVI'] = '3';
                $row['IDORDEMCLC'] = 9;
                $row['IDEVOLESTOQ'] = 'N';
                $row['NRLANCESTQ'] = $nrlancto;
                $row['NRRETIRADA'] = $nrretirada;
                $row['NRCARDAPIO'] = $nrcardapio;
                $row['IDINTESIST'] = 'N';
                
                $isValidStock = $this->service->validaEstoque($row, $error);
                if(!$isValidStock) {
                    $response->addDataSet(new DataSet('error', $error));
                    throw new \Exception("INVALID_STOCK");
                }
                $row['QTPREVISRETI'] = 0;
                $this->service->insereItemLancto($row, $cdfilial_logada);
                $this->service->insereItemreti($row, $cdfilial_logada);
                $this->connection->commit();
                $response->addDataSet(new DataSet("retirada", $row));
            } catch(\Exception $e){
                $this->connection->rollback();
                $response->addDataSet(new DataSet("error", array($e->getMessage())));
            }
         $this->service->atualizarEstoque($cdfilial_logada, $nrlancto); // passava idevolestoq como 'S'
        } catch (\Exception $e) {
            $this->treatError($e, $response);
        }
    }
    
    public function salvaNovoPratoProduto(Request\Row $request, Response $response) {

        try {
            $this->connection->beginTransaction();
            $row = $request->getParameters();
            
             //coleta de dados
            $this->libUtilities->saveCollectedData($row['CDFILIAL'],'RETIRADA');
            
            $cdfilimovi = $row['CDFILIAL'];
            $nrlancestq = $row['NRLANCESTQ'];
            $dtretirada = $row['DTRETIRADA'];
            
            $filialLancamento = $this->service->retornaFilialLancamentoRetirada($cdfilimovi, $row['NRCARDAPIO']);
            $dtposicao = $this->service->getDataDeString($dtretirada);
            $dtposicao->add(new \DateInterval('P1D'));
            $dtposicao = $dtposicao->format('d/m/Y');
            $row['NRSEQUITRETI'] = ' ';
            $itemreti = $this->service->Vd_ExistProdAlmox($row);

            if($itemreti) {
                throw new \Exception("existeItemCadastrado");
            }

            $maiorseq = $this->service->retornaMaxSequencial($row);
            $seq = str_pad(($maiorseq+1), 4, "0", STR_PAD_LEFT);

            $row['CDFILILANC'] = $filialLancamento;
            $row['NRSEQUITEM'] = $seq;
            $row['CDPRODMOVI'] = null;
            $row['DTPOSICAO'] = $dtposicao;
            $this->setParamsEstoq($row);
            $quantidadeInvalida = $this->service->validaQuantidadeEstoque($row);

            if($quantidadeInvalida) {
                throw new \Exception($quantidadeInvalida);
            }

            $controleEstoque = $this->verificaControleEstoque($row);

            if (!$controleEstoque) {
                throw new \Exception("prodNaoContrEstoq");
            }
            
            $row['IDTIPOMOVI'] = '3';
            $row['IDORDEMCLC'] = 9;
            $row['IDEVOLESTOQ'] = 'N';
            $row['IDINTESIST'] = 'N';
            
            $this->service->insereItemLancto($row, $filialLancamento);
            $this->service->insereItemreti($row, $filialLancamento);
            $this->service->atualizarEstoque($filialLancamento, $nrlancestq);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $this->treatError($e, $response);
        }
    }
    
    public function atualizarEstoque(Request\Row $request, Response $response) {
        try {
            $row = $request->getParameters();
            $cdfilancestq = $this->service->retornaFilialLancamentoRetirada($row['CDFILIAL'], $row['NRCARDAPIO']);
            $this->service->atualizarEstoque($cdfilancestq, $row['NRLANCESTQ'], false);
            $prodNaoEvoluidos = $this->service->getProdutosNaoEvoluidos($row['CDFILIAL'], $row['NRRETIRADA'], $row['NRCARDAPIO']);
            $prodNaoEvoluidosLista = array();
            foreach($prodNaoEvoluidos as &$prod){
                array_push($prodNaoEvoluidosLista, $prod["PRODUTO"]);
            }
            $response->addDataSet(new DataSet('produtosNaoEvoluidos', $prodNaoEvoluidosLista));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    public function retornaIdbloqincprodretipla(Request\Row $request, Response $response) {
        try {
            $row = $request->getParameters();
            $idbloqincprodretipla = $this->service->retornaIdbloqincprodretipla($row);
            $response->addDataSet(new DataSet('bloqInclusao', $idbloqincprodretipla));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), 1));
        }
    }
    
    public function atualizaCustoTotal(Request\Row $request, Response $response){
        $row = $request->getParameters();
        $custoTotalRow = $this->service->getCustoTotal($row['CDFILIAL'], $row['NRRETIRADA'], $row['NRCARDAPIO']);
        if(!$custoTotalRow){
            $response->addDataSet(new DataSet("custototal", [0]));
        }
        else{
            $vrcustototal = $custoTotalRow['CUSTOTOTAL'];
            if($vrcustototal == null)
                $vrcustototal = 0;
            $response->addDataSet(new DataSet("custototal", [$vrcustototal]));
        }
    }

}