<?php
namespace PlaNext\Services;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class PedidoAlteracaoCardapioService {
    protected $novoCodigo;
    public function __construct($entityManager, $environment, $novoCodigo){
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
        $this->novoCodigo = $novoCodigo;
    }
  
    public function retornaUltimoNRPEDALT($nrorg) {
        $nrpedaltcard = $this->novoCodigo->getCodeByTableName('PEDALTCARD', $nrorg);
        return (int)$nrpedaltcard;
    }
    
    public function retornaNRPEDALTCARDDIA($nrorg) {
        $nrpedaltcarddia = $this->novoCodigo->getCodeByTableName('PEDALTCARDDIA', $nrorg);
        return (int)$nrpedaltcarddia;
    }
    
    public function returnNRPEDALTCARDCOMP($nrorg) {
        $nrpedaltcarddia = $this->novoCodigo->getCodeByTableName('NRPEDALTCARDCOMP', $nrorg);
        return (int)$nrpedaltcarddia;
    }
    
    public function insertNovoPedido($row) {
        $sql = "INSERT INTO PEDALTCARD ( NRPEDALTCARD, CDPEDALTCARD, DTPEDALTCARD, CDOPERADOR, CDFILIAL, CDSERVFILI, DTMESCARD, IDSITUACAO, IDSOLEXTRA, IDRECEMCONCLUIDO, IDRECEMAPROVREPR, DSJUSTICATIVA, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU ) 
            VALUES ( :P_NRPEDALTCARD, :P_CDPEDALTCARD, SYSDATE, :P_CDOPERADOR, :P_CDFILIAL, :P_CDSERVFILI, :P_DTMESCARD, :P_IDSITUACAO, :P_IDSOLEXTRA, :P_IDRECEMCONCLUIDO, :P_IDRECEMAPROVREPR, :P_DSJUSTICATIVA, :P_NRORG, 'S', SYSDATE, NULL, :P_NRORG, :P_CDOPERADOR, NULL, NULL )";
        $params = array(
            "P_NRPEDALTCARD"      => $row['NRPEDALTCARD'],
            "P_CDPEDALTCARD"      => $row['CDPEDALTCARD'],
            "P_CDOPERADOR"        => $row['CDOPERADOR'],
            "P_CDFILIAL"          => $row['CDFILIAL'],
            "P_CDSERVFILI"        => $row['CDSERVFILIMODAL'],
            "P_DTMESCARD"         => $row['DTMESMODAL'],
            "P_IDSITUACAO"        => 'E',
            "P_IDSOLEXTRA"        => 'N',
            "P_IDRECEMCONCLUIDO"  => 'N',
            "P_IDRECEMAPROVREPR"  => 'N',
            "P_DSJUSTICATIVA"     => $row['JUSTIFMODAL'],
            "P_NRORG"             => $row['NRORG']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $this->connection->commit();
        }catch(Exception $e) {
            $this->connection->rollback();
            throw $e; 
          }
    }
    
    public function deletePedidoService($row){
        $sql = "DELETE FROM PEDALTCARD WHERE CDPEDALTCARD = :P_CDPEDALTCARD AND CDFILIAL = :P_CDFILIAL";
        $params = array(
            "P_CDPEDALTCARD"      => $row['CDPEDALTCARD'],
            "P_CDFILIAL"          => $row['CDFILIAL']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $this->connection->commit();
        }catch(Exception $e) {
            $this->connection->rollback();
            throw $e; 
        }
    }
    
    public function atualizaPedidoService($row){
        $sql = "UPDATE PEDALTCARD SET DSJUSTICATIVA = :P_DSJUSTICATIVA, DTULTATU = SYSDATE, NRORGULTATU = :P_NRORG, CDOPERULTATU = :P_CDOPERADOR WHERE NRPEDALTCARD = :P_NRPEDALTCARD AND CDPEDALTCARD = :P_CDPEDALTCARD AND CDFILIAL = :P_CDFILIAL";
        $params = array(
            "P_DSJUSTICATIVA"     => $row['DSJUSTICATIVA'],
            "P_NRPEDALTCARD"      => $row['NRPEDALTCARD'],
            "P_CDPEDALTCARD"      => $row['CDPEDALTCARD'],
            "P_CDFILIAL"          => $row['CDFILIAL'],
            "P_NRORG"             => $row['NRORG'],
            "P_CDOPERADOR"        => $row['CDOPERADOR']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $this->connection->commit();
        }catch(Exception $e) {
            $this->connection->rollback();
            throw $e; 
        }
    }
    
    public function carregaColunasCardapioService($row){
        $sql = "SELECT C.NRORG, C.IDRESERVAMYMENU, CDFILIAL, DSCOMPLTPMES, CDSERVFILI, NRSEQCNFMES, DTREFECNFMES, C.CDTIPOPRAT, NMTIPOPRAT, IDOPCCNFMES, IDIMPCNFMES, NRORDCNFMES, IDCONCNFMES, IDTPPRATSEGM, IDTPPRATTERM, IDTPPRATQUAM, IDTPPRATQUIM, IDTPPRATSEXM, IDTPPRATSABM, IDTPPRATDOMM, IDTPPRATFERM FROM CNFCAMES C JOIN TIPOPRAT T ON C.CDTIPOPRAT = T.CDTIPOPRAT WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND C.NRORG = :P_NRORG AND DTREFECNFMES = :P_DTREFECNFMES";
        $params = array(
          "P_CDFILIAL"  => $row['CDFILIAL'],
          "P_CDSERVFILI"  => $row['CDSERVFILI'],
          "P_NRORG"  => $row['NRORG'],
          "P_DTREFECNFMES"  => $row['DTREFECNFMES']
        );   
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function retornaPratosCardapio($row){
        $dataInicial = $row['DTMESCARD'];
        list($dia, $mes, $ano) = explode('/', $dataInicial);
        $dataFinal = date('t', strtotime("$ano-$mes-$dia"));
        $dataFinalFormatada = sprintf('%02d', $dataFinal) . '/' . sprintf('%02d', $mes) . '/' . $ano;
        $sql = "SELECT TO_CHAR(CD.DTCARDAPIO, 'DD/MM/YYYY') AS DTCARDAPIO, CC.NRCARDAPIO, CC.NRSEQPRATCAR, CD.CDFILIAL, CD.CDSERVFILI, CC.NRORDPRATCAR, TP.NMTIPOPRAT, CC.CDTIPOPRAT, CC.CDPRATO, PT.NMPRATO, PT.CDNVPRATO, ROUND(CC.VRPECONSCARD, 2) AS PERCENTUAL, ROUND(((CC.VRPECONSCARD / 100) * CD.QTCOMESTICAR), 2) AS COMENS, CD.QTCOMESTICAR, CC.VRPECONSCARD, C.DTREFECNFMES, C.IDCONCNFMES, CD.NRORG, CC.IDPRATOESPE, P.TIPO                                                AS TIPO_RECEITA, PT.CDFILAUXPRAT, CC.CDFILAUXPRATMAT, CC.CDPRATOMAT, P.NRCOR, PT.IDATIVO                                            AS IDATIVOPRAT, P.IDATIVORECE FROM CARDADIA CD JOIN COMPCARD CC ON CC.CDFILIAL = CD.CDFILIAL AND CC.NRCARDAPIO = CD.NRCARDAPIO AND CD.NRORG = CC.NRORG JOIN PRATO    PT ON PT.CDPRATO = CC.CDPRATO AND PT.CDFILAUXPRAT = CC.CDFILAUXPRAT AND CC.NRORG = PT.NRORG JOIN TIPOPRAT TP ON TP.CDTIPOPRAT = CC.CDTIPOPRAT AND PT.NRORG = TP.NRORG JOIN CNFCAMES C ON C.CDTIPOPRAT = TP.CDTIPOPRAT AND C.NRSEQCNFMES = CC.NRSEQPRATCAR AND CD.CDFILIAL = C.CDFILIAL AND CD.CDSERVFILI = C.CDSERVFILI AND TP.NRORG = C.NRORG AND C.DTREFECNFMES = CD.DTMESCARD INNER JOIN ( SELECT DISTINCT RS.CDPRATO, RS.CDFILAUXPRAT, PS.NRCORPRISERV      AS NRCOR, 'S'                  AS TIPO, NVL(PS.IDATIVO, 'S') AS IDATIVORECE FROM RECESERV RS LEFT JOIN PRATSERV PS ON RS.CDFILIAL = PS.CDFILIAL AND RS.CDSERVFILI = PS.CDSERVFILI AND RS.CDPRATO = PS.CDPRATO AND RS.CDFILAUXPRAT = PS.CDFILAUXPRAT WHERE RS.CDFILIAL = :P_CDFILIAL AND RS.CDSERVFILI = :P_CDSERVFILI UNION ALL SELECT DISTINCT RF.CDPRATO, RF.CDFILAUXPRAT, PF.NRCORPRIFILI      AS NRCOR, 'F'                  AS TIPO, NVL(PF.IDATIVO, 'S') AS IDATIVORECE FROM RECEFILI RF LEFT JOIN PRATFILI PF ON RF.CDFILIAL = PF.CDFILIAL AND RF.CDPRATO = PF.CDPRATO AND RF.CDFILAUXPRAT = PF.CDFILAUXPRAT WHERE RF.CDFILIAL = :P_CDFILIAL AND NOT EXISTS ( SELECT RS.CDPRATO FROM RECESERV RS WHERE RS.CDFILIAL = :P_CDFILIAL AND RS.CDSERVFILI = :P_CDSERVFILI AND RS.CDPRATO = RF.CDPRATO AND RS.CDFILAUXPRAT = RF.CDFILAUXPRAT ) UNION ALL SELECT DISTINCT RP.CDPRATO, RP.CDFILAUXPRAT, PP.NRCORPRIPRAT      AS NRCOR, 'F'                  AS TIPO, NVL(PP.IDATIVO, 'S') AS IDATIVORECE FROM RECEPADR RP LEFT JOIN PRATPADR PP ON RP.CDPRATO = PP.CDPRATO AND RP.CDFILAUXPRAT = PP.CDFILAUXPRAT WHERE NOT EXISTS ( SELECT RS.CDPRATO FROM RECESERV RS WHERE RS.CDFILIAL = :P_CDFILIAL AND RS.CDSERVFILI = :P_CDSERVFILI AND RS.CDPRATO = RP.CDPRATO AND RS.CDFILAUXPRAT = RP.CDFILAUXPRAT ) AND NOT EXISTS ( SELECT RF.CDPRATO FROM RECEFILI RF WHERE RF.CDFILIAL = :P_CDFILIAL AND RF.CDPRATO = RP.CDPRATO AND RF.CDFILAUXPRAT = RP.CDFILAUXPRAT ) )        P ON P.CDFILAUXPRAT = CC.CDFILAUXPRAT AND P.CDPRATO = CC.CDPRATO WHERE CD.CDFILIAL = :P_CDFILIAL AND CD.CDSERVFILI = :P_CDSERVFILI AND C.DTREFECNFMES = :P_DTREFECNFMES AND CD.NRORG = :P_NRORG AND CD.DTCARDAPIO BETWEEN TO_DATE(:P_DATAINICIAL, 'DD/MM/YYYY') AND TO_DATE(:P_DATAFINAL, 'DD/MM/YYYY') ORDER BY CD.DTCARDAPIO";
        $params = array(
          "P_CDFILIAL"  => $row['CDFILIAL'],
          "P_CDSERVFILI"  => $row['CDSERVFILI'],
          "P_DTREFECNFMES"  => $row['DTREFECNFMES'],
          "P_NRORG"  => $row['NRORG'],
          "P_DATAINICIAL" => $dataInicial,
          "P_DATAFINAL" => $dataFinalFormatada
        );   
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function retornaComensaisCardapio($row){
        $dataInicial = $row['DTMESCARD'];
        list($dia, $mes, $ano) = explode('/', $dataInicial);
        $dataFinal = date('t', strtotime("$ano-$mes-$dia"));
        $dataFinalFormatada = sprintf('%02d', $dataFinal) . '/' . sprintf('%02d', $mes) . '/' . $ano;
        $sql = "SELECT C.CDFILIAL, C.NRCARDAPIO, C.CDSERVFILI, C.CDOPERADOR, TO_CHAR(C.DTCARDAPIO, 'DD/MM/YYYY') AS DTCARDAPIO, C.QTCOMESTICAR, C.QTESTITURNO1, C.QTESTITURNO2, C.QTESTITURNO3, C.QTESTITURNO4, C.QTCLIESTICAR, C.QTFUNESTICAR, C.QTDOAESTICAR, C.QTCOMEFETCAR, C.QTCLIEFETCAR, C.QTFUNEFETCAR, C.QTDOAEFETCAR, C.DTMESCARD, C.DSCARDAPIO, C.DTULTATUACAR, C.NRORG FROM CARDADIA C LEFT JOIN CARDAMES CM ON C.CDFILIAL = CM.CDFILIAL AND C.CDSERVFILI = CM.CDSERVFILI AND C.DTMESCARD = CM.DTMESCARDAPIO AND C.NRORG = CM.NRORG WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI AND C.NRORG = :P_NRORG AND C.DTCARDAPIO BETWEEN TO_DATE(:P_DATAINICIAL, 'DD/MM/YYYY') AND TO_DATE(:P_DATAFINAL, 'DD/MM/YYYY') ORDER BY C.DTCARDAPIO";
        $params = array(
          "P_CDFILIAL"  => $row['CDFILIAL'],
          "P_CDSERVFILI"  => $row['CDSERVFILI'],
          "P_NRORG"  => $row['NRORG'],
          "P_DATAINICIAL" => $dataInicial,
          "P_DATAFINAL" => $dataFinalFormatada
        );   
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function verificaExisteCardapio($row){
        $sql = "SELECT COUNT(1) AS EXISTE_CARDAPIO FROM CARDADIA C WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI AND TRUNC(C.DTCARDAPIO, 'MM') = :P_DTMESCARD AND EXISTS (SELECT 1 FROM COMPCARD CC WHERE CC.CDFILIAL = C.CDFILIAL AND CC.NRCARDAPIO = C.NRCARDAPIO)";
        $params = array(
          "P_CDFILIAL"  => $row['CDFILIAL'],
          "P_CDSERVFILI"  => $row['CDSERVFILIMODAL'],
          "P_DTMESCARD"=> $row['DTMESCARD']
        );   
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function verificaTurnosCardaDia($row){
        $nrPedAltCard = isset($row['NRPEDALTCARD']) ? $row['NRPEDALTCARD'] : null;
        $sql = "SELECT CDTURNO, NMTURNO, NVL(CASE CDTURNO WHEN '1' THEN NVL(P.QTESTITURNO1,C.QTESTITURNO1) WHEN '2' THEN NVL(P.QTESTITURNO2, C.QTESTITURNO2) WHEN '3' THEN NVL(P.QTESTITURNO3, C.QTESTITURNO3) WHEN '4' THEN NVL(P.QTESTITURNO4, C.QTESTITURNO4) END,0) AS COMENSAIS FROM CARDADIA C JOIN TURNOSERV T ON C.CDFILIAL = T.CDFILIAL AND C.CDSERVFILI = T.CDSERVFILI LEFT JOIN PEDALTCARDDIA P ON P.CDFILIAL = T.CDFILIAL AND P.DTCARDAPIO = C.DTCARDAPIO AND P.NRPEDALTCARD = :NRPEDALTCARD WHERE C.CDFILIAL = :P_CDFILIAL AND C.CDSERVFILI = :P_CDSERVFILI AND C.DTCARDAPIO = :P_DTCARDAPIO ORDER BY CDTURNO";
        $params = array(
            "NRPEDALTCARD" => $nrPedAltCard,
            "P_CDFILIAL"   => $row['CDFILIAL'],
            "P_CDSERVFILI" => $row['CDSERVFILI'],
            "P_DTCARDAPIO" => $row['DTDIACARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function verificaPontoEntrega($row){
        $sql = "SELECT A.CDFILIAL, A.NRCARDAPIO, A.CDCLIENTE, C.NMRAZSOCCLIE, A.CDDESTFILI, D.NMDESTFILI, A.QTCOMESTICLI FROM DESTFILI D, CLIENTE C, CARDACLI A, CARDADIA B WHERE(B.CDFILIAL = :P_CDFILIAL) AND (B.CDSERVFILI = :P_CDSERVFILI) AND (B.DTCARDAPIO = :P_DTCARDAPIO) AND (A.CDFILIAL = B.CDFILIAL) AND (A.NRCARDAPIO = B.NRCARDAPIO) AND (A.CDCLIENTE = C.CDCLIENTE) AND (A.CDFILIAL = D.CDFILIAL) AND (A.CDDESTFILI = D.CDDESTFILI)";
        $params = array(
            "P_CDFILIAL" => $row['CDFILIAL'],
            "P_CDSERVFILI" => $row['CDSERVFILI'],
            "P_DTCARDAPIO" => $row['DTDIACARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function retornaCenario($row) {
        $sql = "SELECT A.CDFILIAL, A.CDSERVFILI, A.CDCLIENTE, C.NMRAZSOCCLIE, A.CDDESTFILI, D.NMDESTFILI, NVL(DECODE(NVL(:P_DIADASEMANA, '7'), '0',A.QTCOESTCSDOM, '1',A.QTCOESTCSSEG, '2',A.QTCOESTCSTER, '3',A.QTCOESTCSQUA, '4',A.QTCOESTCSQUI, '5',A.QTCOESTCSSEX, '6',A.QTCOESTCSSAB, '7',A.QTCOESTCSFER), 0) QTCOMESTICLI FROM DESTFILI D, CLIENTE C, DESTSRVCLI A WHERE(A.CDFILIAL = :P_CDFILIAL) AND (A.CDSERVFILI = :P_CDSERVFILI) AND (A.CDCLIENTE = C.CDCLIENTE) AND (A.CDFILIAL = D.CDFILIAL) AND (A.CDDESTFILI = D.CDDESTFILI)";
        $params = array(
            "P_CDFILIAL" => $row['CDFILIAL'],
            "P_CDSERVFILI" => $row['CDSERVFILI'],
            "P_DIADASEMANA" => $row['DIADASEMANA']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function retornaNrCardapio($row) {
        $sql = "SELECT NRCARDAPIO FROM CARDADIA WHERE CDFILIAL = :P_CDFILIAL AND CDSERVFILI = :P_CDSERVFILI AND DTCARDAPIO = :P_DTCARDAPIO";
        $params = array(
          "P_CDFILIAL"  => $row['CDFILIAL'],
          "P_CDSERVFILI"  => $row['CDSERVFILI'],
          "P_DTCARDAPIO"=> $row['DTDIACARD']
        );   
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function changedDinerDtMenu($row) {
        $sql = "SELECT * FROM PEDALTCARDDIA WHERE NRPEDALTCARD = :NRPEDALTCARD AND DTCARDAPIO = :DTCARDAPIO";
        $params = array(
          "NRPEDALTCARD"  => $row['valoresUpdate']['row']['NRPEDALTCARD'],
          "DTCARDAPIO"=> $row['valoresUpdate']['row']['DTDIACARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function updateChangedDish($row) {
        $dish = $row['valoresUpdate']['dish'];
        $rowMenu = $row['valoresUpdate']['row'];
        $sql = "UPDATE PEDALTCARDCOMP SET CDPRATO = :CDPRATO, CDFILAUXPRAT = :CDFILAUXPRAT, VRPECONSCARD = :VRPECONSCARD, DTULTATU = SYSDATE, CDOPERULTATU = :CDOPERADOR, NRORGULTATU = :NRORG WHERE NRPEDALTCARD = :NRPEDALTCARD AND NRSEQPRATCAR = :NRSEQPRATCAR AND DTCARDAPIO = :DTCARDAPIO";
        $params = array(
            "CDPRATO"      => $dish['selectPratoModal']['CDPRATO'],
            "CDFILAUXPRAT"      => $dish['selectPratoModal']['CDFILAUXPRAT'],
            "VRPECONSCARD" => $dish['VRPECONSCARD'],
            "CDOPERADOR"   => $this->environment->getCdOperador(),
            "NRORG"        => $this->environment->getNrOrg(),
            "NRPEDALTCARD" => $rowMenu['NRPEDALTCARD'],
            "NRSEQPRATCAR" => $dish['NRSEQPRATCAR'],
            "DTCARDAPIO"   => $dish['DTDIACARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function insertChangedDish($row) {
        $dish = $row['valoresUpdate']['dish'];
        $rowMenu = $row['valoresUpdate']['row'];
        $sql = "INSERT INTO PEDALTCARDCOMP ( NRPEDALTCARDCOMP, NRPEDALTCARD, CDFILIAL, NRCARDAPIO, NRSEQPRATCAR, CDFILAUXPRAT, CDPRATO, VRPECONSCARD, NRORG, IDATIVO, DTINCLUSAO, NRORGINCLUSAO, CDOPERINCLUSAO, DTCARDAPIO ) VALUES ( :NRPEDALTCARDCOMP, :NRPEDALTCARD, :CDFILIAL, :NRCARDAPIO, :NRSEQPRATCAR, :CDFILAUXPRAT, :CDPRATO, :VRPECONSCARD, :NRORG, 'S', SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO, :DTCARDAPIO)";
        $params = array(
            "NRPEDALTCARDCOMP" => $row['NRPEDALTCARDCOMP'],
            "NRPEDALTCARD"     => $rowMenu['NRPEDALTCARD'],
            "CDFILIAL"         => $rowMenu['CDFILIAL'],
            "NRCARDAPIO"       => $row['NRCARDAPIO'],
            "NRSEQPRATCAR"     => $dish['NRSEQPRATCAR'],
            "CDFILAUXPRAT"     => $dish['selectPratoModal']['CDFILAUXPRAT'],
            "CDPRATO"          => $dish['selectPratoModal']['CDPRATO'],
            "VRPECONSCARD"     => $dish['VRPECONSCARD'],
            "NRORG"            => $this->environment->getNrOrg(),
            "NRORGINCLUSAO"    => $this->environment->getNrOrg(),
            "CDOPERINCLUSAO"   => $this->environment->getCdOperador(),
            "DTCARDAPIO"       => $dish['DTDIACARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function changedDishDtMenu($row) {
        $sql = "SELECT P.DTCARDAPIO, P.NRSEQPRATCAR, P.CDPRATO, P.VRPECONSCARD, PT.NMPRATO FROM PEDALTCARDCOMP P JOIN PRATO PT ON PT.CDPRATO = P.CDPRATO WHERE NRPEDALTCARD = :NRPEDALTCARD AND DTCARDAPIO = :DTCARDAPIO AND NRSEQPRATCAR = :NRSEQPRATCAR";
        $params = array(
          "NRPEDALTCARD"  => $row['valoresUpdate']['row']['NRPEDALTCARD'],
          "DTCARDAPIO"=> $row['valoresUpdate']['dish']['DTDIACARD'],
          "NRSEQPRATCAR"=> $row['valoresUpdate']['dish']['NRSEQPRATCAR']
        );
        
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function returnsChangedDiners($row) {
        $sql = "SELECT * FROM PEDALTCARDDIA WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
          "NRPEDALTCARD"  => $row['NRPEDALTCARD']
        );   
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function returnsChangedDishes($row) {
        $sql = "SELECT P.DTCARDAPIO, P.NRSEQPRATCAR, P.CDFILAUXPRAT, P.CDPRATO, P.VRPECONSCARD, PT.NMPRATO FROM PEDALTCARDCOMP P JOIN PRATO PT ON PT.CDPRATO = P.CDPRATO WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
          "NRPEDALTCARD"  => $row['NRPEDALTCARD']
        );   
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function retornaDataCardapioAlteracao($nrCardapio) {
        $sql = "SELECT CDFILIAL, NRCARDAPIO, CDSERVFILI, DTCARDAPIO FROM CARDADIA WHERE NRCARDAPIO = :P_NRCARDAPIO";
        $params = array(
          "P_NRCARDAPIO"=> $nrCardapio
        );   
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function alteraComensaisTurnoService($row) {
        $turnos = $row['valoresUpdate']['turnos'];
        $turno1 = isset($turnos['1']) ? $turnos['1'] : null;
        $turno2 = isset($turnos['2']) ? $turnos['2'] : null;
        $turno3 = isset($turnos['3']) ? $turnos['3'] : null;
        $turno4 = isset($turnos['4']) ? $turnos['4'] : null;
        $sql = "UPDATE PEDALTCARDDIA SET QTCLIESTICAR = :P_QTCLIESTICAR, QTFUNESTICAR = :P_QTFUNESTICAR, QTDOAESTICAR = :P_QTDOAESTICAR, QTESTITURNO1 = :P_QTESTITURNO1, QTESTITURNO2 = :P_QTESTITURNO2, QTESTITURNO3 = :P_QTESTITURNO3, QTESTITURNO4 = :P_QTESTITURNO4, DTULTATU = SYSDATE, NRORGULTATU = :P_NRORG, CDOPERULTATU = :P_CDOPERADOR WHERE NRPEDALTCARDDIA = :P_NRPEDALTCARDDIA";
        $params = array(
            "P_QTCLIESTICAR"      => $row['valoresUpdate']['rowComensais']['clientesComensais'],
            "P_QTFUNESTICAR"      => $row['valoresUpdate']['rowComensais']['funcionariosComensais'],
            "P_QTDOAESTICAR"      => $row['valoresUpdate']['rowComensais']['doacoesComensais'],
            "P_QTESTITURNO1"      => $turno1,
            "P_QTESTITURNO2"      => $turno2,
            "P_QTESTITURNO3"      => $turno3,
            "P_QTESTITURNO4"      => $turno4,
            "P_NRORG"             => $row['NRORG'],
            "P_CDOPERADOR"        => $row['CDOPERADOR'],
            "P_NRPEDALTCARDDIA"   => $row['NRPEDALTDIA']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function insereComensaisTurnoService($row) {
        $turnos = $row['valoresUpdate']['turnos'];
        $turno1 = isset($turnos['1']) ? $turnos['1'] : null;
        $turno2 = isset($turnos['2']) ? $turnos['2'] : null;
        $turno3 = isset($turnos['3']) ? $turnos['3'] : null;
        $turno4 = isset($turnos['4']) ? $turnos['4'] : null;
        $sql = "INSERT INTO PEDALTCARDDIA ( NRPEDALTCARDDIA, NRPEDALTCARD, CDFILIAL, NRCARDAPIO, DTCARDAPIO, QTCLIESTICAR, QTFUNESTICAR, QTDOAESTICAR, QTESTITURNO1, QTESTITURNO2, QTESTITURNO3, QTESTITURNO4, NRORG, IDATIVO, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU ) 
            VALUES ( :P_NRPEDALTCARDDIA, :P_NRPEDALTCARD, :P_CDFILIAL, :P_NRCARDAPIO, :P_DTCARDAPIO, :P_QTCLIESTICAR, :P_QTFUNESTICAR, :P_QTDOAESTICAR, :P_QTESTITURNO1, :P_QTESTITURNO2, :P_QTESTITURNO3, :P_QTESTITURNO4, :P_NRORG, 'S', SYSDATE, NULL, :P_NRORG, :P_CDOPERADOR, NULL, NULL )";
        $params = array(
            "P_NRPEDALTCARDDIA"   => $row['NRPEDALTCARDDIA'],
            "P_NRPEDALTCARD"      => $row['valoresUpdate']['row']['NRPEDALTCARD'],
            "P_CDFILIAL"          => $row['valoresUpdate']['row']['CDFILIAL'],
            "P_NRCARDAPIO"        => $row['NRCARDAPIO'],
            "P_DTCARDAPIO"        => $row['valoresUpdate']['row']['DTDIACARD'],
            "P_QTCLIESTICAR"      => $row['valoresUpdate']['rowComensais']['clientesComensais'],
            "P_QTFUNESTICAR"      => $row['valoresUpdate']['rowComensais']['funcionariosComensais'],
            "P_QTDOAESTICAR"      => $row['valoresUpdate']['rowComensais']['doacoesComensais'],
            "P_QTESTITURNO1"      => $turno1,
            "P_QTESTITURNO2"      => $turno2,
            "P_QTESTITURNO3"      => $turno3,
            "P_QTESTITURNO4"      => $turno4,
            "P_NRORG"             => $row['NRORG'],
            "P_CDOPERADOR"        => $row['CDOPERADOR'],
            
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function completeOrder($row, $idSolExtra, $operator) {
        $sql = "UPDATE PEDALTCARD SET IDSITUACAO = 'C', IDRECEMCONCLUIDO = 'S', DTULTATU = SYSDATE, CDOPERULTATU = :CDOPERADOR, IDSOLEXTRA = :IDSOLEXTRA WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
            "CDOPERADOR"   => $operator,
            "IDSOLEXTRA"   => $idSolExtra,
            "NRPEDALTCARD" =>  $row['NRPEDALTCARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function requestAgain($row, $operator) {
        $procedure = new StoredProcedure($this->connection, 'REPLICA_PEDALTCARD');
        $procedure->addParam(new Param('P_NRPEDALTCARD', Param::PARAM_INPUT, $row['NRPEDALTCARD'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operator, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function disapproveGO($row, $operator) {
        $sql = "UPDATE PEDALTCARD SET IDSITUACAO = 'R', IDRECEMAPROVREPR = 'S', DTAPROVREPRGO = SYSDATE, CDOPERAPROVREPRGO = :CDOPERADOR, DSRESPOSTA = :DSRESPOSTA WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
            "CDOPERADOR"   => $operator,
            "DSRESPOSTA"   => $row['JUSTIFICATION'],
            "NRPEDALTCARD" =>  $row['CURRENTROW']['NRPEDALTCARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function approveGO($row, $operator) {
        $sql = "UPDATE PEDALTCARD SET IDRECEMCONCLUIDO = 'S', DTAPROVREPRGO = SYSDATE, CDOPERAPROVREPRGO = :CDOPERADOR, DSRESPOSTA = :DSRESPOSTA WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
            "CDOPERADOR"   => $operator,
            "DSRESPOSTA"   => $row['JUSTIFICATION'],
            "NRPEDALTCARD" =>  $row['CURRENTROW']['NRPEDALTCARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function disapprove($row, $operator) {
        $sql = "UPDATE PEDALTCARD SET IDSITUACAO = 'R', IDRECEMAPROVREPR = 'S', DTAPROVREPR = SYSDATE, CDOPERAPROVREPR = :CDOPERADOR, DSRESPOSTA = :DSRESPOSTA WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
            "CDOPERADOR"   => $operator,
            "DSRESPOSTA"   => $row['JUSTIFICATION'],
            "NRPEDALTCARD" =>  $row['CURRENTROW']['NRPEDALTCARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function approve($row, $operator) {
        $procedure = new StoredProcedure($this->connection, 'APROVA_PEDALTCARD');
        $procedure->addParam(new Param('P_NRPEDALTCARD', Param::PARAM_INPUT, $row['CURRENTROW']['NRPEDALTCARD'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operator, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DSRESPOSTA', Param::PARAM_INPUT, $row['JUSTIFICATION'], Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function withdrawalMenu($row) {
        $sql = "SELECT COUNT(1) AS RETIRADAS FROM PEDALTCARD P INNER JOIN CARDADIA C ON C.CDFILIAL = P.CDFILIAL AND C.CDSERVFILI = P.CDSERVFILI AND C.DTMESCARD = P.DTMESCARD INNER JOIN RETIRADA R ON R.CDFILIAL = C.CDFILIAL AND R.NRCARDAPIO = C.NRCARDAPIO WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
            "NRPEDALTCARD"    =>  $row['NRPEDALTCARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function effectivesMenu($row) {
        $sql = "SELECT COUNT(1) AS EFETIVOS FROM ( SELECT 1 FROM PEDALTCARD P INNER JOIN EFETCLIE E ON E.CDFILIAL = P.CDFILIAL AND (E.CDSERVFILI = P.CDSERVFILI OR E.CDSERVPLAN = P.CDSERVFILI) WHERE P.NRPEDALTCARD = :NRPEDALTCARD AND E.DTEFETCLIE BETWEEN TRUNC(P.DTMESCARD, 'MM') AND LAST_DAY(P.DTMESCARD) AND E.NRAPURACFAT IS NOT NULL UNION ALL SELECT 1 FROM PEDALTCARD P INNER JOIN EFECLIFAT E ON E.CDFILIAL = P.CDFILIAL AND (E.CDSERVFILI = P.CDSERVFILI OR E.CDSERVPLAN = P.CDSERVFILI) WHERE P.NRPEDALTCARD = :NRPEDALTCARD AND E.DTFATURAM BETWEEN TRUNC(P.DTMESCARD, 'MM') AND LAST_DAY(P.DTMESCARD) AND E.NRAPURACFAT IS NOT NULL )";
        $params = array(
            "NRPEDALTCARD"    =>  $row['NRPEDALTCARD']
        );
        try{
            $std = $this->connection->prepare($sql);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        }catch(Exception $e) {
            throw $e; 
        }
    }
    
    public function setAsViewedCompleted($nrPedAltCard) {
        $sql = "UPDATE PEDALTCARD SET IDRECEMCONCLUIDO = 'N' WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
            "NRPEDALTCARD" =>  $nrPedAltCard
        );
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function setAsViewedApDisap($nrPedAltCard) {
        $sql = "UPDATE PEDALTCARD SET IDRECEMAPROVREPR = 'N' WHERE NRPEDALTCARD = :NRPEDALTCARD";
        $params = array(
            "NRPEDALTCARD" =>  $nrPedAltCard
        );    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    public function setEmailPlan($cdfilial, $dsemail, $isNew) {
        $sql = "
        UPDATE EMAILPEDALTCARD
        SET DSEMAIL = :DSEMAIL,
        DTULTATU = SYSDATE,
        CDOPERULTATU = :CDOPERADOR
        WHERE CDFILIAL = :CDFILIAL";
        if($isNew) {
            $sql = "
            INSERT INTO EMAILPEDALTCARD
            (CDFILIAL, DSEMAIL, DTINCLUSAO, DTULTATU,
            CDOPERINCLUSAO, CDOPERULTATU)
            VALUES
            (:CDFILIAL, :DSEMAIL, SYSDATE, SYSDATE,
            :CDOPERADOR, :CDOPERADOR)";
        }
        $params = array(
            "CDFILIAL" =>  $cdfilial,
            "DSEMAIL" => $dsemail,
            "CDOPERADOR" => $this->environment->getCdOperador()
        );    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
}