import {
  Http,
  Metadata,
  IDictionary,
  IEventParam,
  I18n,
} from '@zeedhi/core';
import {
  Select,
  Card,
  Grid,
  Form,
  Modal,
  Button,
  Divider,
  Number as NumberInput,
  LoadingService,
  GridColumnEditable,
  GridEditable,
  ModalService,
  AlertService,
  DialogService,
  Textarea,
  IComponentRender,
} from '@zeedhi/common';

import { Utilities } from '@/controllers/Utilities';

export class PedidoAlteracaoCardapioController {
  public newRow = false;

  public isInlineEditing = false;

  private modalIncluir!: Modal;

  private modalAlteracao!: Modal;

  private modalResumo!: Modal;

  private modalAlteraCellComensais!: Modal;

  private modalAlteraPratos!: Modal;

  private approveDisapproveModal!: Modal;

  private completeOrderModal!: Modal;

  private emailModal!: Modal;

  private emailPlan = '';

  private modalInCreation = false;

  private initialValues: any = {};

  private emailCallback: any = null;

  public checkModeApproval() {
    const page = Metadata.getInstance<Card>('pedidoAlteracaoCardapio');
    return (page.cssClass === 'APPROVAL');
  }

  public visibleButtonsApproval() {
    const addButton = Metadata.getInstance<Button>('botaoIncluirPedido');
    const deleteOrderButton = Metadata.getInstance<Button>('botaoExcluirPedido');
    const refreshButton = Metadata.getInstance<Button>('refreshGrid');
    const divisor1 = Metadata.getInstance<Divider>('divisor2');
    const concludeButton = Metadata.getInstance<Button>('botaoConcluirPedido');
    const requestButton = Metadata.getInstance<Button>('botaoSolicitarNovamente');
    const mailButton = Metadata.getInstance<Button>('mailButton');
    const divisor2 = Metadata.getInstance<Divider>('divisor3');
    const cancelButton = Metadata.getInstance<Button>('cancelButton');
    const saveButton = Metadata.getInstance<Button>('saveButton');
    addButton.isVisible = false;
    deleteOrderButton.isVisible = false;
    refreshButton.isVisible = true;
    divisor1.isVisible = false;
    concludeButton.isVisible = false;
    requestButton.isVisible = false;
    divisor2.isVisible = false;
    cancelButton.isVisible = false;
    saveButton.isVisible = false;
    mailButton.isVisible = false;
  }

  public async getInicialPedidoAlteracao() {
    Http.post('/getInitialValues').then((response) => {
      this.initialValues = response.data.dataset.initialValues;
      const filialFiltro = Metadata.getInstance<Select>('FILIAL_FILTRO');
      filialFiltro.value = this.initialValues.CDFILIAL;
    });
  }

  public destroyModal() {
    this.destroySheetModals();
  }

  public idSolExtraAction({ row }: any) {
    return row.IDSOLEXTRA === 'S';
  }

  public idSolExtraActionOff({ row }: any) {
    return row.IDSOLEXTRA === 'N';
  }

  public filtroFilialFilioper() {
    const filialFiltro = Metadata.getInstance<Select>('FILIAL_FILTRO');
    filialFiltro.datasource.get();
  }

  public configuraGridAlteracao() {
    const cardGridAlteracao = Metadata.getInstance<Card>('cardGrid');
    const gridAlteracao = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const cardForms = Metadata.getInstance<Card>('cardForms');
    const filialFiltro = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const filtro = {
      CDFILIAL: filialFiltro,
      MODO: (this.checkModeApproval()) ? 'A' : 'P',
    };
    if (this.checkModeApproval()) {
      this.visibleButtonsApproval();
    }
    gridAlteracao.datasource.filter = filtro;
    cardForms.isVisible = false;
    gridAlteracao.datasource.get().then(() => {
      cardGridAlteracao.isVisible = true;
    });
    this.getEmailPlan();
  }

  public returnGrid(component: IDictionary) {
    let grid: IDictionary;
    if (component.type === 'button') grid = component.parent as IDictionary;
    else grid = component;
    return grid;
  }

  public async criaModalIncluir() {
    if (!this.modalInCreation && !this.modalIncluir) {
      this.modalInCreation = true;
      this.modalIncluir = await ModalService.createFromJson('/modals/novoPedido', true);
      this.modalInCreation = false;
    }
  }

  public abreModalIncluir() {
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    if (grid) {
      this.criaModalIncluir().then(() => {
        this.modalIncluir.show();
      });
    }
  }

  public fechaModalIncluir() {
    this.modalIncluir.hide();
    this.modalIncluir.destroy();
    this.modalIncluir = null;
  }

  public filtroFilialServico({ component }: IEventParam<Select>) {
    const cdFilial = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const conditions = {
      CDFILIAL: cdFilial,
    };
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public confirmaNovoPedido() {
    const form = Metadata.getInstance<Form>('containerModalForm');
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    const row = form.value;
    row.CDFILIAL = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const [mes, ano] = row.DTMESMODAL.split('-');
    const dataDia = `01/${mes}/${ano}`;
    row.DTMESCARD = dataDia;
    LoadingService.show();
    Http.post('/verificaExisteCardapio', row).then((response) => {
      const data = response.data.dataset.verificaExisteCardapio;
      const existeCardapio = data[0].EXISTE_CARDAPIO;
      if (form.validate() && existeCardapio !== 0) {
        LoadingService.show();
        Http.post('/insereNovoPedido', row).then(() => {
          this.fechaModalIncluir();
          grid.datasource.get().then(() => {
            grid.reload();
          });
        }).finally(() => {
          LoadingService.hide();
          form.clearForm();
          textAreaJustificativa.value = '';
          textAreaJustificativa.readonly = true;
        });
      } else if (existeCardapio === 0) {
        DialogService.show({
          name: 'error-dialog',
          type: 'error',
          title: 'plaNext::naoIncluiuPedido',
          text: 'plaNext::naoExisteCardapio',
        });
      }
    }).finally(() => {
      LoadingService.hide();
    });
  }

  private configuraAlert(nameAlert, textAlert, colorAlert) {
    return {
      name: nameAlert,
      text: textAlert,
      color: colorAlert,
      top: true,
      left: false,
      timeout: 3000,
      buttonsSlot: [
        {
          name: 'undo-button',
          component: 'ZdButton',
          icon: true,
          iconName: 'close',
          color: 'white',
          events: {
            click: () => { AlertService.hide(); },
          },
        },
      ],
    };
  }

  public deletaPedido({ component }: IDictionary) {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    if (!row.DTINCLUSAO) {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    } else if (row.IDSITUACAO === 'E') {
      DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'plaNext::atencao',
        text: 'plaNext::desejaExcluir',
        buttons: [
          {
            name: 'no',
            component: 'ZdButton',
            label: 'plaNext::nao',
            events: { click: () => DialogService.hide() },
            outline: true,
          },
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'plaNext::sim',
            events: { click: () => this.callDeleteTree() },
          },
        ],
      });
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::noOrderInProgress', 'error'));
    }
  }

  private callDeleteTree() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    const rowDelete = {
      CDPEDALTCARD: row.CDPEDALTCARD,
      CDFILIAL: row.CDFILIAL,
    };
    LoadingService.show();
    Http.post('/deletaPedido', { requestType: 'Row', rowDelete }).then((response) => {
      if (response) {
        AlertService.show(this.configuraAlert('alert', 'plaNext::excluidoComSucesso', 'success'));
        grid.datasource.get().then(() => {
          grid.reload();
        });
      }
    }).catch((error: string) => {
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
      textAreaJustificativa.value = '';
      textAreaJustificativa.readonly = true;
    });
  }

  public configuraCampoJustificativa(evtObj) {
    const cardForms = Metadata.getInstance<Card>('cardForms');
    cardForms.isVisible = true;
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    const { currentRow } = grid.datasource;
    textAreaJustificativa.readonly = this.checkModeApproval() || evtObj.row.IDSITUACAO !== 'E';
    textAreaJustificativa.value = currentRow.DSJUSTICATIVA;
    const respostaPedido = Metadata.getInstance<Textarea>('respostaPedido');
    respostaPedido.value = currentRow.DSRESPOSTA;
    if (this.checkModeApproval()) {
      if (currentRow.IDRECEMCONCLUIDO === 'S') {
        this.setAsViewedCompleted(currentRow);
      }
    } else if (currentRow.IDRECEMAPROVREPR === 'S') {
      this.setAsViewedApDisap(currentRow);
    }
  }

  private configuraActions(actionPadrao, actionJustificativa) {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const toolbar = grid.toolbarSlot;
    toolbar.forEach((e: IComponentRender) => {
      if (e.name === 'incluirPedido') e.children[0].disabled = actionPadrao;
      if (e.name === 'excluirPedido') e.children[0].disabled = actionPadrao;
      if (e.name === 'concluirPedido') e.children[0].disabled = actionPadrao;
      if (e.name === 'solicitarNovamente') e.children[0].disabled = actionPadrao;
      if (e.name === 'aprovacaoReprovacaoPedido') e.children[0].disabled = actionPadrao;
      if (e.name === 'cancelButton') e.disabled = actionJustificativa;
      if (e.name === 'saveButton') e.disabled = actionJustificativa;
    });
  }

  public editandoJustificativa() {
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    if (textAreaJustificativa.readonly === false) {
      this.configuraActions(true, false);
    }
  }

  public cancelaEdicaoJustificativa() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const buttonInclusaoPedido = Metadata.getInstance<Button>('botaoIncluirPedido');
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    this.configuraActions(false, true);
    buttonInclusaoPedido.disabled = false;
    textAreaJustificativa.value = '';
    textAreaJustificativa.readonly = true;
    grid.datasource.get();
  }

  public updateJustificativa() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const textAreaJustificativa = Metadata.getInstance<Textarea>('justificativaPedido');
    const { currentRow } = grid.datasource;
    const valorJustificativa = textAreaJustificativa.value;
    const valoresUpdate = {
      row: currentRow,
      justificativa: valorJustificativa,
    };
    LoadingService.show();
    Http.post('/atualizaPedido', { requestType: 'Row', valoresUpdate }).then((response) => {
      if (response) {
        AlertService.show(this.configuraAlert('alert', 'plaNext::atualizadoComSucesso', 'success'));
      }
    }).catch((error: string) => {
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      const buttonInclusaoPedido = Metadata.getInstance<Button>('botaoIncluirPedido');
      this.configuraActions(false, true);
      buttonInclusaoPedido.disabled = false;
      textAreaJustificativa.value = '';
      textAreaJustificativa.readonly = true;
      grid.datasource.get();
    });
  }

  public async criaModalAlteracao() {
    if (!this.modalInCreation && !this.modalAlteracao) {
      this.modalInCreation = true;
      this.modalAlteracao = await ModalService.createFromJson('/modals/alteracaoCardapio', true);
      this.modalInCreation = false;
    }
  }

  public abreModalAlteracao({ row }) {
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    grid.datasource.currentRow = row;
    this.criaModalAlteracao().then(() => {
      this.modalAlteracao.show();
    });
  }

  public fechaModalAlteracao() {
    this.modalAlteracao.hide();
    this.modalAlteracao.destroy();
    this.modalAlteracao = null;
  }

  private getAllDaysInMonth(dateString: string): string[] {
    const [, month, year] = dateString.split('/').map(Number);
    const daysInMonth = new Date(year, month, 0).getDate();
    const days: string[] = [];
    const weekNames: string[] = [
      'DOM',
      'SEG',
      'TER',
      'QUA',
      'QUI',
      'SEX',
      'SAB',
    ];
    for (let dayIndex = 1; dayIndex <= daysInMonth; dayIndex += 1) {
      const dayOfWeek = new Date(year, month - 1, dayIndex).getDay();
      const formattedDay = `${weekNames[dayOfWeek]} ${String(dayIndex).padStart(2, '0')}`;
      days.push(formattedDay);
    }
    return days;
  }

  // MODAL DE CARDAPIO - CONFIGURAÇÃO E CRIAÇÃO
  private comensais = [];

  public configuraAlteracoesCardapio(row) {
    return Http.post('/verificaAlteracaoCardapio', { requestType: 'Row', row }).then((response) => {
      if (response) {
        if (this.dataCardapioAlteracao.length > 0) {
          this.dataCardapioAlteracao = [];
        }
        if (this.dataAlteracoesExistentes.length > 0) {
          this.dataAlteracoesExistentes = [];
        }
        this.dataCardapioAlteracao.push(response.data.dataset.dataCardapioAlteracao);
        this.dataAlteracoesExistentes.push(response.data.dataset.alteracoesExistentes);
      }
      return this.dataCardapioAlteracao;
    }).catch((error: string) => {
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    });
  }

  private dataCardapioAlteracao = [];

  private dataAlteracoesExistentes = [];

  public configureMenu() {
    this.configureMenuChanged();
  }

  public configureMenuChanged(newChanged = false) {
    LoadingService.show();
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    const { currentRow } = grid.datasource;
    const [data] = currentRow.DTMESCARD.split(' ');
    currentRow.DTMESCARD = data;
    currentRow.DTREFECNFMES = data;
    const gridMenu : GridEditable = Metadata.getInstance('tableCardapio');
    const dateMonth = this.getAllDaysInMonth(currentRow.DTREFECNFMES);
    Http.post('/carregaDadosCardapio', { requestType: 'Row', currentRow }).then((response) => {
      const responseData = response.data.dataset;
      const changedDishes = responseData.ChangedDishes;
      const changedDiners = responseData.ChangedDiners;
      const arrayColumns = responseData.ColunasCardapio;
      const arrayDiners = responseData.ComensaisCardapio;
      const arrayDishes = response.data.dataset.PratosCardapio;
      if (!newChanged) {
        gridMenu.columns = [];
        this.createColumns(gridMenu.columns, gridMenu, arrayColumns);
      }
      this.fillMenu(gridMenu, changedDiners, changedDishes, arrayDiners,
        dateMonth, gridMenu.columns, arrayDishes, currentRow);
    }).finally(() => LoadingService.hide());
  }

  public createColumns(gridColumns, gridMenu, arrayColumns) {
    gridColumns.push(new GridColumnEditable(
      {
        name: 'DIASCARDAPIO',
        label: 'plaNext::diaUpper',
      },
    ));
    gridColumns.push(new GridColumnEditable(
      {
        name: 'COMENSAISCARDAPIO',
        label: 'plaNext::comensaisUpper',
        conditions: {
          cssClass: '{{PedidoAlteracaoCardapioController.getConditionalCssClass}}',
        },
      },
    ));
    arrayColumns.sort((a, b) => {
      if (a.NRORDCNFMES <= b.NRORDCNFMES) {
        return -1;
      }
      return 1;
    });
    for (let i = 0; i < arrayColumns.length; i += 1) {
      const item = arrayColumns[i];
      const name = `${item.NMTIPOPRAT}_${item.NRSEQCNFMES}`;
      let label = item.NMTIPOPRAT;
      if (item.DSCOMPLTPMES) {
        label += `(${item.DSCOMPLTPMES})`;
      }
      if (item.IDOPCCNFMES === 'S') {
        label += '(OPÇÃO)';
      }
      gridColumns.push(new GridColumnEditable(
        {
          name,
          label,
          conditions: {
            cssClass: '{{PedidoAlteracaoCardapioController.getConditionalCssClass}}',
          },
        },
      ));
    }
  }

  public fillMenu(gridMenu, changedDiners, changedDishes, arrayDiners, dateMonth, gridColumns, arrayDishes, currentRow) {
    if (this.comensais.length > 0) {
      this.comensais = [];
    }
    const arrayColumns = [];
    gridMenu.datasource.updateData([]);
    gridColumns.forEach((g) => {
      if (g.name !== 'DIASCARDAPIO' && g.name !== 'COMENSAISCARDAPIO') {
        if (arrayColumns.includes(g.name) === false) {
          arrayColumns.push(
            {
              name: g.name,
              label: g.label,
            },
          );
        }
      }
    });
    dateMonth.forEach((e) => {
      type TipoColunas = typeof arrayColumns[number];
      type TipoPrato = { [key in TipoColunas]?: string };
      type TipoEspecial = {
        DIASCARDAPIO?: string;
        COMENSAISCARDAPIO?: string;
      };
      type NewRow = TipoPrato & TipoEspecial;
      const newRow: NewRow = {} as NewRow;
      newRow.DIASCARDAPIO = e;
      const dayWeek = e.substring(4);
      newRow.CDFILIAL = currentRow.CDFILIAL;
      this.fillDiners(arrayDiners, dayWeek, changedDiners, newRow);
      this.fillDishes(changedDishes, dayWeek, arrayDishes, arrayColumns, newRow);
      gridMenu.addNewRow(newRow);
    });
  }

  public fillDiners(arrayDiners, dayWeek, changedDiners, newRow) {
    const [changedDiner] = changedDiners.filter((item) => (item.DTCARDAPIO.split('/')[0] === dayWeek));
    arrayDiners.forEach((diner) => {
      const dayDiner = diner.DTCARDAPIO.split('/')[0];
      if (dayDiner === dayWeek && changedDiner) {
        this.fillChangedDiner(diner, changedDiner, newRow);
      } else if (dayDiner === dayWeek) {
        newRow.COMENSAISCARDAPIO = diner.QTCOMESTICAR;
      } else if (changedDiner) {
        newRow.CHANGED_COLUMNS = 'changedRecordMenu';
        newRow.COMENSAISCARDAPIO = changedDiner.QTCLIESTICAR + changedDiner.QTDOAESTICAR + changedDiner.QTFUNESTICAR;
      }
    });
    this.comensais.push(arrayDiners);
  }

  public fillDishes(changedDishes, dayWeek, arrayDishes, arrayColumns, newRow) {
    const arrayColumnsChangedDishes = [];
    arrayColumns.forEach((c) => {
      const parts = c.name.split('_');
      const nrSeq = parseInt(parts[1], 10);
      const [changedDish] = changedDishes.filter((item) => (
        item.DTCARDAPIO.split('/')[0] === dayWeek && item.NRSEQPRATCAR === nrSeq
      ));
      const dishCodeCol = `${c.name}_CD`;
      const dishPercentCol = `${c.name}_VR`;
      let dishLabel = null;
      let dishCode = null;
      let dishPercent = null;
      const dish = arrayDishes.find((d) => {
        const dishDay = d.DTCARDAPIO.substring(0, 2);
        const dishNrSeq = d.NRSEQPRATCAR;
        return (dishDay === dayWeek && dishNrSeq === nrSeq);
      });
      // If there is a dish for day and column
      if (dish) {
        dishLabel = `${Number(dish.VRPECONSCARD).toFixed(2)}% - ${dish.NMPRATO}`;
        dishCode = `${dish.CDFILAUXPRAT}_${dish.CDPRATO}`;
        dishPercent = dish.VRPECONSCARD;
      }
      if (changedDish) {
        dishLabel = `${Number(changedDish.VRPECONSCARD).toFixed(2)}% - ${changedDish.NMPRATO}`;
        dishCode = `${changedDish.CDFILAUXPRAT}_${changedDish.CDPRATO}`;
        dishPercent = changedDish.VRPECONSCARD;
        arrayColumnsChangedDishes.push(changedDish.NRSEQPRATCAR);
      }
      newRow[c.name] = dishLabel;
      newRow[dishCodeCol] = dishCode;
      newRow[dishPercentCol] = dishPercent;
    });
    newRow.CHANGED_COLUMNS_DISHES = arrayColumnsChangedDishes;
  }

  public fillChangedDiner(diner, changedDiner, newRow) {
    diner.QTCLIESTICAR = changedDiner.QTCLIESTICAR;
    diner.QTDOAESTICAR = changedDiner.QTDOAESTICAR;
    diner.QTFUNESTICAR = changedDiner.QTFUNESTICAR;
    diner.QTCOMESTICAR = changedDiner.QTCLIESTICAR + changedDiner.QTDOAESTICAR + changedDiner.QTFUNESTICAR;
    diner.QTESTITURNO1 = changedDiner.QTESTITURNO1;
    diner.QTESTITURNO2 = changedDiner.QTESTITURNO2;
    diner.QTESTITURNO3 = changedDiner.QTESTITURNO3;
    diner.QTESTITURNO4 = changedDiner.QTESTITURNO4;
    newRow.CHANGED_COLUMNS = 'changedRecordMenu';
    newRow.COMENSAISCARDAPIO = diner.QTCOMESTICAR;
  }

  public getConditionalCssClass({ row, column }: IDictionary<any>) {
    if (column.name === 'COMENSAISCARDAPIO' && row.CHANGED_COLUMNS) {
      return row.CHANGED_COLUMNS;
    }
    if (row.CHANGED_COLUMNS_DISHES) {
      const parts = column.name.split('_');
      if (parts.length > 1) {
        const nrSeq = parseInt(parts[1], 10);
        const [dish] = row.CHANGED_COLUMNS_DISHES.filter((item) => (item === nrSeq));
        if (dish) {
          return 'changedRecordMenu';
        }
      }
    }
    return '';
  }

  private findDayInDate(dtcardapio, dateMonth) {
    const day = dtcardapio.split(' ')[0].split('/')[0];
    return dateMonth.find((date) => date.includes(day));
  }

  // MODAL DE RESUMO - CONFIGURAÇÃO E CRIAÇÃO
  public async criaModalResumo() {
    if (!this.modalInCreation && !this.modalResumo) {
      this.modalInCreation = true;
      this.modalResumo = await ModalService.createFromJson('/modals/resumoAlteracaoCardapio', true);
      this.modalInCreation = false;
    }
  }

  public async resumeChanges() {
    LoadingService.show();
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    const { currentRow } = grid.datasource;
    const dateMonth = this.getAllDaysInMonth(currentRow.DTREFECNFMES);
    const changesByDay: {
      day: string,
      changes: {
        COMENSAISRESUMO?: string,
        PRATORESUMO?: string,
        COMENSAISDIAPRATORESUMO?: string
      }
    }[] = [];
    try {
      const response = await Http.post('/getChangesResume', { requestType: 'Row', currentRow });
      const changedDishes = response.data.dataset.ChangedDishes;
      const changedDiners = response.data.dataset.ChangedDiners;
      const oldDishes = response.data.dataset.OldDishes;
      const oldDiners = response.data.dataset.OldDiners;
      if (changedDishes.length > 0) {
        changedDishes.forEach((e) => {
          const day = this.findDayInDate(e.DTCARDAPIO, dateMonth);
          const dayEntry = {
            day,
            changes: {
              PRATORESUMO: '',
              COMENSAISDIAPRATORESUMO: '',
              COMENSAISRESUMO: '',
            },
          };
          changesByDay.push(dayEntry);
          oldDishes.forEach((dish) => {
            const change = `${dish.NMPRATO} => ${e.NMPRATO}`;
            dayEntry.changes.PRATORESUMO = change;
            if (dish.VRPECONSCARD !== e.VRPECONSCARD) {
              const changeCons = `${dish.VRPECONSCARD}% => ${e.VRPECONSCARD}%`;
              dayEntry.changes.COMENSAISDIAPRATORESUMO = changeCons;
            } else {
              const changeNoOld = `${e.VRPECONSCARD}%`;
              dayEntry.changes.COMENSAISDIAPRATORESUMO = changeNoOld;
            }
          });
        });
      }
      if (changedDiners.length > 0) {
        changedDiners.forEach((e) => {
          const day = this.findDayInDate(e.DTCARDAPIO, dateMonth);
          const dayEntry = {
            day,
            changes: {
              PRATORESUMO: '',
              COMENSAISDIAPRATORESUMO: '',
              COMENSAISRESUMO: '',
            },
          };
          changesByDay.push(dayEntry);
          const dataCardapio = e.DTCARDAPIO.split(' ')[0];
          oldDiners.forEach((diner) => {
            if (diner.DTCARDAPIO === dataCardapio) {
              const totalOldDiners = diner.QTCLIESTICAR + diner.QTFUNESTICAR + diner.QTDOAESTICAR;
              const totalNewDiners = e.QTCLIESTICAR + e.QTFUNESTICAR + e.QTDOAESTICAR;
              const change = `${totalOldDiners} => ${totalNewDiners}`;
              dayEntry.changes.COMENSAISRESUMO = change;
            } else {
              const changeWithoutOld = `${e.QTCLIESTICAR + e.QTFUNESTICAR + e.QTDOAESTICAR}`;
              dayEntry.changes.COMENSAISRESUMO = changeWithoutOld;
            }
          });
        });
      }
    } finally {
      LoadingService.hide();
    }
    return changesByDay;
  }

  public async abreModalResumo() {
    const changes = await this.resumeChanges();
    if (changes.length > 0) {
      this.criaModalResumo().then(() => {
        this.modalResumo.show();
      });
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::anyoneChange', 'error'));
    }
  }

  public fechaModalResumo() {
    this.modalResumo.hide();
    this.modalResumo.destroy();
    this.modalResumo = null;
  }

  public async loadDataResume() {
    const grid: GridEditable = Metadata.getInstance('gridResumoAlteracao');
    const changes = await this.resumeChanges();
    if (changes.length > 0) {
      changes.forEach((e) => {
        const newRow = {
          DIARESUMO: e.day,
          COMENSAISRESUMO: e.changes.COMENSAISRESUMO,
          COMENSAISDIAPRATORESUMO: e.changes.COMENSAISDIAPRATORESUMO,
          PRATORESUMO: e.changes.PRATORESUMO,
        };
        grid.addNewRow(newRow);
        grid.datasource.data.sort((a, b) => {
          const getDay = (dayString: string) => Number(dayString.split(' ')[1]);
          const monthOrder = ['DOM', 'SEG', 'TER', 'QUA', 'QUI', 'SEX', 'SAB'];
          const dayA = getDay(a.DIARESUMO);
          const dayB = getDay(b.DIARESUMO);
          if (dayA !== dayB) {
            return dayA - dayB;
          }
          return monthOrder.indexOf(a.DIARESUMO.split(' ')[0]) - monthOrder.indexOf(b.DIARESUMO.split(' ')[0]);
        });
      });
    }
  }

  // MODAL DE COMENSAIS - CONFIGURAÇÃO E CRIAÇÃO
  public async criaModalAlteraCellComensais() {
    if (!this.modalInCreation && !this.modalAlteraCellComensais) {
      this.modalInCreation = true;
      this.modalAlteraCellComensais = await ModalService.createFromJson('/modals/alteraCellComensais', true);
      this.modalInCreation = false;
    }
  }

  private obterValorDiaSemana(string) {
    const diasDaSemana = {
      DOM: 0,
      SEG: 1,
      TER: 2,
      QUA: 3,
      QUI: 4,
      SEX: 5,
      SAB: 6,
    };
    const dia = string.substring(0, 3);
    return diasDaSemana[dia] !== undefined ? diasDaSemana[dia] : -1;
  }

  // MODAL DE COMENSAIS - CONFIGURAÇÃO
  public configuraCelulaComensais(row) {
    const currentRow = row;
    // Do not permit changes in day without diner
    if (!currentRow || !(currentRow.COMENSAISCARDAPIO > 0)) {
      return;
    }
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    const currentRowGrid = grid.datasource.currentRow;
    const dtMesCardapio = currentRowGrid.DTMESCARD.substring(2, 10);
    const diaCardapio = currentRow.DIASCARDAPIO.substring(4);
    currentRowGrid.DTDIACARD = diaCardapio + dtMesCardapio;
    const diaDaSemana = this.obterValorDiaSemana(currentRow.DIASCARDAPIO);
    currentRowGrid.DIADASEMANA = diaDaSemana;
    Http.post('/turnosPontoEntrega', { requestType: 'Row', currentRowGrid }).then((response) => {
      const comensaisArray = this.comensais[0];
      const turnosPontoEntregaArray = [];
      const turnos = response.data.dataset.turnosCardaDia;
      const pontoEntrega = response.data.dataset.pontosDeEntrega;
      const formTurno = Metadata.getInstance<Form>('formAlteraTurnos');
      const formPontoEntrega = Metadata.getInstance<Form>('formAlteraPontoEntrega');
      const clientesComensais = Metadata.getInstance<NumberInput>('clientesAlteraComensais');
      turnosPontoEntregaArray.push({
        turnos,
      });
      turnosPontoEntregaArray.push({
        pontoEntrega,
      });
      if (turnos.length > 0) {
        formTurno.isVisible = true;
      }
      if (pontoEntrega.length > 0) {
        formPontoEntrega.isVisible = false;
      }
      if (formPontoEntrega.isVisible === false) {
        clientesComensais.readonly = false;
      } else {
        clientesComensais.readonly = true;
      }
      const turnosArray = [];
      const pontoEntregaArray = [];
      turnosPontoEntregaArray.forEach((item) => {
        if (item.turnos) {
          turnosArray.push(...item.turnos);
        }
        if (item.pontoEntrega) {
          pontoEntregaArray.push(...item.pontoEntrega);
        }
      });
      comensaisArray.forEach((item) => {
        if (item.DTCARDAPIO === currentRowGrid.DTDIACARD) {
          const funcionariosComensais = Metadata.getInstance<NumberInput>('funcionariosAlteraComensais');
          const doacoesComensais = Metadata.getInstance<NumberInput>('doacoesAlteraComensais');
          const totalComensais = Metadata.getInstance<NumberInput>('totalAlteraComensais');
          clientesComensais.value = item.QTCLIESTICAR;
          funcionariosComensais.value = item.QTFUNESTICAR;
          doacoesComensais.value = item.QTDOAESTICAR;
          totalComensais.value = item.QTCOMESTICAR;
          if (item.ALTERACAOATIVA) {
            Object.keys(item).forEach((key) => {
              if (key.startsWith('QTESTITURNO') && item[key] !== null) {
                const turnoNumber = key.slice(-1);
                turnosArray.forEach((turno) => {
                  if (turno.CDTURNO === turnoNumber) {
                    turno.COMENSAIS = item[key];
                  }
                });
              }
            });
          }
        }
      });
      const qtdeTurnos = turnosArray.length;
      const colsGrid = 12 / qtdeTurnos;
      turnosArray.forEach((item) => {
        const child = {
          name: item.NMTURNO + item.CDTURNO,
          component: 'ZdNumber',
          label: item.NMTURNO,
          value: item.COMENSAIS,
          grid: {
            cols: colsGrid,
          },
          mask: {
            decimalPlaces: 0,
          },
        };
        formTurno.addChild(child);
      });
      pontoEntregaArray.forEach((item) => {
        const child = {
          name: item.NMDESTFILI + item.CDDESTFILI,
          component: 'ZdNumber',
          label: item.NMDESTFILI,
          value: item.QTCOMESTICLI,
          mask: {
            decimalPlaces: 0,
          },
        };
        formPontoEntrega.addChild(child);
      });
    });
    this.criaModalAlteraCellComensais().then(() => {
      this.modalAlteraCellComensais.show();
    });
  }

  public fechaModalAlteraCellComensais() {
    this.modalAlteraCellComensais.hide();
    this.modalAlteraCellComensais.destroy();
    this.modalAlteraCellComensais = null;
  }

  private rowComensais = {
    clientesComensais: 0,
    funcionariosComensais: 0,
    doacoesComensais: 0,
    totalComensais: 0,
  };

  public configuraCamposComensais() {
    const clientesComensais = Metadata.getInstance<NumberInput>('clientesAlteraComensais');
    const funcionariosComensais = Metadata.getInstance<NumberInput>('funcionariosAlteraComensais');
    const doacoesComensais = Metadata.getInstance<NumberInput>('doacoesAlteraComensais');
    const totalComensais = Metadata.getInstance<NumberInput>('totalAlteraComensais');
    if (clientesComensais.value + funcionariosComensais.value + doacoesComensais.value !== totalComensais.value) {
      totalComensais.value = clientesComensais.value + funcionariosComensais.value + doacoesComensais.value;
    }
    this.rowComensais = {
      clientesComensais: clientesComensais.value,
      funcionariosComensais: funcionariosComensais.value,
      doacoesComensais: doacoesComensais.value,
      totalComensais: totalComensais.value,
    };
  }

  private events: any = {
    click: () => DialogService.hide(),
  };

  // FUNÇÃO DE CONFIRMAR ALTERAÇÃO COMENSAIS
  public confirmaAlteraComensais() {
    const formTurno = Metadata.getInstance<Form>('formAlteraTurnos');
    const formPontoEntrega = Metadata.getInstance<Form>('formAlteraPontoEntrega');
    const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
    const currentRowGrid = grid.datasource.currentRow;
    const rowTurno = formTurno.value;
    const rowPontoEntrega = formPontoEntrega.value;
    const quantidadeTurnos = [];
    const quantidadePontosEntrega = [];
    this.configuraCamposComensais();
    const { rowComensais } = this;
    Object.keys(rowTurno).forEach((key) => {
      const lastChar = key.slice(-1);
      rowTurno[lastChar] = rowTurno[key];
      if (/[a-zA-Z]/.test(key)) {
        quantidadeTurnos.push(rowTurno[key]);
      }
    });
    Object.keys(rowPontoEntrega).forEach((key) => {
      quantidadePontosEntrega.push(rowPontoEntrega[key]);
    });
    const somaTurnos = quantidadeTurnos.reduce((acumulador, valorAtual) => acumulador + valorAtual, 0);
    if ((Object.keys(rowTurno).length > 0) && (somaTurnos !== rowComensais.totalComensais)) {
      DialogService.show({
        name: 'error-dialog',
        type: 'error',
        title: 'plaNext::erro',
        text: 'plaNext::erroSomaComensaisTurnos',
      });
      return;
    }
    const valoresUpdate = {
      rowComensais,
      turnos: rowTurno,
      row: currentRowGrid,
    };
    LoadingService.show();
    Http.post('/alteraComensaisTurno', { requestType: 'Row', valoresUpdate }).then(() => {
      this.configureMenuChanged(true);
    }).catch((error: string) => {
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      this.fechaModalAlteraCellComensais();
      LoadingService.hide();
    });
    // const somaPontosEntrega = quantidadePontosEntrega.reduce((acumulador, valorAtual) => acumulador + valorAtual, 0);
    // const somaPontosEntregaString = `${somaPontosEntrega}`;
  }

  public confirmChangedDish() {
    const formDish = Metadata.getInstance<Form>('modalChangedDish');
    const gridMenu : GridEditable = Metadata.getInstance('tableCardapio');
    const { currentRow } = gridMenu.datasource;
    const dinerMenu = currentRow.COMENSAISCARDAPIO;
    if (dinerMenu && dinerMenu !== null && dinerMenu !== undefined) {
      if (formDish.validate()) {
        const grid: Grid = Metadata.getInstance('gridAlteracaoCardapio');
        const currentRowGrid = grid.datasource.currentRow;
        const rowDish = formDish.value;
        rowDish.VRPECONSCARD = rowDish.totalComensaisPrato;
        const dtMesCardapio = currentRowGrid.DTMESCARD.substring(2, 10);
        const diaCardapio = currentRow.DIASCARDAPIO.substring(4);
        rowDish.DTDIACARD = diaCardapio + dtMesCardapio;
        currentRowGrid.DTDIACARD = rowDish.DTDIACARD;
        rowDish.NRSEQPRATCAR = this.columnDish;
        const valoresUpdate = {
          dish: rowDish,
          row: currentRowGrid,
          CDNVPRATO: this.initialValues.CDNVPRATO,
        };
        LoadingService.show();
        Http.post('/changedDish', { requestType: 'Row', valoresUpdate }).then(() => {
          this.configureMenuChanged(true);
          AlertService.show(this.configuraAlert('alert', 'plaNext::atualizadoComSucesso', 'success'));
          this.fechaModalAlteraPratos();
        }).catch((e) => {
          let { error } = e.response.data;
          error = error.substr(11);
          AlertService.show(this.configuraAlert('alertError', error, 'error'));
        }).finally(() => {
          LoadingService.hide();
        });
      }
    } else {
      AlertService.show(this.configuraAlert('alertError', 'Não há comensais cadastrados para este dia!', 'error'));
    }
  }

  public async criaModalAlteraPratos() {
    if (!this.modalInCreation && !this.modalAlteraPratos) {
      this.modalInCreation = true;
      this.modalAlteraPratos = await ModalService.createFromJson('/modals/alteraCellPratos', true);
      this.modalInCreation = false;
    }
  }

  private rowPratos = [];

  private columnDish = 0;

  // FUNÇÃO QUE VERIFICA ONDE FOI REALIZADO O CLICK DA CELULA
  public abreCellClick({ column, row }: any) {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const rowGridChangedMenu = grid.datasource.currentRow;
    if (!this.checkModeApproval() && rowGridChangedMenu.IDSITUACAO === 'E') {
      if (column.name === 'COMENSAISCARDAPIO') {
        this.configuraCelulaComensais(row);
      } else if (column.name === 'DIASCARDAPIO') {
        // Não abre nenhum modal.
      } else {
        this.abreModalAlteraPratos();
        if (this.rowPratos.length > 0) {
          this.rowPratos = [];
        }
        this.rowPratos.push(row);
        this.getCodPrato(column.name, row);
        this.getComensalPrato(column.name, row);
        const parts = column.name.split('_');
        this.columnDish = parseInt(parts[1], 10);
      }
    }
  }

  // MODAL DE PRATOS - CONFIGURAÇÃO E CRIAÇÃO
  public abreModalAlteraPratos() {
    this.criaModalAlteraPratos().then(() => {
      this.modalAlteraPratos.show();
    });
  }

  private codPratoRecorrente = '';

  private getCodPrato(columnName, row) {
    if (Object.prototype.hasOwnProperty.call(row, columnName)) {
      const chaveCD = `${columnName}_CD`;
      this.codPratoRecorrente = row[chaveCD];
    }
    return null;
  }

  private comPratoRecorrente = '';

  private getComensalPrato(columnName, row) {
    const percent = row[`${columnName}_VR`];
    this.comPratoRecorrente = percent;
  }

  public configuraModalAlteraPratos() {
    const selectPratos = Metadata.getInstance<Select>('selectPratoModal');
    const totalComensais = Metadata.getInstance<NumberInput>('totalComensaisPrato');
    const cdFilial = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    selectPratos.datasource.filter.CDNVPRATO = this.initialValues.CDNVPRATO;
    selectPratos.datasource.filter.CDFILIAL = cdFilial;
    const valorCdPratoRecorrente = this.codPratoRecorrente;
    const valorComPratoRecorrente = this.comPratoRecorrente;
    totalComensais.value = valorComPratoRecorrente;
    selectPratos.value = valorCdPratoRecorrente;
  }

  public fechaModalAlteraPratos() {
    this.modalAlteraPratos.hide();
    this.modalAlteraPratos.destroy();
    this.modalAlteraPratos = null;
  }

  public async createCompleteOrderModal() {
    if (!this.modalInCreation && !this.completeOrderModal) {
      this.modalInCreation = true;
      this.completeOrderModal = await ModalService.createFromJson('/modals/completeOrder', true);
      this.modalInCreation = false;
    }
  }

  public openCompleteOrderModal() {
    if (this.requireEmailInput(this.openCompleteOrderModal)) {
      return;
    }
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    if (Object.keys(row).length > 0) {
      if (row.IDSITUACAO === 'E') {
        this.createCompleteOrderModal().then(() => {
          this.completeOrderModal.show();
        });
      } else {
        AlertService.show(this.configuraAlert('alertError', 'plaNext::orderInProgress', 'error'));
      }
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    }
  }

  public completeOrder() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    const form = Metadata.getInstance<Form>('formCompleteOrder');
    const { extraRequest } = form.value;
    const sendObject = {
      CURRENTROW: row,
      EXTRAREQUEST: extraRequest,
      JUSTIFICATION: I18n.translate('plaNext::aprovAutoNoDinerAlt'),
    };
    LoadingService.show();
    Http.post('/completeOrder', {
      requestType: 'Row',
      row: sendObject,
    }).then((response) => {
      this.configuraGridAlteracao();
      AlertService.show(this.configuraAlert('alert', 'plaNext::completeOrderSuccess', 'success'));
      if (response.data && response.data.dataset && response.data.dataset.completeOrder) {
        const { operation } = response.data.dataset.completeOrder;
        this.sendEmail(row.NRPEDALTCARD, operation);
      }
    }).catch((e) => {
      let { error } = e.response.data;
      error = error.substr(11);
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      this.completeOrderModal.hide();
      this.completeOrderModal.destroy();
      this.completeOrderModal = null;
    });
  }

  confirmRequestAgain() {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    if (Object.keys(row).length > 0) {
      if (row.IDSITUACAO === 'R') {
        DialogService.show({
          name: 'dialogCompleteOrder',
          type: 'warning',
          title: 'plaNext::solicitacao',
          text: 'plaNext::confirmRequestChanged',
          buttons: [
            {
              name: 'no',
              component: 'ZdButton',
              label: 'plaNext::nao',
              events: {
                click: () => DialogService.hide(),
              },
              outline: true,
            },
            {
              name: 'yes',
              component: 'ZdButton',
              label: 'plaNext::sim',
              events: {
                click: () => this.requestAgain(row),
              },
            },
          ],
        });
      } else {
        AlertService.show(this.configuraAlert('alertError', 'plaNext::requestRejected', 'error'));
      }
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    }
  }

  public requestAgain(currentRow) {
    LoadingService.show();
    Http.post('/requestAgain', {
      requestType: 'Row',
      row: currentRow,
    }).then(() => {
      this.configuraGridAlteracao();
      AlertService.show(this.configuraAlert('alert', 'plaNext::requestSuccessfullySubmitted', 'success'));
    }).catch((e) => {
      let { error } = e.response.data;
      error = error.substr(11);
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
    });
  }

  public async createApproveDisapproveModal() {
    if (!this.modalInCreation && !this.approveDisapproveModal) {
      this.modalInCreation = true;
      this.approveDisapproveModal = await ModalService.createFromJson('/modals/approveDisapprove', true);
      this.modalInCreation = false;
    }
  }

  public openApproveDisapproveModal() {
    if (this.requireEmailInput(this.openApproveDisapproveModal)) {
      return;
    }
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    if (Object.keys(row).length > 0) {
      if (row.IDSITUACAO === 'C') {
        const pageApproval = this.checkModeApproval();
        const apGO = row.APPROVALGO;
        if (!pageApproval && row.IDSOLEXTRA === 'N') {
          AlertService.show(this.configuraAlert('alertError', 'plaNext::noExtraRequest', 'error'));
        } else if (pageApproval && (apGO === null || apGO === ' ') && row.IDSOLEXTRA === 'N') {
          this.createApproveDisapproveModal().then(() => {
            this.approveDisapproveModal.show();
          });
        } else if ((!pageApproval && (apGO === null || apGO === ' ')) || (pageApproval && (apGO !== null && apGO !== ' '))) {
          const cdFilial = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
          // Checks if current operator is GO of unit
          Http.post('/isOperatorGO', {
            requestType: 'Row',
            row: { CDFILIAL: cdFilial },
          }).then((response) => {
            if (!response.data.dataset.isOperatorGO.isGO) {
              AlertService.show(this.configuraAlert('alertError', 'plaNext::isNoGO', 'error'));
            } else {
              this.createApproveDisapproveModal().then(() => {
                this.approveDisapproveModal.show();
              });
            }
          });
        } else if (pageApproval && (apGO === null || apGO === ' ') && row.IDSOLEXTRA === 'S') {
          AlertService.show(this.configuraAlert('alertError', 'plaNext::approvalRequiredGO', 'error'));
        } else if (!pageApproval && (apGO !== null && apGO !== ' ')) {
          AlertService.show(this.configuraAlert('alertError', 'plaNext::existApproveGO', 'error'));
        }
      } else {
        AlertService.show(this.configuraAlert('alertError', 'plaNext::completedOrders', 'error'));
      }
    } else {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::nenhumRegSelecionado', 'error'));
    }
  }

  get titleTextPage() {
    let titleText = '';
    if (this.checkModeApproval()) {
      titleText = 'plaNext::pedidoAlteracaoCardapio';
    } else {
      titleText = 'plaNext::pedidoAlteracaoCardapioGO';
    }
    return titleText;
  }

  get titleText() {
    let titleText = '';
    if (this.checkModeApproval()) {
      titleText = 'plaNext::approveDisapprove';
    } else {
      titleText = 'plaNext::approveDisapproveGO';
    }
    return titleText;
  }

  public confirmApproveDisapprove({ component }) {
    const valueFormJustification = Metadata.getInstance<Textarea>('justification').value;
    let textConfirm = '';
    let approve = '';
    if (!valueFormJustification || valueFormJustification.length <= 0) {
      AlertService.show(this.configuraAlert('alertError', 'plaNext::fillJustification', 'error'));
      return;
    }
    if (component.name === 'approve') {
      textConfirm = 'plaNext::approve';
      approve = 'A';
    } else {
      textConfirm = 'plaNext::disapprove';
      approve = 'R';
    }
    DialogService.show({
      name: 'dialogCompleteOrder',
      type: 'warning',
      text: textConfirm,
      buttons: [
        {
          name: 'no',
          component: 'ZdButton',
          label: 'plaNext::nao',
          events: {
            click: () => DialogService.hide(),
          },
          outline: true,
        },
        {
          name: 'yes',
          component: 'ZdButton',
          label: 'plaNext::sim',
          events: {
            click: () => this.approveDisapprove(approve, valueFormJustification),
          },
        },
      ],
    });
  }

  public approveDisapprove(approve, valueFormJustification) {
    const grid = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    const row = grid.datasource.currentRow;
    const sendObject = {
      CURRENTROW: row,
      APPROVE: approve,
      JUSTIFICATION: valueFormJustification,
      APPROVALPAGE: this.checkModeApproval(),
    };
    let operation = 'aprove';
    if (this.checkModeApproval()) {
      operation = (approve === 'A') ? 'aprov' : 'reprov';
    } else {
      operation = (approve === 'A') ? 'aprovOM' : 'reprov';
    }
    LoadingService.show();
    Http.post('/approveDisapprove', {
      requestType: 'Row',
      row: sendObject,
    }).then(() => {
      this.configuraGridAlteracao();
      AlertService.show(this.configuraAlert('alert', 'plaNext::requestSuccessfullySubmitted', 'success'));
      this.sendEmail(row.NRPEDALTCARD, operation);
    }).catch((e) => {
      let { error } = e.response.data;
      error = error.substr(11);
      AlertService.show(this.configuraAlert('alertError', error, 'error'));
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
      this.approveDisapproveModal.hide();
      this.approveDisapproveModal.destroy();
      this.approveDisapproveModal = null;
    });
  }

  public pedAltCardColors(row: any) {
    const page = Metadata.getInstance<Card>('pedidoAlteracaoCardapio');
    const approvalMode = page.cssClass === 'APPROVAL';
    let lineColor = 'inherit';
    if (row.IDSITUACAO === 'A') {
      lineColor = 'green';
    } else if (row.IDSITUACAO === 'R') {
      lineColor = 'red';
    }
    let fontWeight = 'normal';
    if ((!approvalMode && row.IDRECEMAPROVREPR === 'S') || (approvalMode && row.IDRECEMCONCLUIDO === 'S')) {
      fontWeight = 'bold';
    }
    return { cssStyle: { color: lineColor, fontWeight } };
  }

  public refreshGrid() {
    const cardForms = Metadata.getInstance<Card>('cardForms');
    cardForms.isVisible = false;
    const gridAlteracao = Metadata.getInstance<Grid>('gridAlteracaoCardapio');
    gridAlteracao.datasource.get();
  }

  public closeCompleteOrderModal() {
    this.completeOrderModal.hide();
    this.completeOrderModal.destroy();
    this.completeOrderModal = null;
  }

  public destroySheetModals() {
    if (this.modalIncluir) {
      this.modalIncluir.destroy();
    }
    if (this.modalAlteracao) {
      this.modalAlteracao.destroy();
    }
    if (this.modalResumo) {
      this.modalResumo.destroy();
    }
    if (this.modalAlteraCellComensais) {
      this.modalAlteraCellComensais.destroy();
    }
    if (this.modalAlteraPratos) {
      this.modalAlteraPratos.destroy();
    }
    if (this.completeOrderModal) {
      this.completeOrderModal.destroy();
    }
    if (this.approveDisapproveModal) {
      this.approveDisapproveModal.destroy();
    }
    if (this.emailModal) {
      this.emailModal.destroy();
    }
  }

  public closeAproveModal() {
    this.approveDisapproveModal.hide();
    this.approveDisapproveModal.destroy();
    this.approveDisapproveModal = null;
  }

  public setAsViewedCompleted(row) {
    Http.post('/setAsViewedCompleted', {
      requestType: 'Row',
      row: { NRPEDALTCARD: row.NRPEDALTCARD },
    }).then(() => {
      row.IDRECEMCONCLUIDO = 'N';
    });
  }

  public setAsViewedApDisap(row) {
    Http.post('/setAsViewedApDisap', {
      requestType: 'Row',
      row: { NRPEDALTCARD: row.NRPEDALTCARD },
    }).then(() => {
      row.IDRECEMAPROVREPR = 'N';
    });
  }

  public async openEmailModal() {
    if (!this.modalInCreation && !this.emailModal) {
      this.modalInCreation = true;
      this.emailModal = await ModalService.createFromJson('/modals/pedAltCardEmail', true);
      this.modalInCreation = false;
      this.emailModal.show();
    }
  }

  public initializeEmailField() {
    const emailField = Metadata.getInstance<Textarea>('email');
    emailField.value = this.emailPlan;
  }

  public closeEmailModal() {
    this.emailModal.hide();
    this.emailModal.destroy();
    this.emailCallback = null;
  }

  public getEmailPlan() {
    const cdfilial = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    return Http.post('/getEmailPlan', {
      requestType: 'Row',
      row: {
        CDFILIAL: cdfilial,
      },
    }).then((response) => {
      if (response && response.data && response.data.dataset) {
        const { emailPlan } = response.data.dataset;
        if (emailPlan) {
          this.emailPlan = emailPlan.DSEMAIL;
        } else {
          this.emailPlan = '';
        }
      }
      return {};
    });
  }

  public saveEmailPlan() {
    const cdfilial = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const form = Metadata.getInstance<Form>('pedAltCardEmailForm');
    const dsemail = Metadata.getInstance<Textarea>('email').value;
    if (form.validate()) {
      Http.post('/setEmailPlan', {
        requestType: 'Row',
        row: {
          CDFILIAL: cdfilial,
          DSEMAIL: dsemail,
        },
      }).then(() => {
        const callBack = this.emailCallback;
        this.emailPlan = dsemail;
        AlertService.show(this.configuraAlert('alert', 'plaNext::operationSuccess', 'success'));
        this.closeEmailModal();
        if (callBack !== null) {
          callBack();
        }
      });
    }
  }

  public requireEmailInput(emailCallback) {
    if (this.checkModeApproval()) {
      return false;
    }
    if (!this.emailPlan) {
      this.openEmailModal();
      this.emailCallback = null;
      if (emailCallback) {
        this.emailCallback = emailCallback.bind(this);
      }
      return true;
    }
    return false;
  }

  public async sendEmail(nrPedAltCard, operation) {
    const msgs = {
      conc: I18n.translate('plaNext::pedAltCardMailConc'),
      concAprovAuto: I18n.translate('plaNext::pedAltCardMailConcAprovAuto'),
      concNeedOM: I18n.translate('plaNext::pedAltCardMailConcNeedOM'),
      aprovOM: I18n.translate('plaNext::pedAltCardMailAprovOM'),
      aprov: I18n.translate('plaNext::pedAltCardMailAprov'),
      reprov: I18n.translate('plaNext::pedAltCardMailReprov'),
      requestLabel: I18n.translate('plaNext::pedAltCardMailRequestLabel'),
      creationLabel: I18n.translate('plaNext::pedAltCardMailCreationLabel'),
      unitLabel: I18n.translate('plaNext::pedAltCardMailUnitLabel'),
      serviceLabel: I18n.translate('plaNext::pedAltCardMailServiceLabel'),
      monthLabel: I18n.translate('plaNext::pedAltCardMailMonthLabel'),
      approveLabel: I18n.translate('plaNext::pedAltCardMailApproveLabel'),
      approveGOLabel: I18n.translate('plaNext::pedAltCardMailApproveGOLabel'),
      justifyLabel: I18n.translate('plaNext::pedAltCardMailJustifyLabel'),
      responseLabel: I18n.translate('plaNext::pedAltCardMailResponseLabel'),
      sendByTecfood: I18n.translate('plaNext::sendByTecfood'),
      concluido: I18n.translate('plaNext::concluido'),
      emAndamento: I18n.translate('plaNext::emAndamento'),
      aprovado: I18n.translate('plaNext::aprovado'),
      reprovado: I18n.translate('plaNext::reprovado'),
    };

    const dates = await Http.post('/getDatesToTranslate', {
      requestType: 'Row',
      row: {
        NRPEDALTCARD: nrPedAltCard,
      },
    }).then((res) => res.data.dataset.dates);
    dates.DTPEDALTCARD = Utilities.dateTimeString(dates.DTPEDALTCARD);
    dates.DTMESCARD = Utilities.monthString(dates.DTMESCARD);
    dates.DTAPROVREPR = Utilities.dateTimeString(dates.DTAPROVREPR);
    dates.DTAPROVREPRGO = Utilities.dateTimeString(dates.DTAPROVREPRGO);
    Http.post('/sendEmailPedAltCard', {
      requestType: 'Row',
      row: {
        NRPEDALTCARD: nrPedAltCard,
        OPERATION: operation,
        msgs,
        dates,
      },
    });
  }
}
