import {
  Http,
  Metadata,
  IEventParam,
} from '@zeedhi/core';
import {
  Form,
  Grid,
  Card,
  Select,
  Tabs,
  Button,
  GridEditable,
  DialogService,
  Number,
  Row,
  AlertService,
} from '@zeedhi/common';
import { ZdTabs } from '@zeedhi/vuetify';

export class RetiradaPlanejamentoController {
  // Variavel para desabilitar ou habilitar botões edit in line.
  public isDisabledButtonCardProd = true;

  // Variaveis para saber se os component's grid da aba lancamento foram montados.
  public estadoGridTipoPrato = false;

  public estadoGridResumo = false;

  public estadoGridProduto = false;

  // Variaveis para montar a legenda da aba lancamento.
  public gridNrretirada = '';

  public gridCdservfili = '';

  public gridDtretirada = '';

  public gridDtcardapio = '';

  // Variavel com dados da unidade selecionada.
  public paramUnid;

  // Variaveis manipula disabled das ações dos grids.
  public bloqTodasAcoes = false;

  public bloqAcoesAdicionar = false;

  public novaRetirada = false;

  public async getDadosIniciais() {
    Http.post('/getDadosIniciais').then((response) => {
      const data = response.data.dataset.dadosIniciais;
      localStorage.setItem('CDOPERADOR', data.CDOPERADOR);
      localStorage.setItem('NRORG', data.NRORG);
      localStorage.setItem('FILIAL_LOGADA', data.FILIAL_LOGADA);
    });
  }

  public filtroFilialFilioper() {
    const filialFiltro = Metadata.getInstance<Select>('FILIAL_FILTRO');
    const cdoperador = localStorage.getItem('CDOPERADOR');
    const nrorg = localStorage.getItem('NRORG');
    const conditions = {
      CDOPERADOR: cdoperador,
      NRORG: nrorg,
    };
    filialFiltro.datasource.filter = conditions;
    filialFiltro.datasource.get();
    filialFiltro.value = localStorage.getItem('FILIAL_LOGADA');
  }

  public filtroFilialServico({ component }: IEventParam<Select>) {
    const cdFilial = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const conditions = {
      CDFILIAL: cdFilial,
    };
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  // Carrega dados de bloqueio e informações da Unidade.
  public async carregaParametrosUnidade() {
    const cdFilial = Metadata.getInstance<Select>('FILIAL_FILTRO').value;
    const formFilter = Metadata.getInstance<Form>('retiradaPlanejamentoForm');
    formFilter.value.FILTRO_SERVICO = '';
    formFilter.value.DATA_RETIRADA = '';
    const data = {
      CDFILIAL: cdFilial,
    };
    Http.post('/carregaParametrosUnidade', data).then((response) => {
      const parametros = response.data.dataset.parametros[0];
      const idctrlestq = parametros.IDCTRLESTQ;
      const idutilalmox = parametros.IDUTILALMOX;
      const idutililote = parametros.IDUTILLOTE;
      parametros.VALIDAR_ESTOQUE = idctrlestq !== 'N' && (idutilalmox === 'S' || idutililote === 'S');
      this.paramUnid = JSON.parse(JSON.stringify(parametros));
      const diaDtprocessa = this.paramUnid.DTPROCESSA.substring(0, 2);
      const mesDtprocessa = this.paramUnid.DTPROCESSA.substring(3, 5) - 1;
      const anoDtprocessa = this.paramUnid.DTPROCESSA.substring(6, 10);
      const diasApos = new Date(anoDtprocessa, mesDtprocessa, diaDtprocessa);
      const diasAntes = new Date(anoDtprocessa, mesDtprocessa, diaDtprocessa);
      diasAntes.setDate(diasAntes.getDate() - this.paramUnid.NRDIASSAIDFIL);
      diasApos.setDate(diasApos.getDate() + this.paramUnid.NRDIASSAIDPDFIL);
      this.paramUnid.DIAS_ANTES_DTPROCESSA = diasAntes;
      this.paramUnid.DIAS_APOS_DTPROCESSA = diasApos;
      formFilter.value.DATA_RETIRADA = [parametros.DTPROCESSA, parametros.DTPROCESSA];
      formFilter.value.DATA_CARDAPIO = [parametros.DTPROCESSA, parametros.DTPROCESSA];
    });
  }

  // Aplica filtro inicial da tela (Romaneio)
  public async aplicaFiltroRetiradaPlanejamento() {
    const grid = Metadata.getInstance<Grid>('retiradaPlanejamentoGrid');
    const cardRetiradas = Metadata.getInstance<Card>('cardRetiradas');
    if (this.paramUnid.IDCTRLESTQ !== 'N' && (this.paramUnid.IDUTILALMOX === 'S' || this.paramUnid.IDUTILLOTE === 'S')) {
      DialogService.show({
        name: 'error-dialog',
        type: 'error',
        title: 'plaNext::erroFiltroRomaneio',
        text: 'plaNext::erroFilialNaoContEstoqueUtilLoteOuAlmox',
      });
      await grid.datasource.updateData([]);
      cardRetiradas.isVisible = false;
    } else {
      const formFiltro = Metadata.getInstance<Form>('retiradaPlanejamentoForm');
      const botaoExcluirRomaneio = Metadata.getInstance<Button>('botaoExcluirRomaneio');
      const tabs = Metadata.getInstance<ZdTabs>('tabsRetiradaPlanejamento');
      const conditions = {
        CDFILIAL: formFiltro.value.FILIAL_FILTRO,
        CDSERVICO: formFiltro.value.FILTRO_SERVICO,
        DTRETINI: formFiltro.value.DATA_RETIRADA[0],
        DTRETFIN: formFiltro.value.DATA_RETIRADA[1],
        DTCARDINI: formFiltro.value.DATA_CARDAPIO[0],
        DTCARDFIN: formFiltro.value.DATA_CARDAPIO[1],
      };
      const validacao = formFiltro.validate();
      if (validacao) {
        grid.datasource.filter = conditions;
        await grid.datasource.get();
        cardRetiradas.isVisible = true;
      }
      botaoExcluirRomaneio.disabled = true;
      tabs.tabs.forEach((tab) => {
        if (tabs.activeTab === 0) {
          if (tab.name === 'tabLancamento' || tab.name === 'tabResumo') {
            tab.disabled = true;
          }
        } else {
          tab.disabled = false;
        }
      });
    }
  }

  // Muda para tab de lancamento, exibe o formulario para incluir uma nova retirada
  public async adicionaRetirada() {
    const formNovaRetirada = Metadata.getInstance<Form>('lancamentoForm');
    const selectAddRequisicao = Metadata.getInstance<Select>('selectAddRequisicao');
    const campoCustoTotal = Metadata.getInstance<Number>('campoCustoTotal');
    campoCustoTotal.value = 0;
    this.novaRetirada = true;
    let retiradaPorRequisicao = true;
    if (!formNovaRetirada.value.selectAddRequisicao || formNovaRetirada.value.selectAddRequisicao === '0000000000') {
      const nrrequextrDefalt = '0000000000';
      const retiradaDefalt = {
        NRORG: localStorage.getItem('NRORG'),
        NRREQUEXTR: nrrequextrDefalt,
      };
      selectAddRequisicao.datasource.data.push(retiradaDefalt);
      await selectAddRequisicao.datasource.updateData(selectAddRequisicao.datasource.data);
      formNovaRetirada.value.selectAddRequisicao = '0000000000';
      retiradaPorRequisicao = false;
    }
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
    const validacao = formNovaRetirada.validate();
    if (validacao) {
      const conditions = {
        NRRETIRADA: formNovaRetirada.value.selectAddRequisicao,
        CDSERVFILI: formNovaRetirada.value.servicoAddRequisicao,
        DTRETIRADA: formNovaRetirada.value.dtRetiradaAddReq,
        DTCARDAPIO: formNovaRetirada.value.dtCardapioAddReq,
        CDTPRETIPADR: this.paramUnid.CDTPRETIPADR,
        CDALMOXARIFE: this.paramUnid.CDALMOXARIFE,
        CDLOCALESTOQ: this.paramUnid.CDLOCALESTOQ,
        DTPROCESSA: this.paramUnid.DTPROCESSA,
        IDLANCDTMEN: this.paramUnid.IDLANCDTMEN,
        IDLANCDTMAI: this.paramUnid.IDLANCDTMAI,
        NRDIASSAID: this.paramUnid.NRDIASSAID,
        NRDIASSAIDPD: this.paramUnid.NRDIASSAIDPD,
        IDLANCDTMENFIL: this.paramUnid.IDLANCDTMENFIL,
        NRDIASSAIDFIL: this.paramUnid.NRDIASSAIDFIL,
        IDLANCDTMAIFIL: this.paramUnid.IDLANCDTMAIFIL,
        NRDIASSAIDPDFIL: this.paramUnid.NRDIASSAIDPDFIL,
        CDFILIAL: this.paramUnid.CDFILIAL,
      };
      Http.post('/adicionaRetirada', conditions).then(async (response) => {
        if (response && response.data && response.data.error) {
          DialogService.show({
            name: 'error-dialog',
            type: 'error',
            title: 'plaNext::erroAddRetirada',
            text: `plaNext::${response.data.error}`,
          });
        } else if (retiradaPorRequisicao) {
          this.trocaVisibilidadeFormLancamento();
          gridTipoPrato.datasource.clearFilter();
          gridProduto.datasource.clearFilter();
          const cdfilial = response.data.dataset.retirada[0].CDFILIAL;
          const nrretirada = response.data.dataset.retirada[0].NRRETIRADA;
          const nrcardapio = response.data.dataset.retirada[0].NRCARDAPIO;
          const dtretirada = response.data.dataset.retirada[0].DTRETIRADA;
          const cdservfili = response.data.dataset.retirada[0].CDSERVFILI;
          if (response.data.dataset.retirada[0].TIPO_RETIRADA === 'S') {
            this.carregaGridTipoPrato(cdfilial, nrretirada, nrcardapio, retiradaPorRequisicao, dtretirada, cdservfili);
          } else {
            this.carregaGridProduto(cdfilial, nrretirada, dtretirada, nrcardapio, retiradaPorRequisicao, cdservfili);
          }
        } else {
          this.trocaVisibilidadeFormLancamento();
          await gridTipoPrato.datasource.clearFilter();
          await gridProduto.datasource.clearFilter();
          const retiradaTipoPrato = response.data.dataset.retirada[0].TIPO_RETIRADA;
          if (retiradaTipoPrato === 'S') {
            this.estadoAdicionarGridTipoPrato(true);
          } else {
            this.estadoAdicionarGridProduto(true);
          }
        }
      });
    }
  }

  // Verifica se é necessario exibir o formulario de nova retirada ou a legenda de uma existente
  public trocaVisibilidadeFormLancamento() {
    const formNovaRetirada = Metadata.getInstance<Form>('lancamentoForm');
    const legendaLancamento = Metadata.getInstance<Row>('dadosGridRomaneio');
    if (formNovaRetirada.isVisible) {
      const nrretirada = formNovaRetirada.value.selectAddRequisicao.NRRETIRADA;
      this.gridNrretirada = !nrretirada ? '0000000000' : nrretirada;
      this.gridCdservfili = formNovaRetirada.value.servicoAddRequisicao;
      this.gridDtretirada = formNovaRetirada.value.dtRetiradaAddReq;
      this.gridDtcardapio = formNovaRetirada.value.dtCardapioAddReq;
      formNovaRetirada.isVisible = false;
      legendaLancamento.isVisible = true;
    } else {
      legendaLancamento.isVisible = false;
      formNovaRetirada.clearForm();
      formNovaRetirada.isVisible = true;
    }
  }

  // Limpa o campo Nº da Requisição se mudar o serviço ou data da retirada
  public async limpaCampoNrretiradaForm() {
    const formNovaRetirada = Metadata.getInstance<Form>('lancamentoForm');
    if (formNovaRetirada.value.selectAddRequisicao) {
      const selectAddRequisicao = Metadata.getInstance<Select>('selectAddRequisicao');
      selectAddRequisicao.datasource.clearFilter();
      await selectAddRequisicao.datasource.updateData([]);
      selectAddRequisicao.setValue('');
      formNovaRetirada.value.selectAddRequisicao = null;
    }
  }

  // Manipula valores do edit in line referente as quantidades dos produtos
  public async validaQuantidadeInline({ component, row }: IEventParam<Number>) {
    const grid = component.parent as GridEditable;
    const { uniqueKey } = grid.datasource;
    const rowKey = row[uniqueKey];
    const qtextrareti = component.name.includes('QTEXTRARETI') === true ? component.value : row.QTEXTRARETI;
    const qtdevolvret = component.name.includes('QTDEVOLVRETI') === true ? component.value : row.QTDEVOLVRETI;
    const qtretirada = component.name.includes('QTRETIRADA') === true ? component.value : row.QTRETIRADA;
    const qtprevista = row.QTPREVISRETI;
    const totalPrevistaExtra = qtprevista + qtextrareti;
    const totalMenosDevolvida = totalPrevistaExtra - qtdevolvret;
    if (component.name.includes('QTEXTRARETI') === true || component.name.includes('QTDEVOLVRETI') === true) {
      const devolvidaMaiorQueTotal = qtdevolvret > totalPrevistaExtra;
      if (devolvidaMaiorQueTotal === true) {
        DialogService.show({
          name: 'error-dialog',
          type: 'error',
          title: 'plaNext::erro',
          text: 'plaNext::erroQtDevMaiorQtReq',
        });
      } else {
        grid.updateCell(rowKey, 'QTRETIRADA', totalMenosDevolvida);
        row.QTRETIRADA = totalMenosDevolvida;
      }
    } else if (component.name.includes('QTRETIRADA') === true) {
      if (qtretirada > qtprevista) {
        grid.updateCell(rowKey, 'QTDEVOLVRETI', 0);
        grid.updateCell(rowKey, 'QTEXTRARETI', qtretirada - qtprevista);
        row.QTDEVOLVRETI = 0;
        row.QTEXTRARETI = qtretirada - qtprevista;
      } else {
        grid.updateCell(rowKey, 'QTDEVOLVRETI', qtprevista - qtretirada);
        grid.updateCell(rowKey, 'QTEXTRARETI', 0);
        row.QTDEVOLVRETI = qtprevista - qtretirada;
        row.QTEXTRARETI = 0;
      }
    }
  }

  // Função responsavel para preparar a linha de um novo tipo de prato
  public async estadoAdicionarGridTipoPrato(novaRetirada = false) {
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
    const buttonSalvaNovoTP = Metadata.getInstance<Button>('botaoSalvarTipoPrato');
    const buttonCancelaNovoTP = Metadata.getInstance<Button>('botaoCancelaTipoPrato');
    const botaoatualizaEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    const botaoAddTipoPrato = Metadata.getInstance<Button>('botaoAdicionaTipoPrato');
    gridProdVinc.datasource.updateData([]);
    botaoatualizaEstoqueTP.disabled = true;
    this.novaRetirada = novaRetirada;
    const dados = {
      CDTIPOPRAT: '',
      CDPRATO: '',
      CHAVE_ITEMRETI: '',
    };
    gridTipoPrato.columns.forEach((field) => {
      if (field.name === 'CDTIPOPRAT' || field.name === 'CDPRATO') {
        field.editable = true;
      } else {
        field.isVisible = false;
      }
      if (field.name === 'CDPRATO') {
        this.carregaPratos(field);
      }
    });
    buttonSalvaNovoTP.isVisible = true;
    buttonCancelaNovoTP.isVisible = true;
    cardTipoPrato.isVisible = true;
    gridTipoPrato.editingNewRows = true;
    gridTipoPrato.addNewRow(dados, 'end');
    botaoAddTipoPrato.disabled = true;
    this.estadoAdicionarGridProdVinc();
  }

  // Função responsavel para preparar a linha de um novo Produto vinculado ao tipo de prato
  public async estadoAdicionarGridProdVinc() {
    const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    const cardProduto = Metadata.getInstance<Card>('lancamentoProduto');
    const botaoatualizaEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
    const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
    const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
    botaoatualizaEstoqueTP.disabled = true;
    cardProduto.isVisible = false;
    /* gridProdVinc.columns.forEach((field) => {
      if (
        field.name === 'CDPRODUTO'
        || field.name === 'QTEXTRARETI'
        || field.name === 'QTDEVOLVRETI'
        || field.name === 'QTRETIRADA'
        || field.name === 'CDTIPORETI'
      ) {
        field.editable = true;
      } else {
        field.isVisible = false;
      }
      if (field.name === 'CDPRODUTO') {
        this.carregaProdutos(field);
      }
    }); */
    gridProdVinc.columns.forEach((field) => {
      if (
        field.name === 'CDPRODUTO' || field.name === 'CDTIPORETI') {
        field.editable = true;
      } else {
        field.isVisible = false;
      }
      if (field.name === 'CDPRODUTO') {
        this.carregaProdutos(field);
      }
    });
    const dadosProdVinc = {
      CDPRODUTO: '',
      QTEXTRARETI: 0,
      QTDEVOLVRETI: 0,
      QTRETIRADA: 0,
      QTPREVISRETI: 0,
      CDTIPORETI: '',
    };
    if (gridTipoPrato.editingNewRows === false) {
      this.isDisabledButtonCardProd = false;
    }
    if (!this.novaRetirada) {
      cardTipoPrato.disabled = true;
    }
    cardProdVinc.isVisible = true;
    botaoExcluirProdVinculado.disabled = true;
    gridProdVinc.editingNewRows = true;
    gridProdVinc.addNewRow(dadosProdVinc, 'end');
    botaoAddProdVinculado.disabled = true;
  }

  // Função responsavel para preparar a linha de um novo Produto
  public async estadoAdicionarGridProduto(novaRetirada = false) {
    const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
    const cardProduto = Metadata.getInstance<Card>('lancamentoProduto');
    const botaoExcluirProduto = Metadata.getInstance<Card>('botaoExcluirProduto');
    const botaoatualizaEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
    const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
    const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
    const botaoAddProduto = Metadata.getInstance<Button>('botaoAdicionaProduto');
    botaoatualizaEstoqueTP.disabled = true;
    this.novaRetirada = novaRetirada;
    cardProdVinc.isVisible = false;
    cardTipoPrato.isVisible = false;
    /* gridProduto.columns.forEach((field) => {
      if (
        field.name === 'CDPRODUTO'
        || field.name === 'QTEXTRARETI'
        || field.name === 'QTDEVOLVRETI'
        || field.name === 'QTRETIRADA'
        || field.name === 'CDTIPORETI'
      ) {
        field.editable = true;
      } else {
        field.isVisible = false;
      }
      if (field.name === 'CDPRODUTO') {
        this.carregaProdutos(field);
      }
    }); */
    gridProduto.columns.forEach((field) => {
      if (
        field.name === 'CDPRODUTO' || field.name === 'CDTIPORETI') {
        field.editable = true;
      } else {
        field.isVisible = false;
      }
      if (field.name === 'CDPRODUTO') {
        this.carregaProdutos(field);
      }
    });
    const dados = {
      CDPRODUTO: '',
      QTEXTRARETI: 0,
      QTDEVOLVRETI: 0,
      QTRETIRADA: 0,
      QTPREVISRETI: 0,
      CDTIPORETI: '',
    };
    this.isDisabledButtonCardProd = false;
    cardProduto.isVisible = true;
    botaoExcluirProduto.disabled = true;
    gridProduto.editingNewRows = true;
    gridProduto.addNewRow(dados, 'end');
    botaoAddProduto.disabled = true;
  }

  // Função responsável por salvar uma nova retirada mediante ao tipo de retirada por prato.
  public salvaGridProdutoVinculadoNovaRetirada() {
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
    const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
    const formNovaRetirada = Metadata.getInstance<Form>('lancamentoForm');
    const buttonSalvaNovoTP = Metadata.getInstance<Button>('botaoSalvarTipoPrato');
    const buttonCancelaNovoTP = Metadata.getInstance<Button>('botaoCancelaTipoPrato');
    const botaoAddTipoPrato = Metadata.getInstance<Button>('botaoAdicionaTipoPrato');
    const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
    if (this.novaRetirada === true) {
      if (gridProdVinc.editing) {
        const validacaoProdVinc = gridProdVinc.isGridValid(true);
        const validacaoTipoPrato = gridTipoPrato.isGridValid(true);
        if (validacaoProdVinc && validacaoTipoPrato) {
          const cdtipoprat = gridTipoPrato.getEditedRows()[0].CDTIPOPRAT;
          const cdprato = gridTipoPrato.getEditedRows()[0].CDPRATO;
          const cdfilauxprat = gridTipoPrato.getEditedRows()[0].CDFILAUXPRAT;
          const cdproduto = gridProdVinc.getEditedRows()[0].CDPRODUTO;
          const qtextrareti = gridProdVinc.getEditedRows()[0].QTEXTRARETI;
          const qtdevolvreti = gridProdVinc.getEditedRows()[0].QTDEVOLVRETI;
          const qtretirada = gridProdVinc.getEditedRows()[0].QTRETIRADA;
          const qtprevista = gridProdVinc.getEditedRows()[0].QTPREVISRETI;
          const cdtiporeti = gridProdVinc.getEditedRows()[0].CDTIPORETI;
          const exibeSaldoEstreti = this.paramUnid.IDEXIBESALDOESTRETI;
          const conditions = {
            NRRETIRADA: formNovaRetirada.value.selectAddRequisicao,
            CDSERVFILI: formNovaRetirada.value.servicoAddRequisicao,
            DTRETIRADA: formNovaRetirada.value.dtRetiradaAddReq,
            DTCARDAPIO: formNovaRetirada.value.dtCardapioAddReq,
            DTPROCESSA: this.paramUnid.DTPROCESSA,
            CDFILIAL: this.paramUnid.CDFILIAL,
            CDTIPOPRAT: cdtipoprat,
            CDPRATO: cdprato,
            CDPRODUTO: cdproduto,
            CDALMOXARIFE: this.paramUnid.CDALMOXARIFE,
            CDLOCALESTOQ: this.paramUnid.CDLOCALESTOQ,
            QTEXTRARETI: qtextrareti,
            QTDEVOLVRETI: qtdevolvreti,
            QTRETIRADA: qtretirada,
            QTPREVISRETI: qtprevista,
            CDTIPORETI: cdtiporeti,
            CDFILAUXPRAT: cdfilauxprat,
            IDEXIBESALDOESTRETI: exibeSaldoEstreti,
          };
          cardTipoPrato.disabled = true;
          cardProdVinc.disabled = true;
          Http.post('/salvaNovaRetirada', conditions).then((response) => {
            if (response && response.data && response.data.error) {
              DialogService.show({
                name: 'error-dialog',
                type: 'error',
                title: 'plaNext::erroAddRetirada',
                text: response.data.error,
              });
              cardTipoPrato.disabled = false;
              cardProdVinc.disabled = false;
            } else {
              buttonSalvaNovoTP.isVisible = false;
              buttonCancelaNovoTP.isVisible = false;
              this.isDisabledButtonCardProd = true;
              gridProdVinc.cancelEditedRows();
              gridProdVinc.datasource.clearFilter();
              const retornoRetirada = response.data.dataset.retirada;
              const cdfilial = retornoRetirada.CDFILIAL;
              const nrretirada = retornoRetirada.NRRETIRADA;
              const nrcardapio = retornoRetirada.NRCARDAPIO;
              const dtretirada = retornoRetirada.DTRETIRADA;
              const cdservfili = retornoRetirada.CDSERVFILI;
              this.carregaGridTipoPrato(cdfilial, nrretirada, nrcardapio, false, dtretirada, cdservfili);
              gridTipoPrato.reload();
              cardTipoPrato.disabled = false;
              cardProdVinc.disabled = false;
              botaoAddTipoPrato.disabled = false;
              botaoAddProdVinculado.disabled = false;
              if (this.estadoGridResumo) {
                this.carregaGridResumo();
              }
            }
          }, () => {
            cardTipoPrato.disabled = false;
            cardProdVinc.disabled = false;
            botaoAddTipoPrato.disabled = false;
            botaoAddProdVinculado.disabled = false;
          });
        }
      }
    } else {
      this.salvaGridProdutoVinculadoRetiradaExistente();
    }
  }

  // Função responsável por salvar uma  retirada existente mediante ao tipo de retirada por prato.
  public salvaGridProdutoVinculadoRetiradaExistente() {
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
    const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
    const buttonSalvaNovoTP = Metadata.getInstance<Button>('botaoSalvarTipoPrato');
    const buttonCancelaNovoTP = Metadata.getInstance<Button>('botaoCancelaTipoPrato');
    const botaoAddTipoPrato = Metadata.getInstance<Button>('botaoAdicionaTipoPrato');
    const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
    if (gridProdVinc.editing) {
      const validacaoProdVinc = gridProdVinc.isGridValid(true);
      const validacaoTipoPrato = gridTipoPrato.isGridValid(true);
      if (validacaoProdVinc && validacaoTipoPrato) {
        let cdtipopratEditRows = null;
        let cdpratoEditRows = null;
        let cdfilauxpratEditRows = null;
        if (gridTipoPrato.editing === true) {
          cdtipopratEditRows = gridTipoPrato.getEditedRows()[0].CDTIPOPRAT;
          cdpratoEditRows = gridTipoPrato.getEditedRows()[0].CDPRATO;
          cdfilauxpratEditRows = gridTipoPrato.getEditedRows()[0].CDFILAUXPRAT;
        }
        const cdtipopratCurrentRow = gridTipoPrato.datasource.currentRow.CDTIPOPRAT;
        const cdpratoCurrentRow = gridTipoPrato.datasource.currentRow.CDPRATO;
        const cdfilauxpratCurrentRow = gridTipoPrato.datasource.currentRow.CDFILAUXPRAT;
        const cdtipoprat = cdtipopratEditRows !== null ? cdtipopratEditRows : cdtipopratCurrentRow;
        const cdprato = cdpratoEditRows !== null ? cdpratoEditRows : cdpratoCurrentRow;
        const cdfilauxprat = cdfilauxpratEditRows !== null ? cdfilauxpratEditRows : cdfilauxpratCurrentRow;
        const cdproduto = gridProdVinc.getEditedRows()[0].CDPRODUTO;
        const qtextrareti = gridProdVinc.getEditedRows()[0].QTEXTRARETI;
        const qtdevolvreti = gridProdVinc.getEditedRows()[0].QTDEVOLVRETI;
        const qtretirada = gridProdVinc.getEditedRows()[0].QTRETIRADA;
        const qtprevista = gridProdVinc.getEditedRows()[0].QTPREVISRETI;
        const cdtiporeti = gridProdVinc.getEditedRows()[0].CDTIPORETI;
        const cdfilial = gridTipoPrato.datasource.data[0].CDFILIAL;
        const nrretirada = gridTipoPrato.datasource.data[0].NRRETIRADA;
        const nrcardapio = gridTipoPrato.datasource.data[0].NRCARDAPIO;
        const nrlancestq = gridTipoPrato.datasource.data[0].NRLANCESTQ;
        const cdservfili = gridTipoPrato.datasource.data[0].CDSERVFILI;
        const dtcardapio = gridTipoPrato.datasource.data[0].DTCARDAPIO;
        const dtretirada = gridTipoPrato.datasource.data[0].DTRETIRADA;
        const conditions = {
          NRRETIRADA: nrretirada,
          CDSERVFILI: cdservfili,
          DTRETIRADA: dtretirada,
          DTCARDAPIO: dtcardapio,
          NRCARDAPIO: nrcardapio,
          NRLANCESTQ: nrlancestq,
          DTPROCESSA: this.paramUnid.DTPROCESSA,
          CDFILIAL: cdfilial,
          CDTIPOPRAT: cdtipoprat,
          CDPRATO: cdprato,
          CDPRODUTO: cdproduto,
          CDALMOXARIFE: this.paramUnid.CDALMOXARIFE,
          CDLOCALESTOQ: this.paramUnid.CDLOCALESTOQ,
          QTEXTRARETI: qtextrareti,
          QTDEVOLVRETI: qtdevolvreti,
          QTRETIRADA: qtretirada,
          QTPREVISRETI: qtprevista,
          CDTIPORETI: cdtiporeti,
          CDFILAUXPRAT: cdfilauxprat,
        };
        cardTipoPrato.disabled = true;
        cardProdVinc.disabled = true;
        Http.post('/salvaNovoPratoProduto', conditions).then((response) => {
          if (response && response.data && response.data.error) {
            DialogService.show({
              name: 'error-dialog',
              type: 'error',
              title: 'plaNext::erroSalvar',
              text: `plaNext::${response.data.error}`,
            });
            cardTipoPrato.disabled = false;
            cardProdVinc.disabled = false;
          } else {
            buttonSalvaNovoTP.isVisible = false;
            buttonCancelaNovoTP.isVisible = false;
            this.isDisabledButtonCardProd = true;
            gridProdVinc.cancelEditedRows();
            gridProdVinc.datasource.clearFilter();
            if (gridTipoPrato.editing) {
              this.carregaGridTipoPrato(cdfilial, nrretirada, nrcardapio, false, dtretirada, cdservfili);
            } else {
              const botaoAtualizarEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
              botaoAtualizarEstoqueTP.disabled = false;
              this.carregaGridProdutoVinculado();
            }
            if (this.estadoGridResumo) {
              this.carregaGridResumo();
            }
            cardTipoPrato.disabled = false;
            cardProdVinc.disabled = false;
            botaoAddTipoPrato.disabled = false;
            botaoAddProdVinculado.disabled = false;
          }
        }, () => {
          cardTipoPrato.disabled = false;
          cardProdVinc.disabled = false;
          botaoAddTipoPrato.disabled = false;
          botaoAddProdVinculado.disabled = false;
        });
      }
    }
  }

  // Função responsável por salvar uma nova retirada mediante ao tipo de retirada por produto.
  public salvaGridProdutoNovaRetirada() {
    const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
    const cardProduto = Metadata.getInstance<Card>('lancamentoProduto');
    const botaoAddProduto = Metadata.getInstance<Button>('botaoAdicionaProduto');
    if (gridProduto.editing === true) {
      const formNovaRetirada = Metadata.getInstance<Form>('lancamentoForm');
      if (this.novaRetirada === true) {
        const cdproduto = gridProduto.getEditedRows()[0].CDPRODUTO;
        const qtextrareti = gridProduto.getEditedRows()[0].QTEXTRARETI;
        const qtdevolvreti = gridProduto.getEditedRows()[0].QTDEVOLVRETI;
        const qtretirada = gridProduto.getEditedRows()[0].QTRETIRADA;
        const qtprevista = gridProduto.getEditedRows()[0].QTPREVISRETI;
        const cdtiporeti = gridProduto.getEditedRows()[0].CDTIPORETI;
        const cdprato = null;
        const cdfilauxprat = null;
        const cdtipoprat = null;
        const validacao = gridProduto.isGridValid(true);
        if (validacao) {
          const conditions = {
            NRRETIRADA: formNovaRetirada.value.selectAddRequisicao,
            CDSERVFILI: formNovaRetirada.value.servicoAddRequisicao,
            DTRETIRADA: formNovaRetirada.value.dtRetiradaAddReq,
            DTCARDAPIO: formNovaRetirada.value.dtCardapioAddReq,
            DTPROCESSA: this.paramUnid.DTPROCESSA,
            CDFILIAL: this.paramUnid.CDFILIAL,
            CDPRODUTO: cdproduto,
            CDALMOXARIFE: this.paramUnid.CDALMOXARIFE,
            CDLOCALESTOQ: this.paramUnid.CDLOCALESTOQ,
            QTEXTRARETI: qtextrareti,
            QTDEVOLVRETI: qtdevolvreti,
            QTRETIRADA: qtretirada,
            QTPREVISRETI: qtprevista,
            CDTIPORETI: cdtiporeti,
            CDPRATO: cdprato,
            CDFILAUXPRAT: cdfilauxprat,
            CDTIPOPRAT: cdtipoprat,
          };
          cardProduto.disabled = true;
          Http.post('/salvaNovaRetirada', conditions).then((response) => {
            if (response && response.data && response.data.dataset && response.data.dataset.error) {
              DialogService.show({
                name: 'error-dialog',
                type: 'error',
                title: 'plaNext::erroAddRetirada',
                text: response.data.dataset.error[0],
              });
              cardProduto.disabled = false;
            } else {
              this.isDisabledButtonCardProd = true;
              const retornoRetirada = response.data.dataset.retirada;
              const cdfilial = retornoRetirada.CDFILIAL;
              const nrretirada = retornoRetirada.NRRETIRADA;
              const dtretirada = retornoRetirada.DTRETIRADA;
              const nrcardapio = retornoRetirada.NRCARDAPIO;
              const cdservfili = retornoRetirada.CDSERVFILI;
              this.carregaGridProduto(cdfilial, nrretirada, dtretirada, nrcardapio, false, cdservfili);
              cardProduto.disabled = false;
              botaoAddProduto.disabled = false;
              if (this.estadoGridResumo) {
                this.carregaGridResumo();
              }
            }
          }, () => {
            cardProduto.disabled = false;
            botaoAddProduto.disabled = false;
          });
        }
      } else {
        this.salvaGridProdutoRetiradaExistente();
      }
    }
  }

  // Função responsável por salvar uma retirada existente mediante ao tipo de retirada por Produto.
  public salvaGridProdutoRetiradaExistente() {
    const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
    const cardProduto = Metadata.getInstance<Card>('lancamentoProduto');
    const botaoAddProduto = Metadata.getInstance<Button>('botaoAdicionaProduto');
    if (gridProduto.editing === true) {
      const cdproduto = gridProduto.getEditedRows()[0].CDPRODUTO;
      const qtextrareti = gridProduto.getEditedRows()[0].QTEXTRARETI;
      const qtdevolvreti = gridProduto.getEditedRows()[0].QTDEVOLVRETI;
      const qtretirada = gridProduto.getEditedRows()[0].QTRETIRADA;
      const qtprevista = gridProduto.getEditedRows()[0].QTPREVISRETI;
      const cdtiporeti = gridProduto.getEditedRows()[0].CDTIPORETI;
      const cdprato = null;
      const cdfilauxprat = null;
      const cdtipoprat = null;
      const validacao = gridProduto.isGridValid(true);
      if (validacao) {
        const cdfilial = gridProduto.datasource.data[0].CDFILIAL;
        const nrretirada = gridProduto.datasource.data[0].NRRETIRADA;
        const dtretirada = gridProduto.datasource.data[0].DTRETIRADA;
        const nrcardapio = gridProduto.datasource.data[0].NRCARDAPIO;
        const cdservfili = gridProduto.datasource.data[0].CDSERVFILI;
        const dtcardapio = gridProduto.datasource.data[0].DTCARDAPIO;
        const nrlancestq = gridProduto.datasource.data[0].NRLANCESTQ;
        const conditions = {
          NRRETIRADA: nrretirada,
          CDSERVFILI: cdservfili,
          DTRETIRADA: dtretirada,
          DTCARDAPIO: dtcardapio,
          NRCARDAPIO: nrcardapio,
          NRLANCESTQ: nrlancestq,
          DTPROCESSA: this.paramUnid.DTPROCESSA,
          CDFILIAL: cdfilial,
          CDPRODUTO: cdproduto,
          CDALMOXARIFE: this.paramUnid.CDALMOXARIFE,
          CDLOCALESTOQ: this.paramUnid.CDLOCALESTOQ,
          QTEXTRARETI: qtextrareti,
          QTDEVOLVRETI: qtdevolvreti,
          QTRETIRADA: qtretirada,
          QTPREVISRETI: qtprevista,
          CDTIPORETI: cdtiporeti,
          CDPRATO: cdprato,
          CDFILAUXPRAT: cdfilauxprat,
          CDTIPOPRAT: cdtipoprat,
        };
        cardProduto.disabled = true;
        Http.post('/salvaNovoPratoProduto', conditions).then((response) => {
          if (response && response.data && response.data.dataset && response.data.dataset.error) {
            DialogService.show({
              name: 'error-dialog',
              type: 'error',
              title: 'plaNext::erroSalvar',
              text: response.data.dataset.error[0],
            });
            cardProduto.disabled = false;
          } else {
            this.isDisabledButtonCardProd = true;
            this.carregaGridProduto(cdfilial, nrretirada, dtretirada, nrcardapio, false, cdservfili);
            cardProduto.disabled = false;
            botaoAddProduto.disabled = false;
            if (this.estadoGridResumo) {
              this.carregaGridResumo();
            }
          }
        }, () => {
          cardProduto.disabled = false;
          botaoAddProduto.disabled = false;
        });
      }
    }
  }

  public async salvaGridTipoPratoNovaRetirada() {
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    gridProdVinc.setFocus();
  }

  // Carrega filtro do datasource campo de pratos ao acrescentar um novo Tipo de Prato
  public async carregaPratos(field) {
    const nrorg = localStorage.getItem('NRORG');
    const filialLogada = localStorage.getItem('FILIAL_LOGADA');
    const cdnvprato = '4';
    const conditions = {
      NRORG: nrorg,
      CDFILIAL: filialLogada,
      CDNVPRATO: cdnvprato,
    };
    field.componentProps.datasource.filter = conditions;
  }

  // Carrega filtro do datasource campo de produtos ao acrescentar um novo Produto
  public async carregaProdutos(field) {
    const nrorg = localStorage.getItem('NRORG');
    const filialLogada = localStorage.getItem('FILIAL_LOGADA');
    const nivelMax = this.paramUnid.CDPICTPROD.split('.').length;
    const cdnvproduto = nivelMax;
    const conditions = {
      NRORG: nrorg,
      CDFILIAL: filialLogada,
      CDNVPRODUTO: cdnvproduto,
    };
    field.componentProps.datasource.filter = conditions;
  }

  // Função responsável para mudar a visualização para tab de lançamento e exibir o formcom os dados para uma nova retirada
  public async exibeFormNovaRetirada() {
    const tabs = Metadata.getInstance<Tabs>('tabsRetiradaPlanejamento');
    const cardFiltroInicial = Metadata.getInstance<Card>('retiradaPlanejamentoCard');
    if (this.estadoGridTipoPrato || this.estadoGridProduto) {
      const formNovaRetirada = Metadata.getInstance<Form>('lancamentoForm');
      const legendaLancamento = Metadata.getInstance<Row>('dadosGridRomaneio');
      formNovaRetirada.clearForm();
      legendaLancamento.isVisible = false;
      formNovaRetirada.isVisible = true;
    }
    if (this.estadoGridTipoPrato && this.estadoGridProduto) {
      const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
      const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
      const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
      const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
      const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
      const cardProduto = Metadata.getInstance<Card>('lancamentoProduto');
      gridTipoPrato.datasource.updateData([]);
      gridProdVinc.datasource.updateData([]);
      gridProduto.datasource.updateData([]);
      cardProduto.isVisible = false;
      cardTipoPrato.isVisible = false;
      cardProdVinc.isVisible = false;
    }
    tabs.tabs.forEach((tab) => {
      if (tab.name === 'tabLancamento') {
        tab.disabled = false;
        tab.children.forEach((chil) => {
          if (chil.name === 'lancamentoForm') {
            chil.isVisible = true;
          } else {
            chil.isVisible = false;
          }
        });
      } else if (tab.name === 'tabResumo') {
        tab.disabled = true;
      }
    });
    cardFiltroInicial.disabled = true;
    await tabs.nextTab(true);
    this.estadoGridTipoPrato = true;
    this.estadoGridProduto = true;
  }

  // Função que roda ao selecionar um romaneio na tab inicial
  public async selecionaRetiradaPlanejamento() {
    const tabs = Metadata.getInstance<ZdTabs>('tabsRetiradaPlanejamento');
    const grid = Metadata.getInstance<Grid>('retiradaPlanejamentoGrid');
    const buttonExcluir = Metadata.getInstance<Button>('botaoExcluirRomaneio');
    const tipoRetirada = grid.datasource.currentRow.IDRETIPRAT;
    tabs.tabs.forEach((tab) => {
      if (tab.name === 'tabLancamento') {
        tab.disabled = false;
        tab.children.forEach((chil) => {
          if (tipoRetirada === 'S') {
            if (chil.name === 'lancamentoTipoDePrato' || chil.name === 'lancamentoProdutoVinculado') {
              chil.isVisible = true;
            }
            if (chil.name === 'lancamentoProduto') {
              chil.isVisible = false;
            }
            if (this.estadoGridTipoPrato && chil.name === 'lancamentoTipoDePrato') {
              const cardProduto = Metadata.getInstance<Card>('lancamentoProduto');
              cardProduto.isVisible = false;
              if (Object.keys(grid.datasource.currentRow).length > 0) {
                const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
                gridProdVinc.datasource.updateData([]);
                this.carregaGridTipoPrato();
              }
            }
          }
          if (tipoRetirada === 'N') {
            if (chil.name === 'lancamentoTipoDePrato' || chil.name === 'lancamentoProdutoVinculado') {
              chil.isVisible = false;
            } else if (chil.name === 'lancamentoProduto') {
              chil.isVisible = true;
              if (this.estadoGridProduto) {
                const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
                const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
                cardTipoPrato.isVisible = false;
                cardProdVinc.isVisible = false;
                const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
                gridProduto.datasource.updateData([]);
                this.carregaGridProduto();
              }
            }
          }
          if (chil.name === 'lancamentoForm') {
            chil.isVisible = false;
          }
          if (chil.name === 'dadosGridRomaneio') {
            chil.isVisible = true;
          }
        });
      } else if (tab.name === 'tabResumo') {
        tab.disabled = false;
        if (this.estadoGridResumo) {
          if (Object.keys(grid.datasource.currentRow).length > 0) {
            const gridResumo = Metadata.getInstance<GridEditable>('resumoGrid');
            gridResumo.datasource.updateData([]);
            this.carregaGridResumo();
          }
        }
      }
    });
    const dtretirada = grid.datasource.currentRow.DTLANCESTQ;
    const cdfilial = grid.datasource.currentRow.CDFILIAL;
    const cdservfili = grid.datasource.currentRow.CDSERVFILI;
    this.gridNrretirada = grid.datasource.currentRow.NRRETIRADA;
    this.gridCdservfili = cdservfili;
    this.gridDtretirada = dtretirada;
    this.gridDtcardapio = grid.datasource.currentRow.DTCARDAPIO;
    buttonExcluir.disabled = false;
    const bloqExclusao = await this.verificaRetiradaBloqueadaAlteracao(dtretirada, cdfilial, cdservfili);
    if (bloqExclusao) {
      buttonExcluir.disabled = true;
    } else {
      buttonExcluir.disabled = false;
    }
  }

  private deleteRomaneio: any = {
    click: () => this.excluirRomaneio(),
  };

  public async confirmaExclusaoRomaneio() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'plaNext::excluirRomaneio',
      text: 'plaNext::solicitaExclusaoRomaneio',
      buttons: [
        {
          name: 'no', component: 'ZdButton', label: 'Não', events: this.cancelaModalDialogservice, outline: true,
        },
        {
          name: 'yes', component: 'ZdButton', label: 'Sim', events: this.deleteRomaneio,
        },
      ],
    });
  }

  public async excluirRomaneio() {
    const grid = Metadata.getInstance<Grid>('retiradaPlanejamentoGrid');
    const tabs = Metadata.getInstance<ZdTabs>('tabsRetiradaPlanejamento');
    const botaoExcluirRomaneio = Metadata.getInstance<Button>('botaoExcluirRomaneio');
    if (Object.keys(grid.datasource.currentRow).length > 0) {
      const conditions = {
        CDFILIAL: grid.datasource.currentRow.CDFILIAL,
        NRRETIRADA: grid.datasource.currentRow.NRRETIRADA,
        NRCARDAPIO: grid.datasource.currentRow.NRCARDAPIO,
        NRLANCESTQ: grid.datasource.currentRow.NRLANCESTQ,
      };
      Http.post('/excluirRomaneio', conditions).then((response) => {
        if (response && response.data && response.data.error) {
          DialogService.show({
            name: 'error-dialog',
            type: 'error',
            title: 'plaNext::erroExcluirRomaneio',
            text: response.data.error,
          });
        } else {
          grid.datasource.get();
          tabs.tabs.forEach((tab) => {
            if (tab.name === 'tabLancamento' || tab.name === 'tabResumo') {
              tab.disabled = true;
            }
          });
        }
      });
      botaoExcluirRomaneio.disabled = true;
    }
    DialogService.hide();
  }

  // Define visualizações no onMonted da tab Lançamento
  public async iniciaTabLancamento() {
    const cardFiltroInicial = Metadata.getInstance<Card>('retiradaPlanejamentoCard');
    const botaoExcluirTipoPrato = Metadata.getInstance<Button>('botaoExcluirTipoPrato');
    const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
    const botaoAdicionarProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
    const botaoExcluirProduto = Metadata.getInstance<Button>('botaoExcluirProduto');
    botaoExcluirTipoPrato.disabled = true;
    botaoExcluirProdVinculado.disabled = true;
    botaoAdicionarProdVinculado.disabled = true;
    botaoExcluirProduto.disabled = true;
    cardFiltroInicial.disabled = true;
  }

  // Define visualizações no onMonted da tab Resumo
  public async iniciaTabResumo() {
    const cardFiltroInicial = Metadata.getInstance<Card>('retiradaPlanejamentoCard');
    cardFiltroInicial.disabled = true;
  }

  // Converte uma string data formato brasileiro para um new Date
  public converteDate(data) {
    const dia = data.substring(0, 2);
    const mes = data.substring(3, 5) - 1;
    const ano = data.substring(6, 10);
    const formatoDate = new Date(ano, mes, dia);
    return formatoDate;
  }

  // Verifica bloqueios referente a retirada (Intervalo data de processamento, serviço automático, bloqueio de inclusão)
  public async verificaRetiradaBloqueadaAlteracao(dtretirada, cdfilial, cdservfili) {
    const dtretiradaDate = this.converteDate(dtretirada);
    const dtprocessaDate = this.converteDate(this.paramUnid.DTPROCESSA);
    let bloqueiaInclusao;
    let bloqueia;
    const conditions = {
      CDFILIAL: cdfilial,
      CDSERVFILI: cdservfili,
    };
    Http.post('/retornaIdbloqincprodretipla', conditions).then((response) => {
      bloqueiaInclusao = response.data.dataset.bloqInclusao[0].IDBLOQINCPRODRETIPLA;
    });
    if (dtretiradaDate < dtprocessaDate) {
      if (this.paramUnid.IDLANCDTMENFIL === 'N' && this.paramUnid.IDLANCDTMEN === 'N') {
        bloqueia = dtretiradaDate < dtprocessaDate;
      } else if (this.paramUnid.IDLANCDTMENFIL === 'N' && this.paramUnid.IDLANCDTMEN !== 'N') {
        const dtProcessaCopy = new Date(dtprocessaDate);
        dtProcessaCopy.setDate(dtProcessaCopy.getDate() - this.paramUnid.NRDIASSAID);
        bloqueia = dtretiradaDate < dtProcessaCopy;
      } else if (this.paramUnid.IDLANCDTMENFIL !== 'N') {
        const dtProcessaCopy = new Date(dtprocessaDate);
        dtProcessaCopy.setDate(dtProcessaCopy.getDate() - this.paramUnid.NRDIASSAIDFIL);
        bloqueia = dtretiradaDate < dtProcessaCopy;
      }
    } else if (this.paramUnid.IDLANCDTMAIFIL === 'S') {
      const dtProcessaCopy = new Date(dtprocessaDate);
      dtProcessaCopy.setDate(dtProcessaCopy.getDate() + this.paramUnid.NRDIASSAIDPDFIL);
      bloqueia = dtretiradaDate > dtProcessaCopy;
    } else if (this.paramUnid.IDLANCDTMAI === 'S') {
      const dtProcessaCopy = new Date(dtprocessaDate);
      dtProcessaCopy.setDate(dtProcessaCopy.getDate() + this.paramUnid.NRDIASSAIDPD);
      bloqueia = dtretiradaDate > dtProcessaCopy;
    } else {
      bloqueia = dtretiradaDate > dtprocessaDate;
    }
    if (cdservfili === this.paramUnid.CDSERVRETPME) {
      bloqueia = true;
      this.bloqueioTotalRetirada();
    } else if (bloqueia) {
      this.bloqueiaTodasAcoes();
    } else if (bloqueiaInclusao === 'S') {
      this.bloqueiaAcoesInclusao();
    } else {
      this.visualizacaoDefaultAcoes();
    }
    return bloqueia;
  }

  // Bloqueio referente ao CDSERVRETPME ser igual ao serviço da retirada em questão
  public bloqueioTotalRetirada() {
    const tabs = Metadata.getInstance<ZdTabs>('tabsRetiradaPlanejamento');
    tabs.tabs.forEach((tab) => {
      if (tab.name === 'tabResumo') {
        tab.disabled = true;
      }
    });
    if (this.estadoGridTipoPrato) {
      // Grid Tipo Prato
      const botaoAddTipoPrato = Metadata.getInstance<Button>('botaoAdicionaTipoPrato');
      const botaoExcluirTipoPrato = Metadata.getInstance<Button>('botaoExcluirTipoPrato');
      const botaoAtualizarEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
      botaoAddTipoPrato.disabled = true;
      botaoExcluirTipoPrato.disabled = true;
      botaoAtualizarEstoqueTP.disabled = true;
      // Grid Produto Vinculado
      const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
      const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
      // const botaoCopiarQuantProdVinc = Metadata.getInstance<Button>('botaoCopiarQuantProdVinc');
      botaoAddProdVinculado.disabled = true;
      botaoExcluirProdVinculado.disabled = true;
      // botaoCopiarQuantProdVinc.disabled = true;
    }
    // Grid Produto
    if (this.estadoGridProduto) {
      const botaoAddProduto = Metadata.getInstance<Button>('botaoAdicionaProduto');
      const botaoExcluirProduto = Metadata.getInstance<Button>('botaoExcluirProduto');
      const botaoAtualizarEstoqueProduto = Metadata.getInstance<Button>('botaoatualizaEstoqueProd');
      botaoAddProduto.disabled = true;
      botaoExcluirProduto.disabled = true;
      botaoAtualizarEstoqueProduto.disabled = true;
    }
    if (this.estadoGridTipoPrato || this.estadoGridProduto) {
      DialogService.show({
        name: 'error-dialog',
        type: 'warning',
        title: 'plaNext::aviso',
        text: 'plaNext::retiradaServAuto',
      });
    }
    this.bloqTodasAcoes = true;
    this.bloqAcoesAdicionar = true;
  }

  // Bloqueio referente a inclusões dentro da retirada
  public bloqueiaAcoesInclusao() {
    if (this.estadoGridTipoPrato) {
      // Grid Tipo Prato
      const botaoAddTipoPrato = Metadata.getInstance<Button>('botaoAdicionaTipoPrato');
      const botaoExcluirTipoPrato = Metadata.getInstance<Button>('botaoExcluirTipoPrato');
      const botaoAtualizarEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
      botaoAddTipoPrato.disabled = true;
      botaoExcluirTipoPrato.disabled = false;
      botaoAtualizarEstoqueTP.disabled = false;
      // Grid Produto Vinculado
      const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
      const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
      // const botaoCopiarQuantProdVinc = Metadata.getInstance<Button>('botaoCopiarQuantProdVinc');
      botaoAddProdVinculado.disabled = true;
      botaoExcluirProdVinculado.disabled = false;
      // botaoCopiarQuantProdVinc.disabled = true;
    }
    // Grid Produto
    if (this.estadoGridProduto) {
      const botaoAddProduto = Metadata.getInstance<Button>('botaoAdicionaProduto');
      const botaoExcluirProduto = Metadata.getInstance<Button>('botaoExcluirProduto');
      const botaoAtualizarEstoqueProduto = Metadata.getInstance<Button>('botaoatualizaEstoqueProd');
      botaoAddProduto.disabled = true;
      botaoExcluirProduto.disabled = false;
      botaoAtualizarEstoqueProduto.disabled = false;
    }
    if (this.estadoGridTipoPrato || this.estadoGridProduto) {
      DialogService.show({
        name: 'error-dialog',
        type: 'warning',
        title: 'plaNext::aviso',
        text: 'plaNext::retiradaBloqIncl',
      });
    }
    this.bloqAcoesAdicionar = true;
  }

  // Bloqueio referente a todas as ações dentro da retirada
  public bloqueiaTodasAcoes() {
    if (this.estadoGridTipoPrato) {
      // Grid Tipo Prato
      const botaoAddTipoPrato = Metadata.getInstance<Button>('botaoAdicionaTipoPrato');
      const botaoExcluirTipoPrato = Metadata.getInstance<Button>('botaoExcluirTipoPrato');
      const botaoAtualizarEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
      botaoAddTipoPrato.disabled = true;
      botaoExcluirTipoPrato.disabled = true;
      botaoAtualizarEstoqueTP.disabled = true;
      // Grid Produto Vinculado
      const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
      const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
      // const botaoCopiarQuantProdVinc = Metadata.getInstance<Button>('botaoCopiarQuantProdVinc');
      botaoAddProdVinculado.disabled = true;
      botaoExcluirProdVinculado.disabled = true;
      // botaoCopiarQuantProdVinc.disabled = true;
    }
    // Grid Produto
    if (this.estadoGridProduto) {
      const botaoAddProduto = Metadata.getInstance<Button>('botaoAdicionaProduto');
      const botaoExcluirProduto = Metadata.getInstance<Button>('botaoExcluirProduto');
      const botaoAtualizarEstoqueProduto = Metadata.getInstance<Button>('botaoatualizaEstoqueProd');
      botaoAddProduto.disabled = true;
      botaoExcluirProduto.disabled = true;
      botaoAtualizarEstoqueProduto.disabled = true;
    }
    if (this.estadoGridTipoPrato || this.estadoGridProduto) {
      DialogService.show({
        name: 'error-dialog',
        type: 'warning',
        title: 'plaNext::aviso',
        text: 'plaNext::retiradaBloqAlteracao',
      });
    }
    this.bloqTodasAcoes = true;
  }

  // Volta as configurações default de cada retirada se não houver bloqueios
  public visualizacaoDefaultAcoes() {
    const tabs = Metadata.getInstance<ZdTabs>('tabsRetiradaPlanejamento');
    tabs.tabs.forEach((tab) => {
      if (tab.name === 'tabResumo') {
        tab.disabled = false;
      }
    });
    if (this.estadoGridTipoPrato) {
      // Grid Tipo Prato
      const botaoAddTipoPrato = Metadata.getInstance<Button>('botaoAdicionaTipoPrato');
      const botaoExcluirTipoPrato = Metadata.getInstance<Button>('botaoExcluirTipoPrato');
      const botaoAtualizarEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
      botaoAddTipoPrato.disabled = false;
      botaoExcluirTipoPrato.disabled = true;
      botaoAtualizarEstoqueTP.disabled = false;
      // Grid Produto Vinculado
      const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
      const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
      // const botaoCopiarQuantProdVinc = Metadata.getInstance<Button>('botaoCopiarQuantProdVinc');
      botaoAddProdVinculado.disabled = true;
      botaoExcluirProdVinculado.disabled = true;
      // botaoCopiarQuantProdVinc.disabled = true;
    }
    // Grid Produto
    if (this.estadoGridProduto) {
      const botaoAddProduto = Metadata.getInstance<Button>('botaoAdicionaProduto');
      const botaoExcluirProduto = Metadata.getInstance<Button>('botaoExcluirProduto');
      const botaoAtualizarEstoqueProduto = Metadata.getInstance<Button>('botaoatualizaEstoqueProd');
      botaoAddProduto.disabled = false;
      botaoExcluirProduto.disabled = true;
      botaoAtualizarEstoqueProduto.disabled = false;
    }
    this.bloqTodasAcoes = false;
    this.bloqAcoesAdicionar = false;
  }

  public atualizaCustoTotal(cdfilial, nrretirada, nrcardapio) {
    const dados = {
      CDFILIAL: cdfilial,
      NRRETIRADA: nrretirada,
      NRCARDAPIO: nrcardapio,
    };
    Http.post('/atualizaCustoTotal', dados).then((response) => {
      const campoCustoTotal = Metadata.getInstance<Number>('campoCustoTotal');
      const valorCustoTotal = response.data.dataset.custototal[0];
      campoCustoTotal.value = valorCustoTotal;
    });
  }

  // Função responsável por carregar os dados do grid de Tipo de Prato
  public async carregaGridTipoPrato(cdfilial?, nrretirada?, nrcardapio?, retiradaPorRequisicao?, dtretirada?, cdservfili?) {
    const gridRomaneio = Metadata.getInstance<Grid>('retiradaPlanejamentoGrid');
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    if (!gridTipoPrato.editing && !retiradaPorRequisicao) {
      if (Object.keys(gridRomaneio.datasource.currentRow).length > 0) {
        const dtretiradaRomaneio = gridRomaneio.datasource.currentRow.DTLANCESTQ;
        const cdfilialRomaneio = gridRomaneio.datasource.currentRow.CDFILIAL;
        const cdservfiliRomaneio = gridRomaneio.datasource.currentRow.CDSERVFILI;
        const nrretiradaRomaneio = gridRomaneio.datasource.currentRow.NRRETIRADA;
        const nrcardapioRomaneio = gridRomaneio.datasource.currentRow.NRCARDAPIO;
        const conditionsRomaneio = {
          CDFILIAL: cdfilialRomaneio,
          NRRETIRADA: nrretiradaRomaneio,
          NRCARDAPIO: nrcardapioRomaneio,
        };
        gridTipoPrato.datasource.filter = conditionsRomaneio;
        this.verificaRetiradaBloqueadaAlteracao(dtretiradaRomaneio, cdfilialRomaneio, cdservfiliRomaneio);
        this.atualizaCustoTotal(cdfilialRomaneio, nrretiradaRomaneio, nrcardapioRomaneio);
      }
    } else if (gridTipoPrato.editing || retiradaPorRequisicao) {
      const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
      const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
      cardTipoPrato.isVisible = true;
      cardProdVinc.isVisible = true;
      const conditionsNovaRetirada = {
        CDFILIAL: cdfilial,
        NRRETIRADA: nrretirada,
        NRCARDAPIO: nrcardapio,
      };
      gridTipoPrato.datasource.filter = conditionsNovaRetirada;
      await gridTipoPrato.cancelEditedRows();
      gridTipoPrato.editingNewRows = false;
      if (retiradaPorRequisicao) {
        const title = 'plaNext::retiradaPlanejamento';
        const text = 'plaNext::retiradaPorRequisicao';
        DialogService.show({ name: 'basic-dialog', title, text });
      }
      this.verificaRetiradaBloqueadaAlteracao(dtretirada, cdfilial, cdservfili);
      this.atualizaCustoTotal(cdfilial, nrretirada, nrcardapio);
    }
    this.estadoGridTipoPrato = true;
    await gridTipoPrato.datasource.updateData([]);
    await gridTipoPrato.datasource.get();
    gridTipoPrato.columns.forEach((field) => {
      field.editable = false;
      field.isVisible = true;
    });
    const card = gridTipoPrato.parent as Card;
    const botaoatualizaEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
    if (gridTipoPrato.datasource.data.length > 0 && card.isVisible && !this.bloqTodasAcoes) {
      botaoatualizaEstoqueTP.disabled = false;
    } else if (gridTipoPrato.datasource.data.length === 0 && card.isVisible) {
      botaoatualizaEstoqueTP.disabled = true;
    }
    if (this.estadoGridResumo) {
      this.carregaGridResumo();
    }
    // this.estadoBtcopiarQuantidade();
  }

  // Função responsável por carregar os dados do grid de Produto Vinculado a algum Tipo de Prato
  public async carregaGridProdutoVinculado() {
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    const botaoExcluirTipoPrato = Metadata.getInstance<Button>('botaoExcluirTipoPrato');
    const botaoAdicionarProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
    const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
    if (!gridTipoPrato.editing) {
      if (Object.keys(gridTipoPrato.datasource.currentRow).length > 0) {
        const cdfilial = gridTipoPrato.datasource.currentRow.CDFILIAL;
        const nrretirada = gridTipoPrato.datasource.currentRow.NRRETIRADA;
        const nrcardapio = gridTipoPrato.datasource.currentRow.NRCARDAPIO;
        const conditions = {
          CDFILIAL: cdfilial,
          NRRETIRADA: nrretirada,
          CDPRATO: gridTipoPrato.datasource.currentRow.CDPRATO,
          NRORG: gridTipoPrato.datasource.currentRow.NRORG,
          DTRETIRADA: gridTipoPrato.datasource.currentRow.DTRETIRADA,
          NRCARDAPIO: nrcardapio,
          CDTIPOPRAT: gridTipoPrato.datasource.currentRow.CDTIPOPRAT,
          CDFILAUXPRAT: gridTipoPrato.datasource.currentRow.CDFILAUXPRAT,
        };
        gridProdVinc.datasource.filter = conditions;
        await gridProdVinc.datasource.updateData([]);
        await gridProdVinc.datasource.get();
        if (!this.bloqTodasAcoes) {
          botaoExcluirTipoPrato.disabled = false;
        }
        botaoExcluirProdVinculado.disabled = true;
        gridProdVinc.cancelEditedRows();
        gridProdVinc.editingNewRows = false;
        gridProdVinc.columns.forEach((field) => {
          if (field.name === 'CDPRODUTO') {
            field.editable = false;
          } else if (
            field.name === 'QTEXTRARETI'
            || field.name === 'QTDEVOLVRETI'
            || field.name === 'QTRETIRADA'
            || field.name === 'CDTIPORETI'
          ) {
            field.editable = true;
          }
          field.isVisible = true;
        });
        if (this.estadoGridResumo) {
          this.carregaGridResumo();
        }
        this.atualizaCustoTotal(cdfilial, nrretirada, nrcardapio);
        if (!this.bloqTodasAcoes && !this.bloqAcoesAdicionar) {
          botaoAdicionarProdVinculado.disabled = false;
        }
      }
    }
  }

  // Função responsável por carregar os dados do grid de Resumo
  public async carregaGridResumo() {
    const gridRomaneio = Metadata.getInstance<Grid>('retiradaPlanejamentoGrid');
    const gridResumo = Metadata.getInstance<Grid>('resumoGrid');
    this.estadoGridResumo = true;
    if (Object.keys(gridRomaneio.datasource.currentRow).length > 0) {
      const conditionsRomaneio = {
        CDFILIAL: gridRomaneio.datasource.currentRow.CDFILIAL,
        NRRETIRADA: gridRomaneio.datasource.currentRow.NRRETIRADA,
        DTPOSESTCON: gridRomaneio.datasource.currentRow.DTLANCESTQ,
      };
      gridResumo.datasource.filter = conditionsRomaneio;
    } else {
      const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
      const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
      const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
      let dtretirada;
      let cdfilial;
      let nrretirada;
      if (cardTipoPrato.isVisible === true) {
        dtretirada = gridTipoPrato.datasource.data[0].DTRETIRADA;
        cdfilial = gridTipoPrato.datasource.data[0].CDFILIAL;
        nrretirada = gridTipoPrato.datasource.data[0].NRRETIRADA;
      } else {
        dtretirada = gridProduto.datasource.data[0].DTRETIRADA;
        cdfilial = gridProduto.datasource.data[0].CDFILIAL;
        nrretirada = gridProduto.datasource.data[0].NRRETIRADA;
      }
      const conditionsNovaRetiradaTP = {
        CDFILIAL: cdfilial,
        NRRETIRADA: nrretirada,
        DTPOSESTCON: dtretirada,
      };
      gridResumo.datasource.filter = conditionsNovaRetiradaTP;
    }
    await gridResumo.datasource.get();
  }

  // Função responsável por carregar os dados do grid de Produto
  public async carregaGridProduto(cdfilial?, nrretirada?, dtretirada?, nrcardapio?, retiradaPorRequisicao?, cdservfili?) {
    const gridRomaneio = Metadata.getInstance<Grid>('retiradaPlanejamentoGrid');
    const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
    const nrorg = localStorage.getItem('NRORG');
    if (!gridProduto.editing && !retiradaPorRequisicao) {
      if (Object.keys(gridRomaneio.datasource.currentRow).length > 0) {
        const dtretiradaRomaneio = gridRomaneio.datasource.currentRow.DTLANCESTQ;
        const cdfilialRomaneio = gridRomaneio.datasource.currentRow.CDFILIAL;
        const cdservfiliRomaneio = gridRomaneio.datasource.currentRow.CDSERVFILI;
        const nrretiradaRomaneio = gridRomaneio.datasource.currentRow.NRRETIRADA;
        const nrcardapioRomaneio = gridRomaneio.datasource.currentRow.NRCARDAPIO;
        const conditionsRomaneio = {
          CDFILIAL: cdfilialRomaneio,
          NRRETIRADA: nrretiradaRomaneio,
          NRORG: gridRomaneio.datasource.currentRow.NRORG,
          DTRETIRADA: dtretiradaRomaneio,
          NRCARDAPIO: nrcardapioRomaneio,
        };
        gridProduto.datasource.filter = conditionsRomaneio;
        this.verificaRetiradaBloqueadaAlteracao(dtretiradaRomaneio, cdfilialRomaneio, cdservfiliRomaneio);
        this.atualizaCustoTotal(cdfilialRomaneio, nrretiradaRomaneio, nrcardapioRomaneio);
      }
    } else if (gridProduto.editing || retiradaPorRequisicao) {
      const cardProduto = Metadata.getInstance<Card>('lancamentoProduto');
      cardProduto.isVisible = true;
      const conditionsNovaRetirada = {
        CDFILIAL: cdfilial,
        NRRETIRADA: nrretirada,
        NRORG: nrorg,
        DTRETIRADA: dtretirada,
        NRCARDAPIO: nrcardapio,
      };
      gridProduto.datasource.filter = conditionsNovaRetirada;
      await gridProduto.cancelEditedRows();
      this.verificaRetiradaBloqueadaAlteracao(dtretirada, cdfilial, cdservfili);
      this.atualizaCustoTotal(cdfilial, nrretirada, nrcardapio);
      if (retiradaPorRequisicao) {
        const title = 'plaNext::retiradaPlanejamento';
        const text = 'plaNext::retiradaPorRequisicao';
        DialogService.show({ name: 'basic-dialog', title, text });
      }
    }
    this.estadoGridProduto = true;
    await gridProduto.cancelEditedRows();
    await gridProduto.datasource.updateData([]);
    await gridProduto.datasource.get();
    gridProduto.editingNewRows = false;
    gridProduto.columns.forEach((field) => {
      if (field.name === 'CDPRODUTO') {
        field.editable = false;
      } else if (
        field.name === 'QTEXTRARETI'
        || field.name === 'QTDEVOLVRETI'
        || field.name === 'QTRETIRADA'
        || field.name === 'CDTIPORETI'
      ) {
        field.editable = true;
      }
      field.isVisible = true;
    });
    const botaoatualizaEstoqueProd = Metadata.getInstance<Button>('botaoatualizaEstoqueProd');
    const card = gridProduto.parent as Card;
    if (gridProduto.datasource.data.length > 0 && card.isVisible && !this.bloqTodasAcoes) {
      botaoatualizaEstoqueProd.disabled = false;
    } else if (gridProduto.datasource.data.length === 0 && card.isVisible) {
      botaoatualizaEstoqueProd.disabled = true;
    }
    if (this.estadoGridResumo) {
      this.carregaGridResumo();
    }
    // this.estadoBtcopiarQuantidade();
  }

  // Função responsável por carregar o campo Nº da Requisição do form de uma nova retirada
  public async carregaSelectRetirada({ component }: IEventParam<Select>) {
    const formFiltro = Metadata.getInstance<Form>('retiradaPlanejamentoForm');
    const formLancamento = Metadata.getInstance<Form>('lancamentoForm');
    const conditions = {
      CDFILIAL: formFiltro.value.FILIAL_FILTRO,
      DTPREVRETI: formLancamento.value.dtRetiradaAddReq,
      CDSERVFILI: formLancamento.value.servicoAddRequisicao,
    };
    component.datasource.filter = conditions;
    await component.datasource.get();
  }

  public gridProdutoAntesDaEdicao({ component }: IEventParam<GridEditable>) {
    this.isDisabledButtonCardProd = !component.editing;
  }

  // Cancela a inclusão de um novo Tipo de Prato
  public async cancelaNovoTipoPrato() {
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    const buttonSalvaNovoTP = Metadata.getInstance<Button>('botaoSalvarTipoPrato');
    const buttonCancelaNovoTP = Metadata.getInstance<Button>('botaoCancelaTipoPrato');
    const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
    const botaoatualizaEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
    const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
    const botaoAddTipoPrato = Metadata.getInstance<Button>('botaoAdicionaTipoPrato');
    cardTipoPrato.disabled = false;
    botaoatualizaEstoqueTP.disabled = false;
    gridTipoPrato.columns.forEach((field) => {
      field.editable = false;
      field.isVisible = true;
    });
    gridProdVinc.columns.forEach((field) => {
      if (field.name === 'CDPRODUTO') {
        field.editable = false;
      } else if (
        field.name === 'QTEXTRARETI'
        || field.name === 'QTDEVOLVRETI'
        || field.name === 'QTRETIRADA'
        || field.name === 'CDTIPORETI'
      ) {
        field.editable = true;
      }
      field.isVisible = true;
    });
    buttonSalvaNovoTP.isVisible = false;
    buttonCancelaNovoTP.isVisible = false;
    botaoAddTipoPrato.disabled = false;
    this.isDisabledButtonCardProd = true;
    botaoAddProdVinculado.disabled = true;
    await gridTipoPrato.cancelEditedRows();
    await gridProdVinc.cancelEditedRows();
    gridTipoPrato.editingNewRows = false;
    gridProdVinc.editingNewRows = false;
    await gridTipoPrato.datasource.get();
    gridProdVinc.datasource.updateData([]);
    if (gridTipoPrato.datasource.data.length === 0) {
      const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
      const formNovaRetirada = Metadata.getInstance<Form>('lancamentoForm');
      const legendaLancamento = Metadata.getInstance<Row>('dadosGridRomaneio');
      botaoatualizaEstoqueTP.disabled = true;
      legendaLancamento.isVisible = false;
      formNovaRetirada.isVisible = true;
      cardProdVinc.isVisible = false;
      cardTipoPrato.isVisible = false;
    }
  }

  // Cancela a inclusão de um novo Produto
  public async cancelaNovoProduto() {
    const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
    const botaoatualizaEstoqueTP = Metadata.getInstance<Button>('botaoatualizaEstoqueTP');
    const botaoAddProduto = Metadata.getInstance<Button>('botaoAdicionaProduto');
    botaoatualizaEstoqueTP.disabled = false;
    gridProduto.columns.forEach((field) => {
      if (field.name === 'CDPRODUTO') {
        field.editable = false;
      } else if (
        field.name === 'QTEXTRARETI'
        || field.name === 'QTDEVOLVRETI'
        || field.name === 'QTRETIRADA'
        || field.name === 'CDTIPORETI'
      ) {
        field.editable = true;
      }
      field.isVisible = true;
    });
    botaoAddProduto.disabled = false;
    this.isDisabledButtonCardProd = true;
    await gridProduto.cancelEditedRows();
    gridProduto.editingNewRows = false;
    await gridProduto.datasource.get();
    if (gridProduto.datasource.data.length === 0) {
      const cardProduto = Metadata.getInstance<Card>('lancamentoProduto');
      const formNovaRetirada = Metadata.getInstance<Form>('lancamentoForm');
      const legendaLancamento = Metadata.getInstance<Row>('dadosGridRomaneio');
      botaoatualizaEstoqueTP.disabled = true;
      legendaLancamento.isVisible = false;
      formNovaRetirada.isVisible = true;
      cardProduto.isVisible = false;
    }
  }

  // Cancela edição de um Produto existente, se for um novo produto direciona para a função responsável
  public async cancelaEdicaoProduto({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    if (grid.editingNewRows === false) {
      await grid.cancelEditedRows();
      grid.editingNewRows = false;
      this.gridProdutoAntesDaEdicao({ component: grid });
    } else if (grid.name === 'lancamentoProdutoVinculadoGrid') {
      this.cancelaNovoTipoPrato();
    } else if (grid.name === 'lancamentoProdutoGrid') {
      this.cancelaNovoProduto();
    }
  }

  // Salva edição in line de um produto existente, se for um novo produto direciona para a função responsável
  public async salvaEdicaoProduto({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const cardGrid = component.parent as Card;
    const cardPrincipal = cardGrid.parent as Card;
    const cardTipoPrato = Metadata.getInstance<Card>('lancamentoTipoDePrato');
    if (grid.editingNewRows === false) {
      cardPrincipal.disabled = true;
      const editRows = grid.getEditedRows();
      const validarEstoque = this.paramUnid.VALIDAR_ESTOQUE;
      const exibeSaldoEstreti = this.paramUnid.IDEXIBESALDOESTRETI;
      const dtretirada = grid.datasource.data[0].DTRETIRADA;
      const nrlancestq = grid.datasource.data[0].NRLANCESTQ;
      editRows.forEach((prod, index) => {
        editRows[index].VALIDAR_ESTOQUE = validarEstoque;
        editRows[index].DTRETIRADA = dtretirada;
        editRows[index].IDEXIBESALDOESTRETI = exibeSaldoEstreti;
        editRows[index].NRLANCESTQ = nrlancestq;
      });
      Http.post('/salvaEditLineProduto', {
        requestType: 'DataSet',
        dataset: editRows,
      }).then(async (response) => {
        if (response && response.data && response.data.error) {
          DialogService.show({
            name: 'error-dialog',
            type: 'error',
            title: 'plaNext::erroEditarProduto',
            text: response.data.error,
          });
          cardPrincipal.disabled = false;
        } else {
          await grid.cancelEditedRows();
          grid.editingNewRows = false;
          cardPrincipal.disabled = false;
          this.gridProdutoAntesDaEdicao({ component: grid });
          const cdfilial = grid.datasource.data[0].CDFILIAL;
          const nrretirada = grid.datasource.data[0].NRRETIRADA;
          const nrcardapio = grid.datasource.data[0].NRCARDAPIO;
          this.atualizaCustoTotal(cdfilial, nrretirada, nrcardapio);
          if (cardPrincipal.name === 'lancamentoProdutoVinculado') {
            cardTipoPrato.disabled = false;
          }
          if (this.estadoGridResumo) {
            this.carregaGridResumo();
          }
        }
      }, () => {
        cardPrincipal.disabled = false;
        cardTipoPrato.disabled = false;
      });
    } else if (grid.name === 'lancamentoProdutoVinculadoGrid') {
      this.salvaGridProdutoVinculadoNovaRetirada();
    } else if (grid.name === 'lancamentoProdutoGrid') {
      this.salvaGridProdutoNovaRetirada();
    }
  }

  public async gerenciaTabs() {
    const tabs = Metadata.getInstance<ZdTabs>('tabsRetiradaPlanejamento');
    const cardFiltroInicial = Metadata.getInstance<Card>('retiradaPlanejamentoCard');
    if (tabs.activeTab === 1 || tabs.activeTab === 2) {
      cardFiltroInicial.disabled = true;
    } else {
      cardFiltroInicial.disabled = false;
    }
  }

  public async selecionaProdutoVinculado() {
    const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    if (Object.keys(gridProdVinc.datasource.currentRow).length > 0 && !gridProdVinc.editing && !this.bloqTodasAcoes) {
      botaoExcluirProdVinculado.disabled = false;
    } else {
      botaoExcluirProdVinculado.disabled = true;
    }
  }

  public async selecionaProdutoGrid() {
    const botaoExcluirProduto = Metadata.getInstance<Button>('botaoExcluirProduto');
    const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
    if (Object.keys(gridProduto.datasource.currentRow).length > 0 && !gridProduto.editing && !this.bloqTodasAcoes) {
      botaoExcluirProduto.disabled = false;
    } else {
      botaoExcluirProduto.disabled = true;
    }
  }

  private cancelaModalDialogservice: any = {
    click: () => DialogService.hide(),
  };

  private confirmaExcluirTipoPrato: any = {
    click: () => this.excluirTipoPrato(),
  };

  public async confirmaExclusaoTipoPrato() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'plaNext::excluirTipoPrato',
      text: 'plaNext::solicitaExclusaoTipoPrato',
      buttons: [
        {
          name: 'no', component: 'ZdButton', label: 'Não', events: this.cancelaModalDialogservice, outline: true,
        },
        {
          name: 'yes', component: 'ZdButton', label: 'Sim', events: this.confirmaExcluirTipoPrato,
        },
      ],
    });
  }

  public async excluirTipoPrato() {
    const gridTipoPrato = Metadata.getInstance<GridEditable>('lancamentoTipoDePratoGrid');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    const botaoExcluirTipoPrato = Metadata.getInstance<Button>('botaoExcluirTipoPrato');
    const botaoAddProdVinculado = Metadata.getInstance<Button>('botaoAddProdVinculado');
    const cdfilial = gridTipoPrato.datasource.currentRow.CDFILIAL;
    const nrretirada = gridTipoPrato.datasource.currentRow.NRRETIRADA;
    const nrcardapio = gridTipoPrato.datasource.currentRow.NRCARDAPIO;
    const dadosGridTipoPrato = [gridTipoPrato.datasource.currentRow];
    if (Object.keys(dadosGridTipoPrato).length > 0) {
      Http.post('/excluirTipoPrato', {
        requestType: 'DataSet',
        dataset: dadosGridTipoPrato,
      }).then((response) => {
        if (response && response.data && response.data.error) {
          DialogService.show({
            name: 'error-dialog',
            type: 'error',
            title: 'plaNext::erroExcluirTipoPrato',
            text: response.data.error,
          });
        } else {
          botaoAddProdVinculado.disabled = true;
          gridTipoPrato.datasource.get();
          if (this.estadoGridResumo) {
            this.carregaGridResumo();
          }
          this.atualizaCustoTotal(cdfilial, nrretirada, nrcardapio);
        }
      });
      botaoExcluirTipoPrato.disabled = true;
      gridProdVinc.datasource.clearFilter();
    }
    DialogService.hide();
  }

  private deleteProduto: any = {
    click: () => this.excluirProduto(),
  };

  public async confirmaExclusaoProduto() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'plaNext::excluirProduto',
      text: 'plaNext::solicitaExclusaoProduto',
      buttons: [
        {
          name: 'no', component: 'ZdButton', label: 'Não', events: this.cancelaModalDialogservice, outline: true,
        },
        {
          name: 'yes', component: 'ZdButton', label: 'Sim', events: this.deleteProduto,
        },
      ],
    });
  }

  public async excluirProduto() {
    const botaoExcluirProduto = Metadata.getInstance<Button>('botaoExcluirProduto');
    const gridProduto = Metadata.getInstance<GridEditable>('lancamentoProdutoGrid');
    const dadosGridProduto = [gridProduto.datasource.currentRow];
    if (Object.keys(dadosGridProduto).length > 0) {
      Http.post('/excluirProduto', {
        requestType: 'DataSet',
        dataset: dadosGridProduto,
      }).then(() => {
        this.carregaGridProduto();
        if (this.estadoGridResumo) {
          this.carregaGridResumo();
        }
      });
      botaoExcluirProduto.disabled = true;
    }
    DialogService.hide();
  }

  private deleteProdutoVinculado: any = {
    click: () => this.excluirProdutoVinculado(),
  };

  public async confirmaExclusaoProdutoVinculado() {
    DialogService.show({
      name: 'buttons-dialog',
      type: 'warning',
      title: 'plaNext::excluirProduto',
      text: 'plaNext::solicitaExclusaoProduto',
      buttons: [
        {
          name: 'no', component: 'ZdButton', label: 'Não', events: this.cancelaModalDialogservice, outline: true,
        },
        {
          name: 'yes', component: 'ZdButton', label: 'Sim', events: this.deleteProdutoVinculado,
        },
      ],
    });
  }

  public async excluirProdutoVinculado() {
    const botaoExcluirProdVinculado = Metadata.getInstance<Button>('botaoExcluirProdVinculado');
    const gridProdVinc = Metadata.getInstance<GridEditable>('lancamentoProdutoVinculadoGrid');
    const dadosGridProdutoVinculado = [gridProdVinc.datasource.currentRow];
    if (Object.keys(dadosGridProdutoVinculado).length > 0) {
      Http.post('/excluirProduto', {
        requestType: 'DataSet',
        dataset: dadosGridProdutoVinculado,
      }).then(() => {
        this.carregaGridProdutoVinculado();
      });
      botaoExcluirProdVinculado.disabled = true;
    }
    DialogService.hide();
  }

  public async estadoBtcopiarQuantidade() {
    /* const botaoCopiarQuantProdVinc = Metadata.getInstance<Button>('botaoCopiarQuantProdVinc');
    const dataAtual = new Date();
    const validaDatas = dataAtual > this.paramUnid.DIAS_ANTES_DTPROCESSA && dataAtual < this.paramUnid.DIAS_APOS_DTPROCESSA;
    if (this.paramUnid.IDUTILALMOX === 'N' && this.paramUnid.IDUTILLOTE === 'N' && validaDatas && !this.bloqTodasAcoes) {
      botaoCopiarQuantProdVinc.disabled = false;
    } else {
      botaoCopiarQuantProdVinc.disabled = true;
    } */
  }

  public configuraAlert(nameAlert, textAlert, colorAlert) {
    return {
      name: nameAlert,
      text: textAlert,
      color: colorAlert,
      top: true,
      left: false,
      right: true,
      timeout: 3000,
      buttonsSlot: [
        {
          name: 'undo-button',
          component: 'ZdButton',
          icon: true,
          iconName: 'close',
          color: 'white',
          events: {
            click: () => { AlertService.hide(); },
          },
        },
      ],
    };
  }

  public async atualizarEstoque({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const nrretirada = grid.datasource.data[0].NRRETIRADA;
    const nrcardapio = grid.datasource.data[0].NRCARDAPIO;
    const nrlancestq = grid.datasource.data[0].NRLANCESTQ;
    const cdfilial = grid.datasource.data[0].CDFILIAL;
    const card = grid.parent as Card;
    const cardProdVinc = Metadata.getInstance<Card>('lancamentoProdutoVinculado');
    const conditions = {
      NRRETIRADA: nrretirada,
      NRCARDAPIO: nrcardapio,
      NRLANCESTQ: nrlancestq,
      CDFILIAL: cdfilial,
    };
    card.disabled = true;
    if (grid.name === 'lancamentoTipoDePratoGrid') {
      cardProdVinc.disabled = true;
    }
    Http.post('/atualizarEstoque', conditions).then(() => {
      if (grid.name === 'lancamentoTipoDePratoGrid') {
        cardProdVinc.disabled = false;
      }
      card.disabled = false;
      AlertService.show(this.configuraAlert('Atualizar Estoque', 'Estoque Atualizado com sucesso !', 'success'));
    }, () => {
      cardProdVinc.disabled = false;
      card.disabled = false;
    });
  }

  public retornaMaskCodProduto() {
    return '#.#.#.##.###.##';
  }

  public retornaMaskCodPrato() {
    return '##.##.##.###';
  }

  /* public async copiarQuantidade({ component }: IEventParam<Button>) {
    const grid = component.parent as GridEditable;
    const copiaDatasourceGrid = grid.datasource.clone();
    const prodAlterados = [];
    copiaDatasourceGrid.forEach((prod, index) => {
      if (prod.QTPREVISRETI > 0 && prod.QTEXTRARETI === 0 && prod.QTDEVOLVRETI === 0 && prod.QTRETIRADA === 0) {
        copiaDatasourceGrid[index].QTRETIRADA = prod.QTPREVISRETI;
        prodAlterados.push(copiaDatasourceGrid[index]);
      }
    });
    if (prodAlterados.length > 0) {
      Http.post('/updateProdCopiarQuantidade', {
        requestType: 'DataSet',
        dataset: prodAlterados,
      });
    }
  } */
}
